/*
*  Copyright 2022 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#ifndef GLOBALFUNC_H
#define GLOBALFUNC_H
#include <fstream>
#include <iostream>
#include <sys/stat.h>
#include <sys/types.h>
#include<cstdlib>
#include<unistd.h>
#include<sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <dirent.h>
#include <ctime>
#include<cstring>
#include<vector>
#include<set>
#include<map>
#include<assert.h>
#include <algorithm>
#include "dataTypeDef.h"
using namespace std;
typedef struct addLiteralEquality{
    string literal;
    int suffix;
}literalAdd;
typedef struct addfuncEquality{
    string func;
    int suffix;
}funcAdd;

inline void TrimStr(string& s)
{
    if (s.empty())
        return;
    s.erase(0, s.find_first_not_of(" "));
    s.erase(s.find_last_not_of(" ") + 1);
}

inline UINT32 BKDRHash(UINT32 hash, int termCode) {
    hash = hash * 31 + termCode;
    return hash & 0x7FFFFFFF; 
}

inline int FindSubStr(string& str, string& subStr)
{
    int pos = -1;
    size_t strLen = str.size(), subStrLen = subStr.size();
    if (strLen < subStrLen)
        return -1;
    while (pos++<strLen) {
        bool flag = false;
        if (str[pos] == subStr[0]) {
            flag = true;
            for (size_t k = 1; k < subStrLen; k++) {
                if (str[pos + k] != subStr[k]) {
                    flag = false;
                    break;
                }
            }
        }
        if (flag && (str[pos + subStrLen] == ')' || str[pos + subStrLen] == ',')) {
            return pos;
        }
    }
    return -1;
}

inline vector<string> Split_c(const string& s, const char delim)
{
    size_t last = 0;
    size_t index = s.find_first_of(delim, last);
    vector<string> ret;
    while (index != string::npos) {
        ret.push_back(s.substr(last, index - last));
        last = index + 1;
        index = s.find_first_of(delim, last);
    }
    if (index - last > 0)
        ret.push_back(s.substr(last, index - last));
    return ret;
}

inline vector<string> Split_s(const string& s, const string& delim)
{
    size_t last = 0;
    size_t index = s.find_first_of(delim, last);
    vector<string> ret;
    while (index != string::npos) {
        ret.push_back(s.substr(last, index - last));
        last = index + 1;
        index = s.find_first_of(delim, last);
    }
    if (index - last > 0)
        ret.push_back(s.substr(last, index - last));
    return ret;
}

inline string GetPred(const string& pred, size_t& i)
{
    size_t predSize = pred.size();
    assert(predSize != 0);		
    i = pred.find_first_of('(');
    assert(i != 0);
    assert(i != string::npos);
    return pred.substr(0, i);
}

inline void SplitSymbol(const string &pred, string &symbol, string &strContent)
{
    size_t pos = 0;
    symbol = GetPred(pred, pos);
    strContent = pred.substr(pos + 1, pred.length() - pos - 2); 
}

inline bool IsBaseLits(vector<string>& vectStrLits)
{
    bool isBase = true;
    for (string &str : vectStrLits) {
        if (str.find('x') != string::npos) {
            isBase = false;
            break;
        }
    }
    return isBase;
}

#endif /* GLOBALFUNC_H */

