/*
*  Copyright 2021 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#include "DataStructure.h"
#include "RedundancyOp.h"
#include "FileOp.h"
#include "Clause.h"
#include "Literal.h"
#include "Term.h"
#include "Formula.h"
#include "StrategyParam.h"
#include "Subsumption.h"
#include "Unify.h"
#include "SortRule.h"
#include "dataTypeDef.h"
#include"globalFunc.h"
#include "TriAlg.h"
extern set<HashNode*, HashNodeCmpByCode> g_HashTree;

Formula::Formula(const Formula& orig) {
}

Formula::~Formula() {
}

RESULT Formula::ReadFOLFile()
{
        switch (FileOp::fileType) {
        case 0:
            return ReadTPTPFile();
        case 1: break;
        case 2:
            return ReadMyDefFile();;
        default: break;
    }
    return UnknownFile;
}

RESULT Formula::ReadMyDefFile() {
    string strClause;
    uOrigClaSize = 0;
    ifstream ifFolFile(FileOp::JudgeFileFullName);
    bool isFindEqn = false;
    while (getline(ifFolFile, strClause)) {
        if (strClause == "")continue;
        if (strClause.find_first_of('-') != string::npos)
            break;
        ++uOrigClaSize;
        if (!(isalpha(strClause[0]) || strClause[0] == '~'))
            return READERR;
        vector<string>vectLit = Split_c(strClause, P_SPLIT_CAHR); 
        if (vectLit.size() > 1 && Subsumption::CheckTautology(vectLit)) {
            ++uOrigClaSize;
            continue;
        }
        if (!isFindEqn) {
            if (strClause == "E(x1,x1)")
                StrategyParam::g_ClaNumNoAxiom = uOrigClaSize;
            else {
                if (StrategyParam::g_ClaNumNoAxiom < uOrigClaSize && strClause == "~E(x1,x2)+E(x2,x1)") {
                    isFindEqn = true;
                    set<Clause*>::iterator it = setCla.end();
                    --it; 
                    setEnq.insert(*it);
                    setCla.erase(it);
                } else
                    StrategyParam::g_ClaNumNoAxiom = MAXINT32;
            }
        }
        vector<string>vectsLit = Split_c(strClause, P_SPLIT_CAHR);
        Clause *c1 = new Clause(vectsLit, uOrigClaSize);
        c1->SortLit();
        if (isFindEqn)
            setEnq.insert(c1);
        else
            setCla.insert(c1);
        string sCla;
        c1->toString(sCla);
        FileOp::PrintINFO(sCla);
    }
    return READOK;
}

RESULT Formula::ReadTPTPFile()
{
    string strClause;
    uOrigClaSize = 1;
    ifstream ifFolFile(FileOp::JudgeFileFullName);
    string strINFO;
    int iBracketNum = 0;
    bool isAxiom = false;
    bool isGoal = false;
    while (getline(ifFolFile, strClause)) {
        TrimStr(strClause);
        if (strClause.length() == 0 || strClause[0] == '%') continue; 
        vector<string>vectLit; 
        if (strClause[0] == 'c') 
        {
            int index = strClause.find("clause", 3);
            int indexGoal = strClause.find("negated_conjecture", 3);
            isAxiom = (index == -1);
            isGoal = (indexGoal != -1);
            iBracketNum = 1;
            continue;
        }
        if (iBracketNum > 0) {
            bool isEqnLit = false;
            bool isFinish = false;
            string strLit;
            for (size_t i = 0; i < strClause.length(); ++i) {
                if (isFinish)break;
                if (strClause[i] == ' ')continue; 
                if (strClause[i] == '(') {
                    if (iBracketNum > 1) strLit += '(';
                    ++iBracketNum;
                    while (strClause[++i] == ' ') {
                    }
                    if (strClause[i] >= '0' && strClause[i] <= '9') {
                        strLit += "a" + to_string(200 + strClause[i] - '0' + 1);
                        i++;
                    }
                }
                while (strClause[i] == ',') {
                    strLit += strClause[i];
                    while (strClause[++i] == ' ') {
                    }
                    if (strClause[i] >= '0' && strClause[i] <= '9') {
                        strLit += "a" + to_string(200 + strClause[i] - '0' + 1);
                        i++;
                    }
                }
                if (strClause[i] == ')')--iBracketNum;
                if (strClause[i] == '|' || iBracketNum == 1) {
                    if (isEqnLit) {
                        strLit += ')';
                        isEqnLit = false;
                    }
                    if (strLit.find('(') == -1) {
                        strLit += "(a5000)";
                    }
                    int iPosA = strLit.find("$true");
                    int iPosB = strLit.find("~$false");
                    if (iPosA > -1 || iPosB > -1) 
                    {
                        strLit = "";
                        i = strClause.length();
                        continue;
                    }
                    iPosA = strLit.find("~$true");
                    iPosB = strLit.find("$false");
                    if (!(iPosA > -1) && !(iPosB > -1))
                    {
                        vectLit.push_back(strLit);
                    }
                    strLit = "";
                    if (strClause[i] == '|')continue;
                }
                if (iBracketNum == 1) {
                    isFinish = true;
                    if((vectLit.size()>760) ||(vectLit.size() == 0)){
                        ++uOrigClaSize;
                        break;
                    }
                    if (vectLit.size() > 1 && RedundancyOp::CheckTautology(vectLit)) {
                        ++uOrigClaSize;
                        break;
                    }
                    Clause* cla = new Clause(vectLit, uOrigClaSize); 
                    ++uOrigClaSize;
                    if(cla->uLitNum< 36)
                        cla->SortLit();
                    if (isAxiom)
                    {
                        cla->isAxtom = true;
                        setEnq.insert(cla);
                    }
                    else{
                        if(isGoal)
                            cla->isGoal = true;
                        else
                            cla->isGoal = false;
                        setCla.insert(cla);
                    }
                    string sCla;
                    cla->toString(sCla);                                                         
                    FileOp::PrintINFO(sCla);
                    continue;
                }
                char ch = strClause[i];
                if (ch >= 'A' && ch <= 'Z' && ch != 'X')
                    strLit += ('x' + to_string(ch - 'A' + 1));
                else {
                    switch (ch) {
                        case 'p':ch = 'P';
                            break;
                        case 'X':ch = 'x';
                            break;
                        case 'c':ch = 'a';
                            break;
                        case '=':ch = ',';
                            strLit = "E(" + strLit;
                            isEqnLit = true;
                            break;
                        case '!':
                            if (strClause[++i] == '=') {
                                ch = ',';
                                strLit = "~E(" + strLit;
                                isEqnLit = true;
                            }
                            break;
                        default:
                            break;
                    }
                    strLit += ch;
                }
                if (strClause[i] == '=') {
                    while (strClause[++i] == ' ') {
                    }
                    if (strClause[i] >= '0' && strClause[i] <= '9') {
                        strLit += "a" + to_string(200 + strClause[i] - '0' + 1);
                    } else
                        --i;
                }
            }
        }
    }
    uOrigClaSize--;
    return READOK;
}

RESULT Formula::ReadTPTPFileAgain()
{
    string strClause;
    uOrigClaSizeCom = 1;
    ifstream ifFolFile(FileOp::JudgeFileFullName);
    string strINFO;
    int iBracketNum = 0;
    bool isAxiom = false;
    bool isGoal = false;
    while (getline(ifFolFile, strClause)) {
        TrimStr(strClause); 
        if (strClause.length() == 0 || strClause[0] == '%') continue; 
        vector<string>vectLit; 
        if (strClause[0] == 'c') 
        {
            int index = strClause.find("clause", 3);
            int indexGoal = strClause.find("negated_conjecture", 3);
            isAxiom = (index == -1);
            isGoal = (indexGoal != -1);
            iBracketNum = 1;
            continue;
        }
        if (iBracketNum > 0) {
            bool isEqnLit = false;
            bool isFinish = false;
            string strLit;
            for (size_t i = 0; i < strClause.length(); ++i) {
                if (isFinish)break;
                if (strClause[i] == ' ')continue; 
                if (strClause[i] == '(') {
                    if (iBracketNum > 1) strLit += '(';
                    ++iBracketNum;
                    while (strClause[++i] == ' ') {
                    }
                    if (strClause[i] >= '0' && strClause[i] <= '9') {
                        strLit += "a" + to_string(200 + strClause[i] - '0' + 1);
                        i++;
                    }
                }
                while (strClause[i] == ',') {
                    strLit += strClause[i];
                    while (strClause[++i] == ' ') {
                    }
                    if (strClause[i] >= '0' && strClause[i] <= '9') {
                        strLit += "a" + to_string(200 + strClause[i] - '0' + 1);
                        i++;
                    }
                }
                if (strClause[i] == ')')--iBracketNum;
                if (strClause[i] == '|' || iBracketNum == 1) {
                    if (isEqnLit) {
                        strLit += ')';
                        isEqnLit = false;
                    }
                    if (strLit.find('(') == -1) {
                        strLit += "(a5000)";
                    }
                    int iPosA = strLit.find("$true");
                    int iPosB = strLit.find("~$false");
                    if (iPosA > -1 || iPosB > -1) 
                    {
                        strLit = "";
                        i = strClause.length();
                        continue;
                    }
                    iPosA = strLit.find("~$true");
                    iPosB = strLit.find("$false");

                    if (!(iPosA > -1) && !(iPosB > -1))
                    {
                        vectLit.push_back(strLit);
                    }
                    strLit = "";
                    if (strClause[i] == '|')continue;
                }
                if (iBracketNum == 1) {
                    isFinish = true;
                    if((vectLit.size()>760) ||(vectLit.size() == 0)){
                        ++uOrigClaSizeCom;
                        break;
                    }
                    if (vectLit.size() > 1 && RedundancyOp::CheckTautology(vectLit)) {
                        ++uOrigClaSizeCom;
                        break;
                    }
                    Clause* cla = new Clause(vectLit, uOrigClaSizeCom);
                    ++uOrigClaSizeCom;
                    if(cla->uLitNum< 36)
                        cla->SortLit();
                    if (isAxiom)
                    {
                        cla->isAxtom = true;
                    }
                    else{
                        if(isGoal)
                            cla->isGoal = true;
                        else
                            cla->isGoal = false;
                    }
                    setOriClasCom.insert(cla);
                    continue;
                }
                char ch = strClause[i];
                if (ch >= 'A' && ch <= 'Z' && ch != 'X')
                    strLit += ('x' + to_string(ch - 'A' + 1));
                else {
                    switch (ch) {
                        case 'p':ch = 'P';
                            break;
                        case 'X':ch = 'x';
                            break;
                        case 'c':ch = 'a';
                            break;
                        case '=':ch = ',';
                            strLit = "E(" + strLit;
                            isEqnLit = true;
                            break;
                        case '!':
                            if (strClause[++i] == '=') {
                                ch = ',';
                                strLit = "~E(" + strLit;
                                isEqnLit = true;
                            }
                            break;
                        default:
                            break;
                    }
                    strLit += ch;
                }
                if (strClause[i] == '=') {
                    while (strClause[++i] == ' ') {
                    }
                    if (strClause[i] >= '0' && strClause[i] <= '9') {
                        strLit += "a" + to_string(200 + strClause[i] - '0' + 1);
                    } else
                        --i;
                }
            }
        }
    }
    uOrigClaSizeCom--;
    return READOK;
}

RESULT Formula::ReadPreFile()
{
    vector<int> funcFixVect;
    funcFixVect.reserve(6);
    string strClause;
    ifstream ifFolFile(FileOp::g_PreFileFullName);
    while (getline(ifFolFile, strClause)) {
        if (strClause == "")continue;
        if (strClause.find_first_of('-') != string::npos)
            break;
        if (strClause[0] == '%') {
            if (strClause.find("ClaNum") != string::npos) {
                int begPos = strClause.find_first_of(":") + 1;
                int endPos = strClause.find_first_of("(");
                uOrigClaSize = stoi(strClause.substr(begPos, endPos - begPos));
                StrategyParam::orginalClaNum = uOrigClaSize;
                int nextPos = strClause.find("EqnAxiom", endPos);
                if (nextPos != string::npos) {
                    begPos = strClause.find(":", nextPos);
                    endPos = strClause.find(")", begPos) - begPos;
                    StrategyParam::g_AxiomNum = stoi(strClause.substr(begPos + 1, endPos));
                    StrategyParam::g_ClaNumNoAxiom = uOrigClaSize - stoi(strClause.substr(begPos + 1, endPos));
                    if((StrategyParam::orginalClaNum > 9500) && (StrategyParam::g_AxiomNum > 0)){
                        StrategyParam::problemDifficult = false;
                    }
                }
            } else if (strClause.find("MaxLitNum") != string::npos) {
                StrategyParam::R_INIT_MAX_NUM = stoi(strClause.substr(strClause.find_first_of(":") + 1));
                if (StrategyParam::R_MAX_NUM == -1)
                    StrategyParam::R_MAX_NUM = stoi(strClause.substr(strClause.find_first_of(":") + 1));
                else if((StrategyParam::R_MAX_NUM < stoi(strClause.substr(strClause.find_first_of(":") + 1))) && !StrategyParam::Rsmall){
                    StrategyParam::R_MAX_NUM = stoi(strClause.substr(strClause.find_first_of(":") + 1));
                }
            } else if (strClause.find("MaxfuncDepth") != string::npos) {
                StrategyParam::R_INIT_MAX_FUNCLAYER = stoi(strClause.substr(strClause.find_first_of(":") + 1));
                if (StrategyParam::R_MAX_FUNCLAYER == -1)
                    StrategyParam::R_MAX_FUNCLAYER = stoi(strClause.substr(strClause.find_first_of(":") + 1));
                else if((StrategyParam::R_MAX_FUNCLAYER < stoi(strClause.substr(strClause.find_first_of(":") + 1))) && !StrategyParam::RFsmall){
                    StrategyParam::R_MAX_FUNCLAYER = stoi(strClause.substr(strClause.find_first_of(":") + 1));
                }
            }else if(strClause.find("goalClause") != string::npos){
                
                string goalId = strClause.substr(strClause.find_first_of(":") + 2);
                vector<string>vectGoalID = Split_c(goalId, Id_SPLIT_CAHR);
                for(int m=0;m<vectGoalID.size();++m){
                    StrategyParam::goalClaId.push_back(atoi(vectGoalID[m].c_str()));
                }
            }else if(strClause.find("singleGoalClaCount") != string::npos){
                StrategyParam::singleGoalCount = stoi(strClause.substr(strClause.find_first_of(":") + 1));
            }
            continue;
        }
        if (!(isalpha(strClause[0]) || strClause[0] == '['))
            return READERR;
        vector<string>vectLit = Split_c(strClause, P_SPLIT_CAHR); 
        int begPos = strClause.find_first_of("[") + 1;
        int endPos = strClause.find_first_of("]");
        UINT32 uClaId = stoi(strClause.substr(begPos, endPos - begPos));
        strClause = strClause.substr(endPos + 1);
        vector<string>vectsLit = Split_c(strClause, P_SPLIT_CAHR);
        Clause *c1 = new Clause(vectsLit, uClaId);
        c1->orgCla = true;
       if (StrategyParam::g_AxiomNum >= uClaId) {
           c1->isAxtom = true;
            setEnq.insert(c1);
        } else {
           if(c1->judgeGoalCla()){
               c1->isGoal = true;
               c1->goaldeepth = 0;
           }else{
               c1->isGoal = false;
           }
            setCla.insert(c1);
        }
        setOriClas.insert(c1);
        int funcDeepth = c1->compFuncDeepthCla();
        if(find(funcFixVect.begin(), funcFixVect.end(), funcDeepth) == funcFixVect.end())
            funcFixVect.push_back(funcDeepth);
    }
    sort(funcFixVect.begin(),funcFixVect.end());
    int end1 = funcFixVect[funcFixVect.size()-1];
    StrategyParam::funcOrgFixDeepthMax = end1;
     if(funcFixVect.size()>=2){
         StrategyParam::funcOrgFixDeepthLessMax = funcFixVect[funcFixVect.size()-2];
     }
     else{
         StrategyParam::funcOrgFixDeepthLessMax = end1;
     }
    funcFixVect.clear();
    vector<int>().swap(funcFixVect);
    return READOK;
}

string Formula::GetFormulaInfo()
{
    StrategyParam::g_ClaNumNoAxiom = setCla.size();
    string sInfo = "%ClaNum:" + to_string(uOrigClaSize) + "(EqnAxiom:" + to_string(setEnq.size()) + ")";
    UINT32 uVarNum = 0, uSingletonVarNum = 0, uMaxLitNum = 0, uMaxFuncDepth = 0;
    string goalClaStr;
    int singleGoalCount = 0;
    bool flag = false;
    for (auto&cla : setCla) {
        if (uMaxLitNum < cla->uLitNum) uMaxLitNum = cla->uLitNum;
        if (uMaxFuncDepth < cla->uFuncDepth) uMaxFuncDepth = cla->uFuncDepth;
        uVarNum += cla->uVarCount;
        uSingletonVarNum += cla->MaxVarId();
        if(cla->isGoal){
            flag = true;
            goalClaStr += " " + to_string(cla->uClaId);
            if(cla->uLitNum == 1){
                ++singleGoalCount;
            }
        }
    }
    sInfo += "\n%VarNum:" + to_string(uVarNum) + "(SingletonVarNum:" + to_string(uSingletonVarNum) + ")"; 
    sInfo += "\n%MaxLitNum:" + to_string(uMaxLitNum);
    sInfo += "\n%MaxfuncDepth:" + to_string(uMaxFuncDepth);
    sInfo += "\n%SharedTerms:" + to_string(g_HashTree.size());
    if(flag)
        sInfo +="\n%goalClause:"+ goalClaStr;
    if(singleGoalCount>0)
        sInfo +="\n%singleGoalClaCount:"+ to_string(singleGoalCount);
    return sInfo;
}

string Formula::ToString(bool isOutInfo)
{
    string sFol = "";
    string sInfo = "";
    set<Clause*, claCmpByRule> sortCla(setCla.begin(), setCla.end());
    for (auto& cla : sortCla) {
        string sCla;
        cla->toString(sCla);
        sFol += sCla + "\n";
        sInfo += sCla + "\n";
    }
    sFol += "%EqnAxiom\n";
    for (auto& cla : setEnq) {
        string sCla;
        cla->toString(sCla);
        sFol += sCla + "\n";
        sInfo += sCla + "\n";
    }
    if (isOutInfo)
        FileOp::PrintINFO(sInfo);
    return sFol;
}

void Formula::initOrgClaSetAttr()
{
    UINT32 maxSize = 0, minSize = 12000;
    UINT32 maxPairSize = 0, minPairSize = 12000;
     UINT32 count = 0;
    for (auto& cla : setOriClas) {
        vector <UINT8> saveTmp;
        saveTmp.reserve(6);
        for(UINT16 i=0; i<cla->uLitNum; ++i){
            if(cla->LitPtr[i]->isConLit()){
                cla->LitPtr[i]->litAttr = 0;
                saveTmp.push_back(0);
            }
            else if(cla->LitPtr[i]->isSingleVarliteral() && !(cla->LitPtr[i]->isShareVarLiteral())){
                cla->LitPtr[i]->litAttr = 1;
                 saveTmp.push_back(1);
            }
            else if(cla->LitPtr[i]->isShareVarLiteral()){
                cla->LitPtr[i]->litAttr = 2;
                 saveTmp.push_back(2);
            }
            cla->LitPtr[i]->countPred = cla->LitPtr[i]->getPredLst()->size() -1;
            cla->LitPtr[i]->countPairPred = cla->LitPtr[i]->getPairPredLst()->size();
            count = 0;
            cla->LitPtr[i]->subTerm->countTermE(count,1,2);
            cla->LitPtr[i]->termCount12 = count;
            cla->termCount12 += count;      
            count = 0;
            cla->LitPtr[i]->subTerm->countTermE(count,1,1);
            cla->LitPtr[i]->termCount11 = count;
            cla->termCount11 += count;     
            count = 0;
            cla->LitPtr[i]->subTerm->countTermE(count,2,1);
            cla->LitPtr[i]->termCount21 = count;
            cla->termCount21 += count;         
            if(maxSize < cla->LitPtr[i]->countPred){
                maxSize = cla->LitPtr[i]->countPred;         
            }
            if(minSize > cla->LitPtr[i]->countPred){
                minSize = cla->LitPtr[i]->countPred;
            }
            if(maxPairSize < cla->LitPtr[i]->countPairPred){
                maxPairSize = cla->LitPtr[i]->countPairPred;
            }
            if(minPairSize > cla->LitPtr[i]->countPairPred){
                minPairSize = cla->LitPtr[i]->countPairPred;
            }
            if(maxPredTermCount12 < cla->LitPtr[i]->termCount12)
                maxPredTermCount12 = cla->LitPtr[i]->termCount12;
            if(maxPredTermCount11 < cla->LitPtr[i]->termCount11)
                maxPredTermCount11 = cla->LitPtr[i]->termCount11;
            if(maxPredTermCount21 < cla->LitPtr[i]->termCount21)
                maxPredTermCount21 = cla->LitPtr[i]->termCount21;
        }
        if(find(saveTmp.begin(),saveTmp.end(),2) != saveTmp.end()){
            cla->claAttr = 2;
        }
        else if(find(saveTmp.begin(),saveTmp.end(),1) != saveTmp.end()){
            cla->claAttr = 1;
        }
        else
            cla->claAttr = 0;
        saveTmp.clear();
        vector <UINT8>().swap(saveTmp);
        cla->maxPred = maxSize;
        cla->minPred = minSize;
        cla->maxPairPred = maxPairSize;
        cla->minPairPred = minPairSize;
        if(maxTermCount12 < cla->termCount12){
            maxTermCount12 = cla->termCount12;
        }
        if(maxTermCount11 < cla->termCount11){
            maxTermCount11 = cla->termCount11;
        }
        if(maxTermCount21 < cla->termCount21){
            maxTermCount21 = cla->termCount21;
        }
    }
}

void Formula::initGenerateClaAttr(Clause* cla)
{
    UINT32 maxSize = 0, minSize = 12000;
    UINT32 maxPairSize = 0, minPairSize = 12000;
    UINT32 count = 0;
    vector <UINT8> saveTmp;
    saveTmp.reserve(6);
    for (UINT16 i = 0; i < cla->uLitNum; ++i) {
        if (cla->LitPtr[i]->isConLit()) {
            cla->LitPtr[i]->litAttr = 0; 
            saveTmp.push_back(0);
        } else if (cla->LitPtr[i]->isSingleVarliteral() && !(cla->LitPtr[i]->isShareVarLiteral())) {
            cla->LitPtr[i]->litAttr = 1; 
            saveTmp.push_back(1);
        } else if (cla->LitPtr[i]->isShareVarLiteral()) {
            cla->LitPtr[i]->litAttr = 2;
            saveTmp.push_back(2);
        }
        cla->LitPtr[i]->countPred = cla->LitPtr[i]->getPredLst()->size() - 1;
        cla->LitPtr[i]->countPairPred = cla->LitPtr[i]->getPairPredLst()->size();
        count = 0;
        cla->LitPtr[i]->subTerm->countTermE(count, 1, 2);
        cla->LitPtr[i]->termCount12 = count;
        cla->termCount12 += count;
        count = 0;
        cla->LitPtr[i]->subTerm->countTermE(count, 1, 1);
        cla->LitPtr[i]->termCount11 = count;
        cla->termCount11 += count;
        count = 0;
        cla->LitPtr[i]->subTerm->countTermE(count, 2, 1);
        cla->LitPtr[i]->termCount21 = count;
        cla->termCount21 += count;
        if (maxSize < cla->LitPtr[i]->countPred) {
            maxSize = cla->LitPtr[i]->countPred;
        } else if (minSize > cla->LitPtr[i]->countPred) {
            minSize = cla->LitPtr[i]->countPred;
        }
        if (maxPairSize < cla->LitPtr[i]->countPairPred) {
            maxPairSize = cla->LitPtr[i]->countPairPred;
        } else if (minPairSize > cla->LitPtr[i]->countPairPred) {
            minPairSize = cla->LitPtr[i]->countPairPred;
        }
    }
    if (find(saveTmp.begin(), saveTmp.end(), 2) != saveTmp.end()) {
        cla->claAttr = 2;
    } else if (find(saveTmp.begin(), saveTmp.end(), 1) != saveTmp.end()) {
        cla->claAttr = 1;
    } else
        cla->claAttr = 0; 
    saveTmp.clear();
    vector <UINT8>().swap(saveTmp);
    cla->maxPred = maxSize;
    cla->minPred = minSize;
    cla->maxPairPred = maxPairSize;
    cla->minPairPred = minPairSize;
}

string Formula::competInitClauseSet(){
    string sFol = "";
    if(FileOp::runMode >1) {
        for (auto& cla : setCla) {
            if (cla->isAxtom)
                continue;
            if (cla->isGoal == false) {
                string sCla;
                cla->toPureString(sCla);
                sFol += sCla + "\n";
            } else {
                string sCla;
                cla->toPureString(sCla);
                sFol += "GOAL" + sCla + "\n";
            }
        } 
    }
    return sFol;
}

string Formula::ToSingleString()
{
    string sFol = "";
    if(StrategyParam::preprocess){
        if(StrategyParam::g_AxiomNum > 0)
                sFol += "等词公理个数:" + to_string(StrategyParam::g_AxiomNum) + "\n";
        for (auto& cla : setOriClas) {
            if (cla->isAxtom) {
                string sCla;
                cla->toPureString(sCla);
                sFol += sCla + "\n";
            }
        }
        for(auto& cla : setOriClas){
            if (cla->isAxtom == false) {
                if (cla->isGoal == false) {
                    string sCla;
                    cla->toPureString(sCla);
                    sFol += sCla + "\n";
                } else {
                    string sCla;
                    cla->toPureString(sCla);
                    sFol += "GOAL" + sCla + "\n";
                }
            }
        }
        return sFol;
    }
    ReadTPTPFileAgain();
    UINT32 axiomCount = 0;
    for (auto& cla : setOriClasCom) {
        if (cla->isAxtom) {
            ++axiomCount;
        }
    } 
    if(StrategyParam::useCombinedSys){
        if((StrategyParam::useCombinedSysMethod == 0) || (StrategyParam::useCombinedSysMethod == 2) || (StrategyParam::useCombinedSysMethod == 4) || (StrategyParam::useCombinedSysMethod == 6)|| 
                (StrategyParam::useCombinedSysMethod == 8) || (StrategyParam::useCombinedSysMethod == 10) || (StrategyParam::useCombinedSysMethod == 12)){
            if(StrategyParam::g_AxiomNum > 0)
                sFol += "等词公理个数:" + to_string(axiomCount) + "\n";
            for (auto& cla : setOriClasCom) {
                if(cla->isAxtom)
                {    
                    string sCla;
                    cla->toPureString(sCla);
                    sFol += sCla + "\n";
                }
            } 
            for (auto& cla : setOriClasCom) {
                if(cla->isAxtom == false)
                {
                    if(cla->isGoal == false ){
                        string sCla;
                        cla->toPureString(sCla);
                        sFol += sCla + "\n";
                    }
                    else{
                        string sCla;
                        cla->toPureString(sCla);
                        sFol += "GOAL" + sCla + "\n";
                    }
                }
                
            } 
        }
        else if((StrategyParam::useCombinedSysMethod == 1) || (StrategyParam::useCombinedSysMethod == 3) || (StrategyParam::useCombinedSysMethod == 5) || (StrategyParam::useCombinedSysMethod == 7)|| 
                (StrategyParam::useCombinedSysMethod == 9) || (StrategyParam::useCombinedSysMethod == 11) || (StrategyParam::useCombinedSysMethod == 13)){
            for (auto& cla : setOriClasCom) {
                if (cla->isAxtom)
                    continue;
                if (cla->isGoal == false) {
                    string sCla;
                    cla->toPureString(sCla);
                    sFol += sCla + "\n";
                } else {
                    string sCla;
                    cla->toPureString(sCla);
                    sFol += "GOAL" + sCla + "\n";
                }
            } 
        }
    }
    else if(FileOp::runMode >1) {
        for (auto& cla : setOriClasCom) {
            if (cla->isAxtom)
                continue;
            if (cla->isGoal == false) {
                string sCla;
                cla->toPureString(sCla);
                sFol += sCla + "\n";
            } else {
                string sCla;
                cla->toPureString(sCla);
                sFol += "GOAL" + sCla + "\n";
            }
        } 
    }
    for (auto& cla : setCla) {
        if(StrategyParam::useCombinedSysMethod == 0) {
            if ((cla ->uLitNum >= 2) || (cla->uClaId <= uOrigClaSize))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if (maxPredTermCount12 < cla->termCount12)
                    continue;
            } else if (StrategyParam::combinedRMode == 1) {
                if (maxPredTermCount11 < cla->termCount11)
                    continue;
            } else if (StrategyParam::combinedRMode == 2) {
                if (maxPredTermCount21 < cla->termCount21)
                    continue;
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 1){
            if ((cla ->uLitNum >= 2) || (cla->uClaId <= uOrigClaSize) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if (maxPredTermCount12 < cla->termCount12)
                    continue;
            } else if (StrategyParam::combinedRMode == 1) {
                if (maxPredTermCount11 < cla->termCount11)
                    continue;
            } else if (StrategyParam::combinedRMode == 2) {
                if (maxPredTermCount21 < cla->termCount21)
                    continue;
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 2){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 1) && (cla->hasEqn() == false)))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 3){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 1) && (cla->hasEqn() == false)) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
       else if(StrategyParam::useCombinedSysMethod == 4){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 2)))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 5){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 2)) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
         else if(StrategyParam::useCombinedSysMethod == 6){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 3)))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 7){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 3)) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 8){
            if ((cla->uClaId <= uOrigClaSize))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 9){
            if ((cla->uClaId <= uOrigClaSize) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 10){
            if ((cla->uClaId <= uOrigClaSize) || (cla->goaldeepth > 2))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 11){
            if ((cla->uClaId <= uOrigClaSize) || (cla->isAxtom) || (cla->goaldeepth > 2))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 12){
            if ((cla->uClaId <= uOrigClaSize) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 13){
            if ((cla->uClaId <= uOrigClaSize) || (cla->isAxtom) )
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        string sCla;
        cla->toPureString(sCla);
        sFol += sCla + "\n";
    }
    for (auto& cla : setDeductCla) {
        if (StrategyParam::useCombinedSysMethod == 0) {
            if ((cla ->uLitNum >= 2) || (cla->uClaId <= uOrigClaSize))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if (maxPredTermCount12 < cla->termCount12)
                    continue;
            } else if (StrategyParam::combinedRMode == 1) {
                if (maxPredTermCount11 < cla->termCount11)
                    continue;
            } else if (StrategyParam::combinedRMode == 2) {
                if (maxPredTermCount21 < cla->termCount21)
                    continue;
            }
        } else if (StrategyParam::useCombinedSysMethod == 1) {
            if ((cla ->uLitNum >= 2) || (cla->uClaId <= uOrigClaSize) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if (maxPredTermCount12 < cla->termCount12)
                    continue;
            } else if (StrategyParam::combinedRMode == 1) {
                if (maxPredTermCount11 < cla->termCount11)
                    continue;
            } else if (StrategyParam::combinedRMode == 2) {
                if (maxPredTermCount21 < cla->termCount21)
                    continue;
            }
        } else if (StrategyParam::useCombinedSysMethod == 2) {
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 1) && (cla->hasEqn() == false)))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if (cla->uLitNum == 1) {
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                } else {
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if (cla->uLitNum == 1) {
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                } else {
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if (cla->uLitNum == 1) {
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                } else {
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        } else if (StrategyParam::useCombinedSysMethod == 3) {
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 1) && (cla->hasEqn() == false)) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if (cla->uLitNum == 1) {
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                } else {
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if (cla->uLitNum == 1) {
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                } else {
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if (cla->uLitNum == 1) {
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                } else {
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 4){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 2)))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 5){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 2)) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
         else if(StrategyParam::useCombinedSysMethod == 6){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 3)))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 7){
            if ((cla->uClaId <= uOrigClaSize) || ((cla->uLitNum > 3)) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 8){
            if ((cla->uClaId <= uOrigClaSize))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 9){
            if ((cla->uClaId <= uOrigClaSize) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 10){
            if ((cla->uClaId <= uOrigClaSize) || (cla->goaldeepth > 2))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 11){
            if ((cla->uClaId <= uOrigClaSize) || (cla->isAxtom) || (cla->goaldeepth > 2))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 12){
            if ((cla->uClaId <= uOrigClaSize) || (cla->isAxtom))
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        else if(StrategyParam::useCombinedSysMethod == 13){
            if ((cla->uClaId <= uOrigClaSize) || (cla->isAxtom) )
                continue;
            if (StrategyParam::combinedRMode == 0) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount12 < cla->termCount12)
                        continue;
                }
                else{
                    if (maxTermCount12 < cla->termCount12)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 1) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount11 < cla->termCount11)
                        continue;
                }
                else{
                    if (maxTermCount11 < cla->termCount11)
                        continue;
                }
            } else if (StrategyParam::combinedRMode == 2) {
                if(cla->uLitNum == 1){
                    if (maxPredTermCount21 < cla->termCount21)
                        continue;
                }
                else{
                    if (maxTermCount21 < cla->termCount21)
                        continue;
                }
            }
        }
        string sCla;
        cla->toPureString(sCla);
        sFol += sCla + "\n";
    }
    return sFol;
}

string Formula::ToSingleStringSaturate()
{
    string sFol = "";
    for (auto& cla : setDeductCla) {
        if((cla ->uLitNum > 2) || (cla->uClaId <= uOrigClaSize) || (cla->goaldeepth > 1))
            continue;
        string sCla;
        cla->toString(sCla);
        sFol += sCla + "\n";
    }
    return sFol;
}

string Formula::ToSingleAfterString()
{
    string sFol = "";
    for (auto& cla : setCla) {
        if(cla ->uLitNum != 1)
            continue;
        string sCla;
        cla->toString(sCla);
        sFol += sCla + "\n";
        uOrigClaSingleSize++;
    }
    for (auto& cla : setEnq) {
        if(cla ->uLitNum != 1)
            continue;
        setSingleEnq.push_back(cla->LitPtr[0]);
        string sCla;
        cla->toString(sCla);
        sFol += sCla + "\n";
    }
    return sFol;
}

void Formula::resetSingleClaNum()
{
    uOrigClaSingleSize = 0;
    for (auto& cla : setCla) {
        if(cla ->uLitNum != 1)
            continue;
        uOrigClaSingleSize++;
    }
}

void Formula::resetSingleClaNumSaturate()
{
    uOrigClaSingleSize = 0;
    for (auto& cla : setDeductCla) {
        if(cla ->uLitNum != 1)
            continue;
        uOrigClaSingleSize++;
    }
}

bool Formula::setHasBindTerm(){
    for (auto& cla : setCla) {
        if(cla->HasTermBind())
            return true;
    }
    return false;
}

string Formula::ToUnUseString()
{
    string sFol = "";
    for (auto& cla : setCla) {
        if(cla ->isUse || cla->uClaId > uOrigClaSize)
            continue;
        string sCla;
        cla->toString(sCla);
        sFol += sCla + "\n";
    }
    for (auto& cla : setEnq) {
        if(cla ->isUse || cla->uClaId > uOrigClaSize)
            continue;
        string sCla;
        cla->toString(sCla);
        sFol += sCla + "\n";
    }
    for (auto& cla : setDeductCla) {
        if(cla ->isUse || cla->uClaId > uOrigClaSize)
            continue;
        string sCla;
        cla->toString(sCla);
        sFol += sCla + "\n";
    }
    return sFol;
}

void Formula::RedundacyByGen() {
    string out;
    int count = 0;
    set<Clause*>::iterator claP_it = setCla.begin();
    clock_t start = clock();
    for (; claP_it != setCla.end();) {
        if((*claP_it)->uLitNum >36 ||(*claP_it)->uVarCount > 36){
            ++claP_it;
            continue;
        }    
        UINT32 candRow;
        Subsumption subsump;
        if (subsump.GenCla(*claP_it, candRow)) {
            out += to_string(candRow) + "->" + to_string((*claP_it)->uClaId) + ";";
            (*claP_it)->DelLits();
            delete(*claP_it);
            claP_it = setCla.erase(claP_it);
            ++count;
        } else
            ++claP_it;
    }
    string time = to_string(double(clock() - start) / CLOCKS_PER_SEC);
    out += "# 冗余子句数:" + to_string(count) + " [用时]：" + time + " s";
    FileOp::LogOut(out);
}

UINT16 Formula::BackwardSubsump(Clause& cla_R)
{
    clock_t start = clock();
    string out;
    UINT16 uDelNum = 0;
    set<Clause*> vRundacyClas;
    Subsumption subsump;
    subsump.InstCla(cla_R, vRundacyClas);
    set<Clause*>::iterator it;
    set<Clause*>::iterator itE;
    set<Clause*>::iterator itDE;
    if (!vRundacyClas.empty()) {
        out += to_string(cla_R.uClaId) + "->";
        for (auto&cla : vRundacyClas) {
            if(cla->topCopyCla != nullptr)
                continue;
            ++uDelNum;
            out += to_string(cla->uClaId) + ',';
            it = setCla.find(cla);
            itE = setEnq.find(cla);
            itDE = setDeductCla.find(cla);
            if (it != setCla.end()) {
                (*it)->DelLits();
                delete(*it);
                setCla.erase(it);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }               
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }               
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }               
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itE != setEnq.end()){
                (*itE)->DelLits();
                delete(*itE);
                setEnq.erase(itE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }               
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itDE != setDeductCla.end()){
                (*itDE)->DelLits();
                delete(*itDE);
                setDeductCla.erase(itDE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
        }
        string time = to_string(double(clock() - start) / CLOCKS_PER_SEC);
        out = "# R" + to_string(cla_R.uClaId) + "删除子句" + to_string(uDelNum) + "条:" + out + " [用时]：" + time + " s";
        FileOp::LogOut(out);
    }
    return uDelNum;
}

UINT16 Formula::BackwardSubsumpClaSet(Clause& cla_R, vector<Clause*>& claSet, bool flag)
{
    clock_t start = clock();
    string out;
    UINT16 uDelNum = 0;
    set<Clause*> vRundacyClas;
    Subsumption subsump;
    subsump.InstCla(cla_R, vRundacyClas);
    set<Clause*>::iterator it;
    set<Clause*>::iterator itE;
    set<Clause*>::iterator itDE;
    if (!vRundacyClas.empty()) {
        out += to_string(cla_R.uClaId) + "->";
        for (auto&cla : vRundacyClas) {
            if(cla->topCopyCla != nullptr)
                continue;
            ++uDelNum;
            out += to_string(cla->uClaId) + ',';
            if(flag){
                if (find(claSet.begin(), claSet.end(), cla) != claSet.end()) {
                    int pos = -1;
                    int posPire = -1;
                    for (pos = 0; pos < claSet.size(); ++pos) {
                        if (claSet[pos] == cla) {
                            break;
                        }
                    }
                    for (posPire = 0; posPire < claSet.size(); ++posPire) {
                        if (claSet[posPire] == &cla_R) {
                            break;
                        }
                    }
                    if (posPire < pos) {
                        swap(claSet[pos], claSet[claSet.size() - 1]);
                        claSet.pop_back();
                    }
                }
            }
            else{
                 if (find(claSet.begin(), claSet.end(), cla) != claSet.end()) {
                    for (UINT8 pos = 0; pos < claSet.size(); ++pos) {
                        if (claSet[pos] == cla) {
                            swap(claSet[pos], claSet[claSet.size() - 1]);
                            claSet.pop_back();
                            break;
                        }
                    }
                 }
            }
            it = setCla.find(cla);
            itE = setEnq.find(cla);
            itDE = setDeductCla.find(cla);
            if (it != setCla.end()) {
                (*it)->DelLits();
                delete(*it);
                setCla.erase(it);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itE != setEnq.end()){
                (*itE)->DelLits();
                delete(*itE);
                setEnq.erase(itE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itDE != setDeductCla.end()){
                (*itDE)->DelLits();
                delete(*itDE);
                setDeductCla.erase(itDE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
        }
        string time = to_string(double(clock() - start) / CLOCKS_PER_SEC);
        out = "# R" + to_string(cla_R.uClaId) + "删除子句" + to_string(uDelNum) + "条:" + out + " [用时]：" + time + " s";
        FileOp::LogOut(out);
    }
    return uDelNum;
}

UINT16 Formula::BackwardSubsumpSuper(Clause& cla_R, vector<Clause*>& chooseCla)
{
    clock_t start = clock();
    string out;
    UINT16 uDelNum = 0;
    set<Clause*> vRundacyClas;
    Subsumption subsump;
    subsump.InstCla(cla_R, vRundacyClas);
    set<Clause*>::iterator it;
    set<Clause*>::iterator itE;
    set<Clause*>::iterator itDE;
    if (!vRundacyClas.empty()) {
        out += to_string(cla_R.uClaId) + "->";
        for (auto&cla : vRundacyClas) {
            ++uDelNum;
            out += to_string(cla->uClaId) + ',';
            if (find(chooseCla.begin(), chooseCla.end(), cla) != chooseCla.end()) {
                for(UINT8 pos=0; pos<chooseCla.size(); ++pos) {
                    if (chooseCla[pos] == cla) {
                        swap(chooseCla[pos], chooseCla[chooseCla.size() - 1]);
                        chooseCla.pop_back();
                        break;
                    }
                }
            }
            it = setCla.find(cla);
            itE = setEnq.find(cla);
            itDE = setDeductCla.find(cla);
            if (it != setCla.end()) {
                (*it)->DelLits();
                delete(*it);
                setCla.erase(it);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }               
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }              
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }               
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itE != setEnq.end()){
                (*itE)->DelLits();
                delete(*itE);
                setEnq.erase(itE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }       
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }               
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itDE != setDeductCla.end()){
                (*itDE)->DelLits();
                delete(*itDE);
                setDeductCla.erase(itDE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }               
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
        }
        string time = to_string(double(clock() - start) / CLOCKS_PER_SEC);
        out = "# R" + to_string(cla_R.uClaId) + "删除子句" + to_string(uDelNum) + "条:" + out + " [用时]：" + time + " s";
        FileOp::LogOut(out);
    }
    return uDelNum;
}

void Formula::resetNoPairClaSet(bool difficulty){
    set<Clause*>::iterator claP_it = noPairSingleCla.begin();
        for (; claP_it != noPairSingleCla.end();) {
            if((*claP_it)->isUse &&!difficulty){
                claP_it = noPairSingleCla.erase(claP_it);
            }
            else
                ++claP_it;
        }
}

UINT16 Formula::BackwardSubsumpDirect(Clause& cla_R,Clause& cla_Del)
{
    clock_t start = clock();
    string out;
    UINT16 uDelNum = 0;
    set<Clause*> vRundacyClas;
    Subsumption subsump;
    subsump.InstClaDirect(cla_R,cla_Del, vRundacyClas);
    set<Clause*>::iterator it;
    set<Clause*>::iterator itE;
    set<Clause*>::iterator itDE;
    if (!vRundacyClas.empty()) {
        out += to_string(cla_R.uClaId) + "->";
        for (auto&cla : vRundacyClas) {
            ++uDelNum;
            out += to_string(cla->uClaId) + ',';
            it = setCla.find(cla);
            itE = setEnq.find(cla);
            itDE = setDeductCla.find(cla);
            if (it != setCla.end()) {
                (*it)->DelLits();
                delete(*it);
                setCla.erase(it);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }              
                set<Clause*, claCmpPDeductNum> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }               
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }               
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itE != setEnq.end()){
                (*itE)->DelLits();
                delete(*itE);
                setEnq.erase(itE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                 set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itDE != setDeductCla.end()){
                 (*itDE)->DelLits();
                delete(*itDE);
                setDeductCla.erase(itDE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                 set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
        }
        string time = to_string(double(clock() - start) / CLOCKS_PER_SEC);
        out = "# R" + to_string(cla_R.uClaId) + "删除子句" + to_string(uDelNum) + "条:" + out + " [用时]：" + time + " s";
        FileOp::LogOut(out);
    }
    return uDelNum;
}

UINT16 Formula::BackwardSubsumpDirectSaturate(Clause& cla_R,Clause& cla_Del,vector<Clause*> saveCurrentCla)
{
    clock_t start = clock();
    string out;
    UINT16 uDelNum = 0;
    set<Clause*> vRundacyClas;
    Subsumption subsump;
    subsump.InstClaDirect(cla_R,cla_Del, vRundacyClas);
    set<Clause*>::iterator it;
    set<Clause*>::iterator itE;
    set<Clause*>::iterator itDE;
    if (!vRundacyClas.empty()) {
        out += to_string(cla_R.uClaId) + "->";
        for (auto&cla : vRundacyClas) {
            if (find(saveCurrentCla.begin(), saveCurrentCla.end(), cla) != saveCurrentCla.end()) {
                for (UINT8 k = 0; k < saveCurrentCla.size(); ++k) {
                    if (cla == saveCurrentCla[k]) {
                        swap(saveCurrentCla[k], saveCurrentCla[saveCurrentCla.size() - 1]);
                        saveCurrentCla.pop_back();
                        break;
                    }
                }
            }
            ++uDelNum;
            out += to_string(cla->uClaId) + ',';
            it = setCla.find(cla);
            itE = setEnq.find(cla);
            itDE = setDeductCla.find(cla);
            if (it != setCla.end()) {
                (*it)->DelLits();
                delete(*it);
                setCla.erase(it);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }             
                set<Clause*, claCmpPDeductNum> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }               
                set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }               
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itE != setEnq.end()){
                (*itE)->DelLits();
                delete(*itE);
                setEnq.erase(itE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                 set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
            else if(itDE != setDeductCla.end()){
                 (*itDE)->DelLits();
                delete(*itDE);
                setDeductCla.erase(itDE);
                if (cla->uClaId < uOrigClaSize + 1 && setOriClas.size() > 0) {
                    set<Clause*> ::iterator delIt = setOriClas.find(cla);
                    if (delIt != setOriClas.end()) {
                        setOriClas.erase(delIt);
                    }
                }
                 set<Clause*> ::iterator delItGoal= saveGoalClaSet.find(cla);
                if (delItGoal != saveGoalClaSet.end()) {
                    saveGoalClaSet.erase(delItGoal);
                }
                set<Clause*> ::iterator delItNoPair = noPairSingleCla.find(cla);
                if (delItNoPair != noPairSingleCla.end()) {
                    noPairSingleCla.erase(delItNoPair);
                }
                set<Clause*> ::iterator delItSingleCla = setSingleCla.find(cla);
                if (delItSingleCla != setSingleCla.end()) {
                    setSingleCla.erase(delItSingleCla);
                }
                set<Clause*> ::iterator delItEqn = eqnDemod.find(cla);
                if (delItEqn != eqnDemod.end()) {
                    eqnDemod.erase(delItEqn);
                }
                continue;
            }
        }
        string time = to_string(double(clock() - start) / CLOCKS_PER_SEC);
        out = "# R" + to_string(cla_R.uClaId) + "删除子句" + to_string(uDelNum) + "条:" + out + " [用时]：" + time + " s";
        FileOp::LogOut(out);
    }
    return uDelNum;
}

set<Clause*>::iterator Formula::NextStartClause(Clause* newCla)
{
    Clause * claPtr(newCla);
    switch (StrategyParam::START_STRATEGY_OTHERTRI) {
        case 0:
            InsertCla(claPtr);
            return min_element(setCla.begin(), setCla.end(), SortRule::ClaCmp);
        case 1: 					
            InsertCla(claPtr);
        default: 
            InsertCla(claPtr);
            return setCla.begin();
    }
}

void Formula::InsertCla(Clause* newCla)
{
    for (UINT16 i = 0; i < newCla->uLitNum; ++i) {
        newCla->LitPtr[i]->AddPredLst();
    }
    return ;
}

extern int g_ConflictNum_HashTree;
void Formula::PrintSharedTerm()
{
   for (auto&t : g_HashTree) {
        t->termPtr->Print();
        cout << " :" << to_string(t->times) << endl;
    }
    cout << "总冲突次数：" << to_string(g_ConflictNum_HashTree);
}

void Formula::DelSubSet(UINT32 cla_id)
{
    set<Clause*>::iterator claP_it = setCla.begin();
    for (; claP_it != setCla.end();) {
        if ((*claP_it)->uClaId > cla_id) {
            (*claP_it)->DelLits();
            delete(*claP_it);
            claP_it = setCla.erase(claP_it);
        }
        else
            claP_it++;
    }
}

void Formula::DelSubSetSuper(UINT32 cla_id)
{
    set<Clause*>::iterator claP_it = eqnDemod.begin();
    for (; claP_it != eqnDemod.end();) {
        if ((*claP_it)->uClaId > cla_id) {
            (*claP_it)->DelLits();
            delete(*claP_it);
            claP_it = eqnDemod.erase(claP_it);
        }
        else
            claP_it++;
    }
}

void Formula::DelSet()
{
    set<Clause*>::iterator claP_itCom = setOriClasCom.begin();
    for (; claP_itCom != setOriClasCom.end();) {
        (*claP_itCom)->DelLits();
        delete(*claP_itCom);
        claP_itCom = setOriClasCom.erase(claP_itCom);
    }
    set<Clause*>::iterator claP_it = setCla.begin();
    for (; claP_it != setCla.end();) {
        (*claP_it)->DelLits();
        delete(*claP_it);
        claP_it = setCla.erase(claP_it);
    }
    claP_it = setEnq.begin();
    for (; claP_it != setEnq.end();) {
        (*claP_it)->DelLits();
        delete(*claP_it);
        claP_it = setEnq.erase(claP_it);
    }
    claP_it = setOriClas.begin();
    for (; claP_it != setOriClas.end();) {
        claP_it = setOriClas.erase(claP_it);
    }
}

bool Formula::unitFindUintClaUnify(TriAlg& triAlg,Clause* givenCla) {
    if(givenCla->uLitNum > 1)
        return false;
    Literal* QLit = givenCla->LitPtr[0];
    vector<Term*> chgVarT;
    chgVarT.reserve(6);
    for (auto& aClap : setCla) {
        if(aClap->uLitNum > 1)
            continue;
        if (QLit->isPairPred(aClap->LitPtr[0]) && triAlg.unify.LitMgu(QLit, aClap->LitPtr[0], chgVarT)) {
            FileOp::CreateRunInfoFile(FileOp::runFileId);
            *FileOp::ofRealInfoOut << "从编号为" + to_string(givenCla->uClaId) + "子句开始构建三角形" << endl;
            triAlg.unify.WriteTriPath(QLit);   
            triAlg.unify.WriteTriPath(aClap->LitPtr[0]);
            *FileOp::ofRealInfoOut << "R:空子句" << endl;
            FileOp::LogOut("# ==子句" + to_string(givenCla->uClaId) + "起步，结论正确(UNSAT)！");
            if(!StrategyParam::orForCompition && (StrategyParam::problemDiv == 0))
                cout << "得出结论:不可满足" << endl;
            triAlg.unify.ClearVarBind(chgVarT);
            return true;
        } else {
            triAlg.unify.ClearVarBind(chgVarT);
        }
    }
    triAlg.unify.ClearVarBind(chgVarT);
    vector<Term*>().swap(chgVarT);
    return false;
}

RESULT Formula::chooseCurrentCLaSet(vector<Clause*>* saveCurrentClaSet,UINT32 incLen,TriAlg& triAlg,UINT32 startId, UINT32 endId,UINT32& uNewClaID){
    UINT32 fifoSize = incLen/3 ;
    UINT32 termCountSize = incLen/3;
    UINT32 goalDeepthSize = incLen - fifoSize-  termCountSize;
    vector<Clause*> fifoSet;
    fifoSet.reserve(6);
    map<UINT32, vector<Clause*>> termCountSet;
    map<UINT32, vector<Clause*>> goalDeepthSet;
    map<UINT32, vector<Clause*>>::iterator iter;
    set<Clause*>::iterator claP_it = setCla.begin();
    for (; claP_it != setCla.end();) {
        if(((*claP_it)->uClaId >= startId && (*claP_it)->uClaId <= endId) && ((*claP_it)->uLitNum == 1) && unitFindUintClaUnify(triAlg,(*claP_it))){
            return UNSAT;
        }
        if((*claP_it)->uClaId >= startId && (*claP_it)->uClaId <= endId){
            if(triAlg.claHasPureLit(*this,(*claP_it)) || triAlg.judgeDemodRedundent(*this,(*claP_it),uNewClaID) ){
                string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]冗余被删除";
                FileOp::LogOut(out);
                (*claP_it)->DelLits();
                delete((*claP_it));
                claP_it = setCla.erase(claP_it);
                continue;
            }
        }
        if((*claP_it)->orgCla &&!(*claP_it)->isUse){
            fifoSet.push_back(*claP_it);
        }
        UINT32 termSum = (*claP_it)->countClaTermE(1,2);
        termCountSet[termSum].push_back(*claP_it);
        goalDeepthSet[(*claP_it)->goaldeepth].push_back(*claP_it);
        ++claP_it;
    }
    set<Clause*>::iterator claP_itE = setEnq.begin();
    for (; claP_itE != setEnq.end();) {
        if((*claP_itE)->isequEqn()){
            ++claP_itE;
            continue;
        }
        if((*claP_itE)->orgCla &&!(*claP_itE)->isUse){
            fifoSet.push_back(*claP_itE);
        }
        UINT32 termSum = (*claP_itE)->countClaTermE(1,2);
        termCountSet[termSum].push_back(*claP_itE);
        goalDeepthSet[(*claP_itE)->goaldeepth].push_back(*claP_itE);
        ++claP_itE;
    }
    stable_sort(fifoSet.begin(), fifoSet.end(), SortRule::noPairClaCmp);
    for(UINT32 i=0; i< fifoSet.size(); ++i){
        if(i == fifoSize)
            break;
         if (find(saveCurrentClaSet->begin(), saveCurrentClaSet->end(), (fifoSet[i])) == saveCurrentClaSet->end())
            saveCurrentClaSet->push_back(fifoSet[i]);
    }
    UINT32 count = 0;
    bool outFlag = false;
    for(iter = termCountSet.begin(); iter != termCountSet.end(); ++iter){
        for(UINT32 i=0; i< iter->second.size(); ++i){
            if (find(saveCurrentClaSet->begin(), saveCurrentClaSet->end(), (iter->second.at(i))) == saveCurrentClaSet->end()) {
                saveCurrentClaSet->push_back(iter->second.at(i));
                ++count;
            }
            if(count == termCountSize){
                outFlag = true;
                break;
            }
            
        }
        if(outFlag)
            break;
    }
    count = 0;
    outFlag = false;
    for(iter = goalDeepthSet.begin(); iter != goalDeepthSet.end(); ++iter){
        for(UINT32 i=0; i< iter->second.size(); ++i){
            if (find(saveCurrentClaSet->begin(), saveCurrentClaSet->end(), (iter->second.at(i))) == saveCurrentClaSet->end()) {
                saveCurrentClaSet->push_back(iter->second.at(i));
                ++count;
            }
            if(count == goalDeepthSize){
                outFlag = true;
                break;
            }
            
        }
        if(outFlag)
            break;
    }
    return fail;
}
