/*
*  Copyright 2022 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#include "Prover.h"
extern map<INT32, vector<INT32>> validPathOut;
Prover::Prover() {
}

Prover::Prover(const Prover& orig) {
}

Prover::~Prover() {
}

void Prover::validOutUnsatPath() {
    vector<INT32> validR;
    validR.reserve(8);
    vector<INT32>* usedClauses = nullptr;
    usedClauses = &validPathOut[0];
    for(int i=0; i<usedClauses->size(); ++i){
        if(usedClauses->at(i) > StrategyParam::orginalClaNum){
            validR.push_back(usedClauses->at(i));
        }
    }
    for (int i = 0; i < validR.size(); ++i) {
        usedClauses = &validPathOut[validR[i]];
        for (int j = 0; j < usedClauses->size(); ++j) {
            if (usedClauses->at(j) > StrategyParam::orginalClaNum) {
                if(find(validR.begin(),validR.end(),usedClauses->at(j)) == validR.end())
                    validR.push_back(usedClauses->at(j));
            }
        }
    } 
    if(validR.empty()==false){
        string strProofComp = FileOp::g_OutputPath  + "/usedCluaseId.txt";
        ofstream* usedId = new ofstream(strProofComp.c_str(), ios::app);
        for (int i = 0; i < validR.size(); ++i) {
            *usedId << validR[i] << endl;
        }
        usedId->close();
    }
    string outFile = FileOp::g_OutputPath + "/" + FileOp::JudgeFileFullNameNoExt + ".out";
    ifstream ifFolFile(outFile);
    string outFileTptp = FileOp::g_OutputPath + "/" + FileOp::JudgeFileFullNameNoExt + ".valid";
    ofstream ofInfoStream(outFileTptp, ios::app);
    string strLine;
    bool outFlag = false;
    while (getline(ifFolFile, strLine)) {
        if (strLine[0] == '%' || strLine[0] == '#') {
            ofInfoStream << strLine << endl;
        }
        else if((strLine.find("cnf",0)!= -1) && (strLine.find("$",0)== -1)){
            int begPos = strLine.find_first_of("(") + 1;
            int endPos = strLine.find_first_of(",");
            INT32 claId = stoi(strLine.substr(begPos, endPos - begPos));
            if(find(validR.begin(),validR.end(),claId) != validR.end()){
                ofInfoStream << strLine << endl;
                outFlag = true;
            }
            else
               outFlag = false; 
            continue;
        }
        else if((strLine.find("cnf",0)!= -1) && (strLine.find("$",0)!= -1)){
            size_t pos = strLine.find('$');
            strLine = strLine.erase(pos,1);
            ofInfoStream << strLine << endl;
            outFlag = true;
            continue;
        }
        if(outFlag){
            ofInfoStream << strLine << endl;
        }
    }
    ifFolFile.close();
    ofInfoStream.close();
    validR.clear();
    vector<INT32>().swap(validR);
}

void Prover::validOutRelease(){
    map<INT32, vector<INT32>>::iterator it;

    for(it=validPathOut.begin();it!=validPathOut.end();++it){
        it->second.clear();
        vector<INT32>().swap(it->second);
        
    }
    map<INT32, vector<INT32>>().swap(validPathOut);
}

RESULT Prover::prover(bool isOrigalS)
{
    Formula fol;
    clock_t readFile = clock();
    fol.ReadPreFile();
    if(StrategyParam::orForCompition){
        StrategyParam::readPreTime = (double) (clock() - readFile) / (double) CLOCKS_PER_SEC;
    }
    
    fol.initOrgClaSetAttr();
    
    bool findRes = false;
    if (findRes == false) {
        if (FileOp::strategyMode == 0)
            FileOp::readStrategy();
        else if (FileOp::strategyMode == 1) {
            FileOp::readStrategyN(0);
        }
    }
    if (StrategyParam::orForCompition) {
        if ((FileOp::runMode != 0)&&(FileOp::runMode != 1)) {
            clock_t end = clock();
            double time_s = (double) (end - StrategyParam::start) / (double) CLOCKS_PER_SEC;
            StrategyParam::timeLimit = (StrategyParam::timeForceLimit - time_s - 10) / 60;
            StrategyParam::timeLimitStrategies = StrategyParam::timeLimit;
        }
        if ((FileOp::runMode == 0) || (FileOp::runMode == 1)) {
            if (StrategyParam::timeLimit * 60 > (StrategyParam::timeForceLimit - 1)) {
                clock_t end = clock();
                double time_s = (double) (end - StrategyParam::start) / (double) CLOCKS_PER_SEC;
                StrategyParam::timeLimit = (StrategyParam::timeForceLimit - time_s - 10) / 60;
                StrategyParam::timeLimitStrategies = StrategyParam::timeLimit;
            }
        }
        StrategyParam::firstTimeLimit = StrategyParam::timeLimit;
    }
    else{
        StrategyParam::timeLimitStrategies = StrategyParam::timeLimit;
        StrategyParam::timeForceLimit = StrategyParam::timeLimit;
        StrategyParam::firstTimeLimit = StrategyParam::timeLimit;
    } 
    
    
    if (fol.setCla.size() == 0)
        return READERR;
    if (StrategyParam::termRewriting != 2) {
        for (auto& cla : fol.setOriClas) {
            cla->directEqn();
        }
    }
    if (!StrategyParam::orForCompition)
        cout<<"开始判定:"<<FileOp::JudgeFileName<<endl;
    
    time_t startTime;
    time(&startTime);
    if (!StrategyParam::orForCompition)
        cout<<"开始时间:"<<ctime(&startTime)<<endl;
    clock_t startA;
    startA = clock();
    Resolution res;
    UINT64 triTotalTimes = 0;
    RESULT rst;
    if(StrategyParam::useSaturate){
        rst = res.BaseAlgByRecodePathSaturate(fol,triTotalTimes); 
    }
    else
    {
        rst = res.BaseAlgByRecodePath(fol,triTotalTimes);
    }
    if((rst == UNSAT)&& (StrategyParam::problemDiv == 0)){
        clock_t end = clock();
        double time_s = (double) (end - StrategyParam::start) / (double) CLOCKS_PER_SEC;
        if((FileOp::runMode != 0 )&&(FileOp::runMode != 1)){
            time_s += StrategyParam::firstRunTime;
        }
        else if(((FileOp::runMode == 0 )||(FileOp::runMode == 1)) && (StrategyParam::cseRunMode == 0) && (StrategyParam::orForCompition)){
            time_s += StrategyParam::distriTime;
        }
        time_s = fabs(time_s);
        *FileOp::ofJudgeAllFileResult << FileOp::JudgeFileName << ":不可满足!"<<endl;
        string str = "判定结论：不可满足！";
        FileOp::insertInfo(str,time_s,triTotalTimes);
        if(StrategyParam::orForCompition) {
            cout << "%-------------------------------------------" << endl;

            cout << "% File        :CSE---1.5" << endl;
            cout << "% Problem     :" + FileOp::judgeOriginalFileName << endl;
            cout << "% Transform   :cnf" << endl;
            cout << "% Format      :tptp:raw" << endl;
            cout << "% Command     :java -jar mcs_scs.jar %d %s" << endl;
            cout << endl;
            cout << "% Result      :Theorem " + to_string(time_s) + "s" << endl;
            cout << "% Output      :CNFRefutation " + to_string(time_s) + "s" << endl;
            cout << "%-------------------------------------------" << endl;
            string cmdFileOut1 = "cat " + FileOp::judgeOriginalFileFullName;
            int res1 = system(const_cast<char*> (cmdFileOut1.c_str()));
            cout << "%-------------------------------------------" << endl;
            cout << "% Proof found" << endl;
            cout << "% SZS status Theorem for " + FileOp::judgeOriginalFileName << endl;
            cout << "% SZS output start Proof" << endl;
            string cmdFileOut2 = "cat " + FileOp::g_WorkDir + "/" + FileOp::JudgeFileFullNameNoExt + ".pre";
            int res2 = system(const_cast<char*> (cmdFileOut2.c_str()));
            cout << "%-------------------------------------------" << endl;
            validOutUnsatPath();
            string cmdFileOut3 = "cat " + FileOp::g_OutputPath + "/" + FileOp::JudgeFileFullNameNoExt + ".valid";
            int res3 = system(const_cast<char*> (cmdFileOut3.c_str()));
            validOutRelease();
            cout << "% SZS output end Proof" << endl;
            cout << "% Total time :" + to_string(time_s) + "s" << endl;
        }
        
        string strProofComp = FileOp::appPath + "/proofStatistics.txt";
        FileOp::ofProofComp = new ofstream(strProofComp.c_str(), ios::app);
        if(StrategyParam::orForCompition)
            *FileOp::ofProofComp << FileOp::judgeOriginalFileName + ": " + to_string(time_s) <<endl;
        else
            *FileOp::ofProofComp << FileOp::JudgeFileFullNameNoExt + ": " + to_string(time_s) <<endl;
        FileOp::ofProofComp->close();
        
        
    }
    else if((rst == SAT) && (!StrategyParam::orForCompition)){ 
        *FileOp::ofJudgeAllFileResult << FileOp::JudgeFileName << ":可满足!"<<endl;
    }
    else if(StrategyParam::preprocess){
        ;
    }
    else{
        if((StrategyParam::orForCompition) && (FileOp::runMode < 2)){
            FileOp::delAllFilesComplete(FileOp::g_OutputPath);
            rmdir(FileOp::g_OutputPath.c_str());
            remove(FileOp::g_PreFileFullName.c_str()); 
            cout << "CSE is exited by timeout" << endl;
        }
        else if((FileOp::runMode > 1)){
            ;
        }
        else if(StrategyParam::delRecord){
            FileOp::delAllFiles();
            if ((StrategyParam::useCombinedSys == 0) && (FileOp::runMode < 2)) {
                rmdir(FileOp::g_OutputPath.c_str());
                remove(FileOp::g_PreFileFullName.c_str());
            }
        }
        *FileOp::ofJudgeAllFileResult << FileOp::JudgeFileName << "无法判定!"<<endl;
        if((StrategyParam::orForCompition) && ((FileOp::runMode == 0 )||(FileOp::runMode == 1))){
            string strProofComp = FileOp::appPath + "/proofStatistics.txt";
            FileOp::ofProofComp = new ofstream(strProofComp.c_str(), ios::app);
            if(StrategyParam::orForCompition)
                *FileOp::ofProofComp << FileOp::judgeOriginalFileName + " :TMO‑Non" << endl;
            else
                *FileOp::ofProofComp << FileOp::JudgeFileFullNameNoExt + " :TMO‑Non" << endl;
            FileOp::ofProofComp->close();
        }
        
    }
    if(1){
        FileOp::delRunFiles();
    }
    FileOp::ReleaseOf();
    fol.DelSet();
    return rst;
}
RESULT Prover::proverTestTptp(int strategyId)
{
    Formula fol;
    clock_t readFile = clock();
    fol.ReadPreFile();
    if(StrategyParam::orForCompition){
        StrategyParam::readPreTime = (double) (clock() - readFile) / (double) CLOCKS_PER_SEC;
    }
    fol.initOrgClaSetAttr();
    FileOp::readStrategyNTptp(strategyId);
    if (StrategyParam::orForCompition) {
        if ((FileOp::runMode != 0)&&(FileOp::runMode != 1)) {
            clock_t end = clock();
            double time_s = (double) (end - StrategyParam::start) / (double) CLOCKS_PER_SEC;
            StrategyParam::timeLimit = (StrategyParam::timeForceLimit - time_s - 10) / 60;
            StrategyParam::timeLimitStrategies = StrategyParam::timeLimit;
        }
        if ((FileOp::runMode == 0) || (FileOp::runMode == 1)) {
            if (StrategyParam::timeLimit * 60 > (StrategyParam::timeForceLimit - 1)) {
                clock_t end = clock();
                double time_s = (double) (end - StrategyParam::start) / (double) CLOCKS_PER_SEC;
                StrategyParam::timeLimit = (StrategyParam::timeForceLimit - time_s - 10) / 60;
                StrategyParam::timeLimitStrategies = StrategyParam::timeLimit;
            }
        }
        StrategyParam::firstTimeLimit = StrategyParam::timeLimit;
    }
    else{
        StrategyParam::timeLimitStrategies = StrategyParam::timeLimit;
        StrategyParam::timeForceLimit = StrategyParam::timeLimit;
        StrategyParam::firstTimeLimit = StrategyParam::timeLimit;
    } 
    if (fol.setCla.size() == 0)
        return READERR;
    if (StrategyParam::termRewriting != 2) {
        for (auto& cla : fol.setOriClas) {
            cla->directEqn();
        }
    }
    if ((!StrategyParam::orForCompition) && (strategyId == 0) && (FileOp::strategyMode == 3))
        cout<<"开始判定:"<<FileOp::JudgeFileName<<endl;
    else if ((!StrategyParam::orForCompition) && (FileOp::strategyMode == 4))
        cout<<"开始判定:"<<FileOp::JudgeFileName<<endl;
    time_t startTime;
    time(&startTime);
    if (!StrategyParam::orForCompition)
        cout<<"开始时间:"<<ctime(&startTime)<<endl;
    clock_t startA;
    startA = clock();
    Resolution res;
    UINT64 triTotalTimes = 0;
    RESULT rst;
    if(StrategyParam::useSaturate){
        rst = res.BaseAlgByRecodePathSaturate(fol,triTotalTimes); 
    }
    else
    {
        rst = res.BaseAlgByRecodePath(fol,triTotalTimes); 
    }
    if(rst == UNSAT){
        clock_t end = clock();
        double time_s = (double) (end - StrategyParam::start) / (double) CLOCKS_PER_SEC;
        if((FileOp::runMode != 0 )&&(FileOp::runMode != 1)){
            time_s += StrategyParam::firstRunTime;
        }
        time_s = fabs(time_s);
        *FileOp::ofJudgeAllFileResult << FileOp::JudgeFileName << ":不可满足!"<<endl;
        string str = "判定结论：不可满足！";
        FileOp::insertInfo(str,time_s,triTotalTimes);
        if(StrategyParam::orForCompition) {
            cout << "%-------------------------------------------" << endl;
            cout << "% File        :CSE---1.5" << endl;
            cout << "% Problem     :" + FileOp::judgeOriginalFileName << endl;
            cout << "% Transform   :cnf" << endl;
            cout << "% Format      :tptp:raw" << endl;
            cout << "% Command     :java -jar mcs_scs.jar %d %s" << endl;
            cout << endl;
            cout << "% Result      :Theorem " + to_string(time_s) + "s" << endl;
            cout << "% Output      :CNFRefutation " + to_string(time_s) + "s" << endl;
            cout << "%-------------------------------------------" << endl;
            string cmdFileOut1 = "cat " + FileOp::judgeOriginalFileFullName;
            int res1 = system(const_cast<char*> (cmdFileOut1.c_str()));
            cout << "%-------------------------------------------" << endl;
            cout << "% Proof found" << endl;
            cout << "% SZS status Theorem for " + FileOp::judgeOriginalFileName << endl;
            cout << "% SZS output start Proof" << endl;
            string cmdFileOut2 = "cat " + FileOp::g_WorkDir + "/" + FileOp::JudgeFileFullNameNoExt + ".pre";
            int res2 = system(const_cast<char*> (cmdFileOut2.c_str()));
            cout << "%-------------------------------------------" << endl;
            validOutUnsatPath();
            string cmdFileOut3 = "cat " + FileOp::g_OutputPath + "/" + FileOp::JudgeFileFullNameNoExt + ".valid";
            int res3 = system(const_cast<char*> (cmdFileOut3.c_str()));
            validOutRelease();
            cout << "% SZS output end Proof" << endl;
            cout << "% Total time :" + to_string(time_s) + "s" << endl;
        }      
        string strProofComp = FileOp::appPath + "/proofStatistics.txt";
        FileOp::ofProofComp = new ofstream(strProofComp.c_str(), ios::app);
        if(StrategyParam::orForCompition)
            *FileOp::ofProofComp << FileOp::judgeOriginalFileName + ": " + to_string(time_s) <<endl;
        else
            *FileOp::ofProofComp << FileOp::JudgeFileFullNameNoExt + ": " + to_string(time_s) <<endl;
        FileOp::ofProofComp->close();     
    }
    else if((rst == SAT) && (!StrategyParam::orForCompition)){ 
        *FileOp::ofJudgeAllFileResult << FileOp::JudgeFileName << ":可满足!"<<endl;
    }
    else{
        if((StrategyParam::orForCompition) && (FileOp::runMode < 2)){
            if(FileOp::strategyMode == 3) {
                FileOp::delRunFiles();
                if(strategyId == FileOp::strategyFileSize) {
                    FileOp::delAllFilesComplete(FileOp::g_OutputPath);
                    rmdir(FileOp::g_OutputPath.c_str());

                    remove(FileOp::g_PreFileFullName.c_str()); 
                }
            } else {
                FileOp::delAllFilesComplete(FileOp::g_OutputPath);
                rmdir(FileOp::g_OutputPath.c_str());

                remove(FileOp::g_PreFileFullName.c_str()); 
            }
        }
        else if((FileOp::runMode > 1)){
            ;
        }
        else if(StrategyParam::delRecord){
            if(strategyId != FileOp::strategyFileSize-1){
                FileOp::delRunFiles();
            }
            else if(strategyId == FileOp::strategyFileSize-1)
                FileOp::delAllFiles();
            if ((StrategyParam::useCombinedSys == 0) && (FileOp::runMode < 2)) {
                rmdir(FileOp::g_OutputPath.c_str());
                if(strategyId == FileOp::strategyFileSize-1){
                    remove(FileOp::g_PreFileFullName.c_str()); 
                    cout << "无法判定" << endl;
                }
            }
        }
        *FileOp::ofJudgeAllFileResult << FileOp::JudgeFileName << "无法判定!"<<endl;
        if((StrategyParam::orForCompition) && ((FileOp::runMode == 0 )||(FileOp::runMode == 1))){
            string strProofComp = FileOp::appPath + "/proofStatistics.txt";
            FileOp::ofProofComp = new ofstream(strProofComp.c_str(), ios::app);
            if(StrategyParam::orForCompition)
                *FileOp::ofProofComp << FileOp::judgeOriginalFileName + " :TMO‑Non" << endl;
            else
                *FileOp::ofProofComp << FileOp::JudgeFileFullNameNoExt + " :TMO‑Non" << endl;
            FileOp::ofProofComp->close();
        }       
    }
    if(1){
        FileOp::delRunFiles();
    }
    fol.DelSet();
    return rst;
}
