/*
*  Copyright 2022 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#ifndef SORTRULE_H
#define SORTRULE_H
#include "StrategyParam.h"
#include"dataTypeDef.h"
#include"HashNode.h"
#include "Literal.h"
#include "Clause.h"
#include"globalFunc.h"
class SortRule {
public:    
    inline static bool CmpHashNode(const HashNode& hashNodeA, const HashNode& hashNodeB) {
        return hashNodeA.uHashCode < hashNodeB.uHashCode;
    }
    inline static bool SingleClaCmp(const Clause* c1, const Clause* c2) {
        if(c1->uLitNum != 1 || c2 ->uLitNum != 1)
            return c1->uLitNum < c2->uLitNum;
        else
            return c1->adeductNum < c2->adeductNum;
    }
    inline static bool noPairClaCmp(const Clause* c1, const Clause* c2) {
        int lenA = c1->termCount11;
        int lenB = c2 ->termCount11;
        if (c1->hasUsed() == c2->hasUsed())
            if (c1->uLitNum == c2->uLitNum)
                return lenA < lenB;
            else
                return c1->uLitNum < c2->uLitNum;
        else
            return c1->hasUsed() < c2->hasUsed();
    }
    inline static bool singleResetId(const Literal* c1, const Literal* c2) {
        return c1->claPtr->uClaId < c2->claPtr->uClaId;
    }
    inline static bool ClaCmp(const Clause* c1, const Clause* c2) {
        if (c1->Priority != c2->Priority)
            return c1->Priority > c2->Priority;
        if (StrategyParam::SEL_STARTLit_STRATEGY == 0) {
            if (c1->uLitNum == c2->uLitNum) {
                if (c1->uFuncDepth != c2->uFuncDepth) return c1->uFuncDepth < c2->uFuncDepth;
                    return (c1->MaxVarId() / (1.0 * c1->uVarCount)) < (c2->MaxVarId() / (1.0 * c2->uVarCount));
            }
            return c1->uLitNum < c2->uLitNum;
        } else {

            if (c1->uFuncDepth != c2->uFuncDepth) return c1->uFuncDepth < c2->uFuncDepth;
            double dSteadyA = c1->MaxVarId() / (1.0 * c1->uVarCount);
            double dSteadyB = c2->MaxVarId() / (1.0 * c2->uVarCount);
            if (dSteadyA != dSteadyB) return dSteadyA < dSteadyB;
            return c1->uLitNum < c2->uLitNum;
        }
    }
    inline static bool ActlitCmpTwo( Literal* litA,  Literal* litB){
        if (litA->uActive == litB->uActive)
            return 1 == TermSteadyCmp(litA->subTerm, litB->subTerm, 0);
        return litA->uActive < litB->uActive;
    }
    inline static bool ActlitCmp( Literal* litA,  Literal* litB) {
        switch (StrategyParam::candLiteral) {
            case 0:
                if (litA->sDuct == litB->sDuct) {
                    if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                        return litA->RelationValue() < litB->RelationValue();
                    } else {
                        switch (StrategyParam::candLiteralSteady)
                        {
                            case 0:
                                return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                            case 1:
                                return true;
                            case 2:
                                return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                        }
                    }
                } else {
                    switch (StrategyParam::candLiteralDeductTimes)
                    {
                        case 0:
                            return litA->sDuct < litB->sDuct;
                        case 1:
                            return true;
                        case 2:
                            return litA->sDuct > litB->sDuct;
                    }
                }
            case 1:
                return true;
            case 2:
                if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                    if (litA->sDuct == litB->sDuct) {
                        return litA->RelationValue() < litB->RelationValue();
                    } else {
                        switch (StrategyParam::candLiteralDeductTimes)
                        {
                            case 0:
                                return litA->sDuct < litB->sDuct;
                            case 1:
                                return true;
                            case 2:
                                return litA->sDuct > litB->sDuct;
                        }
                    }
                } else {
                    switch (StrategyParam::candLiteralSteady)
                    {
                        case 0:
                            return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                        case 1:
                            return true;
                        case 2:
                            return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                    }
                }
        }
        return true;
    }
    static bool claSingCmp(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitSteady) {
                                case 0:
                                    return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitSteady) {
                                case 0:
                                    return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->uFuncDepth == c2->uFuncDepth) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitFuncLayer) {
                                case 0:
                                    return c1->uFuncDepth > c2->uFuncDepth;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->uFuncDepth < c2->uFuncDepth;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->uFuncDepth == c2->uFuncDepth) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitFuncLayer) {
                                case 0:
                                    return c1->uFuncDepth > c2->uFuncDepth;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->uFuncDepth < c2->uFuncDepth;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }
    static bool claSingCmpPairSizeDec(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->LitPtr[0]->pairLitSize()== c2->LitPtr[0]->pairLitSize()) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->pairLitSize() > c2->LitPtr[0]->pairLitSize();
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->LitPtr[0]->pairLitSize() == c2->LitPtr[0]->pairLitSize()) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->pairLitSize() > c2->LitPtr[0]->pairLitSize();
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->LitPtr[0]->pairLitSize() == c2->LitPtr[0]->pairLitSize()) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->pairLitSize() > c2->LitPtr[0]->pairLitSize();
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->LitPtr[0]->pairLitSize() == c2->LitPtr[0]->pairLitSize()) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {

                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->pairLitSize() > c2->LitPtr[0]->pairLitSize();
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }
    static bool claSingCmpVarSzie(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->uVarCount== c2->uVarCount) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->uVarCount < c2->uVarCount;
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->uVarCount == c2->uVarCount) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->uVarCount < c2->uVarCount;
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->uVarCount == c2->uVarCount) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->uVarCount < c2->uVarCount;
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->uVarCount == c2->uVarCount) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->uVarCount < c2->uVarCount;
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }
    
    static bool claSingCmpGenCla(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        return c1->uClaId > c2->uClaId;
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        return c1->uClaId > c2->uClaId;
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                         return c1->uClaId > c2->uClaId;            
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        return c1->uClaId > c2->uClaId;
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }
    static bool claSingCmpVarSzieDec(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->uVarCount== c2->uVarCount) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->uVarCount > c2->uVarCount;
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->uVarCount == c2->uVarCount) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->uVarCount > c2->uVarCount;
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->uVarCount == c2->uVarCount) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->uVarCount > c2->uVarCount;
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->uVarCount == c2->uVarCount) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->uVarCount > c2->uVarCount;
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }
    
    static bool claSingCmpPairSizeInc(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->LitPtr[0]->pairLitSize()== c2->LitPtr[0]->pairLitSize()) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->pairLitSize() < c2->LitPtr[0]->pairLitSize();
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->LitPtr[0]->pairLitSize() == c2->LitPtr[0]->pairLitSize()) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->pairLitSize() < c2->LitPtr[0]->pairLitSize();
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->LitPtr[0]->pairLitSize() == c2->LitPtr[0]->pairLitSize()) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->pairLitSize() < c2->LitPtr[0]->pairLitSize();
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->LitPtr[0]->pairLitSize() == c2->LitPtr[0]->pairLitSize()) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->pairLitSize() < c2->LitPtr[0]->pairLitSize();
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }
    
    static bool claSingCmpComplex(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->compFuncDeepthCla() == c2->compFuncDeepthCla()) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->compFuncDeepthCla() < c2->compFuncDeepthCla();
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->compFuncDeepthCla() == c2->compFuncDeepthCla()) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->compFuncDeepthCla() < c2->compFuncDeepthCla();
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->compFuncDeepthCla() == c2->compFuncDeepthCla()) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->compFuncDeepthCla() < c2->compFuncDeepthCla();
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->compFuncDeepthCla() == c2->compFuncDeepthCla()) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {

                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->compFuncDeepthCla() < c2->compFuncDeepthCla();
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }

    static bool claSingCmpGoal(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->LitPtr[0]->IsNeLit() == c2->LitPtr[0]->IsNeLit()) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->IsNeLit() < c2->LitPtr[0]->IsNeLit();
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->LitPtr[0]->IsNeLit() == c2->LitPtr[0]->IsNeLit()) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->IsNeLit() < c2->LitPtr[0]->IsNeLit();
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->adeductNum == c2->adeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->LitPtr[0]->IsNeLit() == c2->LitPtr[0]->IsNeLit()) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->IsNeLit() < c2->LitPtr[0]->IsNeLit();
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::unitDeduct) {
                        case 0:
                            return c1->adeductNum < c2->adeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->adeductNum > c2->adeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->LitPtr[0]->IsNeLit() == c2->LitPtr[0]->IsNeLit()) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            return c1->LitPtr[0]->IsNeLit() < c2->LitPtr[0]->IsNeLit();
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::unitRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }
    
    static bool claSingGenCmp(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->compFuncDeepthCla() == c2->compFuncDeepthCla()) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                }
                else
                    return c1->compFuncDeepthCla() < c2->compFuncDeepthCla();
                
            case 1:
                if (c1->compFuncDeepthCla() == c2->compFuncDeepthCla()) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->adeductNum == c2->adeductNum) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitDeduct) {
                                case 0:
                                    return c1->adeductNum < c2->adeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->adeductNum > c2->adeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else
                    return c1->compFuncDeepthCla() < c2->compFuncDeepthCla();
               
            case 2:
                 if (c1->compFuncDeepthCla() == c2->compFuncDeepthCla()) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                }
                else
                    return c1->compFuncDeepthCla() < c2->compFuncDeepthCla();
                
            case 3:
                if (c1->compFuncDeepthCla() == c2->compFuncDeepthCla()) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->adeductNum == c2->adeductNum) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitDeduct) {
                                case 0:
                                    return c1->adeductNum < c2->adeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->adeductNum > c2->adeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                }
                else
                    return c1->compFuncDeepthCla() < c2->compFuncDeepthCla();               
            case 4:
                return true;
        }
    }
    
    static bool claSingGenCmpNeg(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::unitChoose) {
            case 0:
                if (c1->LitPtr[0]->goalDeepth == c2->LitPtr[0]->goalDeepth) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else
                    return c1->LitPtr[0]->goalDeepth < c2->LitPtr[0]->goalDeepth;

            case 1:
                if (c1->LitPtr[0]->goalDeepth == c2->LitPtr[0]->goalDeepth) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->adeductNum == c2->adeductNum) {
                            if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                if (c1->uFuncDepth == c2->uFuncDepth) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitFuncLayer) {
                                        case 0:
                                            return c1->uFuncDepth > c2->uFuncDepth;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uFuncDepth < c2->uFuncDepth;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitSteady) {
                                    case 0:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitDeduct) {
                                case 0:
                                    return c1->adeductNum < c2->adeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->adeductNum > c2->adeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else
                    return c1->LitPtr[0]->goalDeepth < c2->LitPtr[0]->goalDeepth;

            case 2:
                if (c1->LitPtr[0]->goalDeepth == c2->LitPtr[0]->goalDeepth) {
                    if (c1->adeductNum == c2->adeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitDeduct) {
                            case 0:
                                return c1->adeductNum < c2->adeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->adeductNum > c2->adeductNum;
                        }
                    }
                } else
                    return c1->LitPtr[0]->goalDeepth < c2->LitPtr[0]->goalDeepth;

            case 3:
                if (c1->LitPtr[0]->goalDeepth == c2->LitPtr[0]->goalDeepth) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->adeductNum == c2->adeductNum) {
                            if (c1->uFuncDepth == c2->uFuncDepth) {
                                if (c1->LitPtr[0]->subTerm->fSteadyValue == c2->LitPtr[0]->subTerm->fSteadyValue) {
                                    return c1->LitPtr[0]->subTerm->RelationValue() < c2->LitPtr[0]->subTerm->RelationValue();
                                } else {

                                    switch (StrategyParam::unitSteady) {
                                        case 0:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue > c2->LitPtr[0]->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->LitPtr[0]->subTerm->fSteadyValue < c2->LitPtr[0]->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::unitFuncLayer) {
                                    case 0:
                                        return c1->uFuncDepth > c2->uFuncDepth;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uFuncDepth < c2->uFuncDepth;
                                }
                            }
                        } else {
                            switch (StrategyParam::unitDeduct) {
                                case 0:
                                    return c1->adeductNum < c2->adeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->adeductNum > c2->adeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::unitRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else
                    return c1->LitPtr[0]->goalDeepth < c2->LitPtr[0]->goalDeepth;

            case 4:
                return true;
        }
    }

    static int unitDeduct(const Literal*litA, const Literal*litB) {

    }

    static bool claSingCmpRand(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        if (c1->adeductNum == c2->adeductNum) {
            if (c1->MaxVarId() == c2->MaxVarId()) {
                if (c1->uFuncDepth == c2->uFuncDepth)
                    return c1->uClaId > c2->uClaId;
                return c1->uFuncDepth < c2->uFuncDepth;
            }
            return c1->MaxVarId() < c2->MaxVarId();
        }
        return c1->adeductNum < c2->adeductNum;
    }

    static bool PoslitCmpLarge(const Literal*litA, const Literal*litB) {
        UINT16 iClaASize = UINT16(litA->claPtr->uLitNum); 
        UINT16 iClaBSize = UINT16(litB->claPtr->uLitNum); 
        UINT32 iClaANum = litA->claPtr->pdeductNum; 
        UINT32 iClaBNum = litB->claPtr->pdeductNum; 
        if (iClaANum == iClaBNum)
            return iClaASize < iClaBSize;
        else
            return iClaANum < iClaBNum;
    }

    static bool PoslitCmpLargeRand(const Literal*litA, const Literal*litB) {
        UINT16 iClaASize = UINT16(litA->claPtr->uLitNum);
        UINT16 iClaBSize = UINT16(litB->claPtr->uLitNum);
        UINT32 iClaANum = litA->claPtr->pdeductNum;
        UINT32 iClaBNum = litB->claPtr->pdeductNum;
        if (iClaANum > 10 && iClaBNum > 10)
            return int(rand()) % 2 != int(rand()) % 2; 
        else if (iClaANum > 0 || iClaBNum > 0)
            return iClaANum < iClaBNum;
        else
            return iClaASize<iClaBSize;
    }

    static bool PoslitCmpLR(const Clause* c1, const Clause* c2) {
        switch (StrategyParam::candChoose) {
            case 0:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->pdeductNum== c2->pdeductNum) {
                         if (c1->reduntTimes  == c2->reduntTimes)
                        {
                             return c1->uVarCount < c2->uVarCount;
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }    
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }
            case 1:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            return c1->uVarCount < c2->uVarCount;
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }    
            case 2:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->pdeductNum== c2->pdeductNum) {
                         if (c1->reduntTimes  == c2->reduntTimes)
                        {
                             return c1->uFuncDepth < c2->uFuncDepth;
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }    
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }
            case 3:
              if (c1->uLitNum == c2->uLitNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            return c1->uFuncDepth < c2->uFuncDepth;
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }    
            case 4:
                return true;
        }
    }

    static bool PoslitCmp(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::candChoose) {
            case 0:
                if (litA->sDuct == litB->sDuct) {
                    if (c1->pdeductNum == c2->pdeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->uLitNum == c2->uLitNum) {
                                if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                    return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::candSteady) {
                                        case 0:
                                            return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candClause) {
                                    case 0:
                                        return c1->uLitNum < c2->uLitNum;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uLitNum > c2->uLitNum;
                                }
                            }
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    return litA->sDuct < litB->sDuct;
                }
            case 1:
                if (litA->sDuct == litB->sDuct) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            if (c1->uLitNum == c2->uLitNum) {
                                if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                    return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::candSteady) {
                                        case 0:
                                            return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candClause) {
                                    case 0:
                                        return c1->uLitNum < c2->uLitNum;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uLitNum > c2->uLitNum;
                                }
                            }
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    return litA->sDuct < litB->sDuct;
                }

            case 2:
                if (litA->sDuct == litB->sDuct) {
                    if (c1->pdeductNum == c2->pdeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                if (c1->uLitNum == c2->uLitNum) {
                                    return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::candClause) {
                                        case 0:
                                            return c1->uLitNum < c2->uLitNum;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uLitNum > c2->uLitNum;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candSteady) {
                                    case 0:
                                        return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    return litA->sDuct < litB->sDuct;
                }

            case 3:
                if (litA->sDuct == litB->sDuct) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                if (c1->uLitNum == c2->uLitNum) {
                                    return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::candClause) {
                                        case 0:
                                            return c1->uLitNum < c2->uLitNum;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uLitNum > c2->uLitNum;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candSteady) {
                                    case 0:
                                        return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    return litA->sDuct < litB->sDuct;
                }
            case 4:
                return true;
        }
    }

    static bool PoslitCmpPriorLR(const Clause* c1, const Clause* c2) {
        switch (StrategyParam::candChoose) {
            case 0:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->pdeductNum == c2->pdeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->uLitNum == c2->uLitNum) {
                                if (StrategyParam::unitSortRule == 3) {
                                    return c1->goaldeepth < c2->goaldeepth;
                                } else if (StrategyParam::unitSortRule == 4) {
                                    return c1->maxPairPred < c2->maxPairPred;
                                } else if (StrategyParam::unitSortRule == 5) {
                                    return c1->maxPairPred > c2->maxPairPred;
                                } else if (StrategyParam::unitSortRule == 6) {
                                    return c1->uVarCount < c2->uVarCount;
                                } else if (StrategyParam::unitSortRule == 7) {
                                    return c1->uVarCount > c2->uVarCount;
                                } else if (StrategyParam::unitSortRule == 8) {
                                    return c1->uClaId > c2->uClaId;
                                } else
                                    return c1->termCount21 < c2->termCount21;
                            } else {
                                switch (StrategyParam::candClause) {
                                    case 0:
                                        return c1->uLitNum < c2->uLitNum;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uLitNum > c2->uLitNum;
                                }
                            }
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return c1->uClaId < c2->uClaId;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }
            case 1:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            if (c1->uLitNum == c2->uLitNum) {
                                if (StrategyParam::unitSortRule == 3) {
                                    return c1->goaldeepth < c2->goaldeepth;
                                } else if (StrategyParam::unitSortRule == 4) {
                                    return c1->maxPairPred < c2->maxPairPred;
                                } else if (StrategyParam::unitSortRule == 5) {
                                    return c1->maxPairPred > c2->maxPairPred;
                                } else if (StrategyParam::unitSortRule == 6) {
                                    return c1->uVarCount < c2->uVarCount;
                                } else if (StrategyParam::unitSortRule == 7) {
                                    return c1->uVarCount > c2->uVarCount;
                                } else if (StrategyParam::unitSortRule == 8) {
                                    return c1->uClaId > c2->uClaId;
                                } else
                                    return c1->termCount21 < c2->termCount21;
                            } else {
                                switch (StrategyParam::candClause) {
                                    case 0:
                                        return c1->uLitNum < c2->uLitNum;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uLitNum > c2->uLitNum;
                                }
                            }
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return c1->uClaId < c2->uClaId;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }

            case 2:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->pdeductNum == c2->pdeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (StrategyParam::unitSortRule == 3) {
                                return c1->goaldeepth < c2->goaldeepth;
                            } else if (StrategyParam::unitSortRule == 4) {
                                return c1->maxPairPred < c2->maxPairPred;
                            } else if (StrategyParam::unitSortRule == 5) {
                                return c1->maxPairPred > c2->maxPairPred;
                            } else if (StrategyParam::unitSortRule == 6) {
                                return c1->uVarCount < c2->uVarCount;
                            } else if (StrategyParam::unitSortRule == 7) {
                                return c1->uVarCount > c2->uVarCount;
                            } else if (StrategyParam::unitSortRule == 8) {
                                return c1->uClaId > c2->uClaId;
                            } else
                                return c1->termCount21 < c2->termCount21;
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return c1->uClaId < c2->uClaId;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }

            case 3:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            if (StrategyParam::unitSortRule == 3) {
                                return c1->goaldeepth < c2->goaldeepth;
                            } else if (StrategyParam::unitSortRule == 4) {
                                return c1->maxPairPred < c2->maxPairPred;
                            } else if (StrategyParam::unitSortRule == 5) {
                                return c1->maxPairPred > c2->maxPairPred;
                            } else if (StrategyParam::unitSortRule == 6) {
                                return c1->uVarCount < c2->uVarCount;
                            } else if (StrategyParam::unitSortRule == 7) {
                                return c1->uVarCount > c2->uVarCount;
                            } else if (StrategyParam::unitSortRule == 8) {
                                return c1->uClaId > c2->uClaId;
                            } else
                                return c1->termCount21 < c2->termCount21;
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return c1->uClaId < c2->uClaId;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }
            case 4:
                return true;
        }
    }

    static bool PoslitCmpPrior(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::candChoose) {
            case 0:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->pdeductNum == c2->pdeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->uLitNum == c2->uLitNum) {
                                if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                    return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::candSteady) {
                                        case 0:
                                            return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candClause) {
                                    case 0:
                                        return c1->uLitNum < c2->uLitNum;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uLitNum > c2->uLitNum;
                                }
                            }
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return c1->uClaId > c2->uClaId;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }
            case 1:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            if (c1->uLitNum == c2->uLitNum) {
                                if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                    return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::candSteady) {
                                        case 0:
                                            return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candClause) {
                                    case 0:
                                        return c1->uLitNum < c2->uLitNum;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->uLitNum > c2->uLitNum;
                                }
                            }
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return c1->uClaId > c2->uClaId;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }

            case 2:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->pdeductNum == c2->pdeductNum) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                if (c1->uLitNum == c2->uLitNum) {
                                    return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::candClause) {
                                        case 0:
                                            return c1->uLitNum < c2->uLitNum;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uLitNum > c2->uLitNum;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candSteady) {
                                    case 0:
                                        return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return c1->uClaId > c2->uClaId;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }

            case 3:
                if (c1->uLitNum == c2->uLitNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                if (c1->uLitNum == c2->uLitNum) {
                                    return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                } else {
                                    switch (StrategyParam::candClause) {
                                        case 0:
                                            return c1->uLitNum < c2->uLitNum;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uLitNum > c2->uLitNum;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candSteady) {
                                    case 0:
                                        return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                    case 1:
                                        return true;
                                    case 2:
                                        return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                }
                            }
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return c1->uClaId > c2->uClaId;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    return c1->uLitNum < c2->uLitNum;
                }
            case 4:
                return true;
        }
    }

    static bool PoslitCmpProRL(const Clause* c1, const Clause* c2) {
        switch (StrategyParam::candChoose) {
            case 0:
                if (c1->pdeductNum == c2->pdeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->uLitNum == c2->uLitNum) {
                            return c1->hasUsed() <= c2->hasUsed();
                        } else {
                            return c1->uLitNum < c2->uLitNum;
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::candDeduct) {
                        case 0:
                            return c1->pdeductNum < c2->pdeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->pdeductNum > c2->pdeductNum;
                    }
                }
            case 1:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->pdeductNum == c2->pdeductNum) {
                        if (c1->uLitNum == c2->uLitNum) {
                            return c1->hasUsed() <= c2->hasUsed();
                        } else {
                            return c1->uLitNum < c2->uLitNum;
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::candRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 2:
                if (c1->pdeductNum == c2->pdeductNum) {
                    if (c1->reduntTimes == c2->reduntTimes) {
                        if (c1->uLitNum == c2->uLitNum) {
                            return c1->hasUsed() <= c2->hasUsed();
                        } else {
                            return c1->uLitNum < c2->uLitNum;
                        }
                    } else {
                        switch (StrategyParam::candRedunt) {
                            case 0:
                                return c1->reduntTimes < c2->reduntTimes;
                            case 1:
                                return true;
                            case 2:
                                return c1->reduntTimes > c2->reduntTimes;
                        }
                    }
                } else {
                    switch (StrategyParam::candDeduct) {
                        case 0:
                            return c1->pdeductNum < c2->pdeductNum;
                        case 1:
                            return true;
                        case 2:
                            return c1->pdeductNum > c2->pdeductNum;
                    }
                }
            case 3:
                if (c1->reduntTimes == c2->reduntTimes) {
                    if (c1->pdeductNum == c2->pdeductNum) {
                        if (c1->uLitNum == c2->uLitNum) {
                            return c1->hasUsed() <= c2->hasUsed();
                        } else {
                            return c1->uLitNum < c2->uLitNum;
                        }
                    } else {
                        switch (StrategyParam::candDeduct) {
                            case 0:
                                return c1->pdeductNum < c2->pdeductNum;
                            case 1:
                                return true;
                            case 2:
                                return c1->pdeductNum > c2->pdeductNum;
                        }
                    }
                } else {
                    switch (StrategyParam::candRedunt) {
                        case 0:
                            return c1->reduntTimes < c2->reduntTimes;
                        case 1:
                            return true;
                        case 2:
                            return c1->reduntTimes > c2->reduntTimes;
                    }
                }
            case 4:
                return true;
        }
    }
    
    static bool PoslitCmpGoalDeepth(const Literal*litA, const Literal*litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        switch (StrategyParam::candChoose) {
            case 0:
                if (litA->sDuct == litB->sDuct) {
                    if (litA->goalDeepth == litB->goalDeepth) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            if (c1->reduntTimes == c2->reduntTimes) {
                                if (c1->uLitNum == c2->uLitNum) {
                                    if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                        return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                    } else {
                                        switch (StrategyParam::candSteady) {
                                            case 0:
                                                return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                            case 1:
                                                return true;
                                            case 2:
                                                return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                        }
                                    }
                                } else {
                                    switch (StrategyParam::candClause) {
                                        case 0:
                                            return c1->uLitNum < c2->uLitNum;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uLitNum > c2->uLitNum;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candRedunt) {
                                    case 0:
                                        return c1->reduntTimes < c2->reduntTimes;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->reduntTimes > c2->reduntTimes;
                                }
                            }
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        return litA->goalDeepth < litB->goalDeepth;
                    }
                } else {
                    return litA->sDuct < litB->sDuct;
                }
            case 1:
                if (litA->sDuct == litB->sDuct) {
                    if (litA->goalDeepth == litB->goalDeepth) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->pdeductNum == c2->pdeductNum) {
                                if (c1->uLitNum == c2->uLitNum) {
                                    if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                        return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                    } else {
                                        switch (StrategyParam::candSteady) {
                                            case 0:
                                                return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                            case 1:
                                                return true;
                                            case 2:
                                                return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                        }
                                    }
                                } else {
                                    switch (StrategyParam::candClause) {
                                        case 0:
                                            return c1->uLitNum < c2->uLitNum;
                                        case 1:
                                            return true;
                                        case 2:
                                            return c1->uLitNum > c2->uLitNum;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candDeduct) {
                                    case 0:
                                        return c1->pdeductNum < c2->pdeductNum;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->pdeductNum > c2->pdeductNum;
                                }
                            }
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        return litA->goalDeepth < litB->goalDeepth;
                    }
                } else {
                    return litA->sDuct < litB->sDuct;
                }

            case 2:
                if (litA->sDuct == litB->sDuct) {
                    if (litA->goalDeepth == litB->goalDeepth) {
                        if (c1->pdeductNum == c2->pdeductNum) {
                            if (c1->reduntTimes == c2->reduntTimes) {
                                if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                    if (c1->uLitNum == c2->uLitNum) {
                                        return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                    } else {
                                        switch (StrategyParam::candClause) {
                                            case 0:
                                                return c1->uLitNum < c2->uLitNum;
                                            case 1:
                                                return true;
                                            case 2:
                                                return c1->uLitNum > c2->uLitNum;
                                        }
                                    }
                                } else {
                                    switch (StrategyParam::candSteady) {
                                        case 0:
                                            return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candRedunt) {
                                    case 0:
                                        return c1->reduntTimes < c2->reduntTimes;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->reduntTimes > c2->reduntTimes;
                                }
                            }
                        } else {
                            switch (StrategyParam::candDeduct) {
                                case 0:
                                    return c1->pdeductNum < c2->pdeductNum;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->pdeductNum > c2->pdeductNum;
                            }
                        }
                    } else {
                        return litA->goalDeepth < litB->goalDeepth;
                    }
                } else {
                    return litA->sDuct < litB->sDuct;
                }
            case 3:
                if (litA->sDuct == litB->sDuct) {
                    if (litA->goalDeepth == litB->goalDeepth) {
                        if (c1->reduntTimes == c2->reduntTimes) {
                            if (c1->pdeductNum == c2->pdeductNum) {
                                if (litA->subTerm->fSteadyValue == litB->subTerm->fSteadyValue) {
                                    if (c1->uLitNum == c2->uLitNum) {
                                        return litA->subTerm->RelationValue() < litB->subTerm->RelationValue();
                                    } else {
                                        switch (StrategyParam::candClause) {
                                            case 0:
                                                return c1->uLitNum < c2->uLitNum;
                                            case 1:
                                                return true;
                                            case 2:
                                                return c1->uLitNum > c2->uLitNum;
                                        }
                                    }
                                } else {
                                    switch (StrategyParam::candSteady) {
                                        case 0:
                                            return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue;
                                        case 1:
                                            return true;
                                        case 2:
                                            return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue;
                                    }
                                }
                            } else {
                                switch (StrategyParam::candDeduct) {
                                    case 0:
                                        return c1->pdeductNum < c2->pdeductNum;
                                    case 1:
                                        return true;
                                    case 2:
                                        return c1->pdeductNum > c2->pdeductNum;
                                }
                            }
                        } else {
                            switch (StrategyParam::candRedunt) {
                                case 0:
                                    return c1->reduntTimes < c2->reduntTimes;
                                case 1:
                                    return true;
                                case 2:
                                    return c1->reduntTimes > c2->reduntTimes;
                            }
                        }
                    } else {
                        return litA->goalDeepth < litB->goalDeepth;
                    }
                } else {
                    return litA->sDuct < litB->sDuct;
                }
            case 4:
                return true;
        }
    }

    inline static bool LitPtrCmp(const Literal* litA, const Literal* litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        if (c1->uLitNum == c2->uLitNum) {
            return (litA->subTerm->uMaxVarId / double(litA->subTerm->uSubTermNum)) < litB->subTerm->uMaxVarId / double(litB->subTerm->uSubTermNum);
        }
        return c1->uLitNum < c2->uLitNum;
    }
    inline static bool LitPtrCmpLarge(const Literal* litA, const Literal* litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        if (c1->uLitNum == c2->uLitNum) {
            return (litA->subTerm->uMaxVarId / double(litA->subTerm->uSubTermNum)) > litB->subTerm->uMaxVarId / double(litB->subTerm->uSubTermNum);
        }
        return c1->uLitNum < c2->uLitNum;
    }
    inline static bool claTwoCmp(const Literal* litA, const Literal* litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        if (c1->uLitNum == c2->uLitNum) {
            if(c1->noUnitDeductTimes == c2->noUnitDeductTimes)
                return c1->uClaId < c2->uClaId ;
            else
                return c1->noUnitDeductTimes < c2->noUnitDeductTimes;

        }
        return c1->uLitNum < c2->uLitNum;
    }
    
    inline static bool claTwoCmp_last(const Literal* litA, const Literal* litB) {
        Clause* c1 = litA->claPtr;
        Clause* c2 = litB->claPtr;
        if (c1->noUnitDeductTimes == c2->noUnitDeductTimes) {      
            if(c1->uLitNum == c2->uLitNum)
                return c1->uVarCount < c2->uVarCount ;
            else
                return c1->uLitNum < c2->uLitNum;

        }
        return c1->noUnitDeductTimes < c2->noUnitDeductTimes;
    }
    inline static bool LitCmp(const Literal* litA, const Literal* litB) {

        return 1 == TermSteadyCmp(litA->subTerm, litB->subTerm, 0);

    }
   inline static bool LitCmp2(const Literal* litA, const Literal* litB) {

        return 1 == TermSteadyCmp(litB->subTerm, litA->subTerm, 0);

    }
    inline static bool LitCmpGoalDeepth(const Literal* litA, const Literal* litB) {

        return 1 == TermSteadyCmpGoalDeepth(litA, litB, 0);

    }
    inline static bool LitCmpDuct(const Literal* litA, const Literal* litB) {
        if (litA->sDuct == litB->sDuct)

            return 1 == TermSteadyCmp(litA->subTerm, litB->subTerm, 0);
        else
            return litA->sDuct < litB->sDuct;

    }
    static int TermSteadyCmp(Term* litA, Term* litB, int iCmpType) {
        switch (iCmpType) {
            case -1:
                return 0;
            case 0:
                
                if (litA->fSteadyValue != litB->fSteadyValue)
                    return litA->fSteadyValue < litB->fSteadyValue ? 1 : -1;
                return litA->RelationValue() < litB->RelationValue() ? 1 : -1;
                
            case 1:
                if (litA->fSteadyValue != litB->fSteadyValue)
                    return litA->fSteadyValue > litB->fSteadyValue ? 1 : -1;
                return litA->RelationValue() > litB->RelationValue() ? 1 : -1;
        }
        return 0;
    }
    
    static int TermSteadyCmpGoalDeepth(const Literal* litA, const Literal* litB, int iCmpType) {
        switch (iCmpType) {
            case -1:
                return 0;
            case 0:
                if(litA->goalDeepth == litB->goalDeepth){
                if (litA->subTerm->fSteadyValue != litB->subTerm->fSteadyValue)
                    return litA->subTerm->fSteadyValue < litB->subTerm->fSteadyValue ? 1 : -1;
                return litA->subTerm->RelationValue() < litB->subTerm->RelationValue() ? 1 : -1;
                }
                else{
                    return litA->goalDeepth < litB->goalDeepth;
                }
            case 1:
                if (litA->subTerm->fSteadyValue != litB->subTerm->fSteadyValue)
                    return litA->subTerm->fSteadyValue > litB->subTerm->fSteadyValue ? 1 : -1;
                return litA->subTerm->RelationValue() > litB->subTerm->RelationValue() ? 1 : -1;
        }
        return 0;
    }
    SortRule();
    SortRule(const SortRule& orig);
    virtual ~SortRule();
private:

};

#endif /* SORTRULE_H */

