/*
*  Copyright 2022 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#include "Resolution.h"
#include "Literal.h"
#include "RedundancyOp.h"
extern map<INT32, vector<INT32>> validPathOut;
Resolution::Resolution() {
    runNum = 0;
    StartId = 0;
    shift = 0;
    ringCount =0;
    startClause = nullptr;
    varBindTerm.clear();
    varBindTerm.reserve(8);
    lockLitVec.clear();
    lockLitVec.reserve(6);
    lockClaVec.clear();
    lockClaVec.reserve(6);
    triAlgTimes = 16;
    difficulty = false;
    noPairInd = 0;
    noPairCla = nullptr;
}

Resolution::Resolution(const Resolution& orig) {
}

RESULT Resolution::InsertSingleSet(Formula& fol,TriAlg& triAlg,int& maxUintID)
{
    maxUintID = 0;
    if(fol.setCla.size()<1)
    {
        return fail;
    }
    for(auto& clapE:fol.setEnq)
    {
        clapE->isSDeduct = false;
        assert(clapE!=NULL);
    }
    for(auto& clap:fol.setCla)
{
        clap->isSDeduct = false;
        assert(clap != NULL);
        if ((triAlg.noCmpCla.find(clap->uClaId) == triAlg.noCmpCla.end()) && (clap->compFuncDeepthCla() <= StrategyParam::funcFixDeepth)) {//noCmpCla用于保存没有互补对的单文字子句
            fol.setSingleCla.insert(clap);
            ++maxUintID;
        }
    }
    return Succes;
}

RESULT Resolution::InsertSingleSetSaturate(Formula& fol,TriAlg& triAlg)
{
    for (auto& clap : fol.setDeductCla) {
        clap->isSDeduct = false; 
        assert(clap != NULL);
        if (clap->uLitNum == 1) {
            if ((triAlg.noCmpCla.find(clap->uClaId) == triAlg.noCmpCla.end()) && (clap->compFuncDeepthCla() <= StrategyParam::funcFixDeepth)) {//noCmpCla用于保存没有互补对的单文字子句
                fol.setSingleCla.insert(clap);
            }
        }
    }
    return Succes;
}

RESULT Resolution::InsertSingleCla(Formula& fol,Clause* cla)
{
    for(auto& clap:fol.setSingleCla)
    {
        if(clap == cla)
        {
            return fail;
        }
    }
    fol.setSingleCla.insert(cla);
    return Succes;
}

void Resolution::shiftSingleCla(vector<Literal*>& orgSingleCla,UINT32 N)
{
    UINT32 size_Tri = orgSingleCla.size();
    if(size_Tri < 2 || N< 1)
        return ;
    if(StrategyParam::unitClaUpdate == 0){
        while (N--) {
            Literal* litT = orgSingleCla[0];
            for (UINT32 i = 0; i < size_Tri - 1; i++) {
                orgSingleCla[i] = orgSingleCla[i + 1];
            }
            orgSingleCla[size_Tri - 1] = litT;
        }
    }
    else if(StrategyParam::unitClaUpdate == 1) {
        Literal* litT = orgSingleCla[0];
        orgSingleCla[0] = orgSingleCla[N];
        orgSingleCla[N] = litT;
    }
    else{
        Literal* litT = orgSingleCla[0];
        orgSingleCla[0] = orgSingleCla[size_Tri-1];
        orgSingleCla[size_Tri-1] = litT;
        litT = orgSingleCla[0];
        orgSingleCla[0] = orgSingleCla[N];
        orgSingleCla[N] = litT;
    }
}

RESULT Resolution::InsertOutTri(TriAlg& triAlg,Formula& fol,UINT16& loop,bool chooseTri)
{
    string str;
    vector<Literal*> singleUintLit;
    singleUintLit.reserve(16);
    for(auto& clap:fol.setSingleCla)
    {
        if(clap->uLitNum > 1)
            return fail;
    }
    if(singleUintLit.size() > 0){
        for(UINT32 id = 0; id < singleUintLit.size(); ++ id){
            triAlg.vOnlyOutTriCmp.push_back(singleUintLit[id]);
        }
    }
    singleUintLit.clear();
    vector<Literal*>().swap(singleUintLit);
    UINT32 size_Tri = triAlg.vOnlyOutTriCmp.size();
    if (shift >= size_Tri){
            shift = 0;
            ringCount++;
            if(ringCount %10)
                triAlg.noCmpCla.clear();
            if(ringCount == StrategyParam::iterRing/100){
                StrategyParam::R_MAX_NUM += 1;
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/50){
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/25){
                StrategyParam::R_MAX_NUM += 1;
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/10){
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/5){
                StrategyParam::R_MAX_NUM += 1;
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/2){
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
    }
    if(loop == 0) { 
        shift++;
        if (shift >= size_Tri)
            shift = 0;
        triAlg.flagT = true;  
    }
    UINT32 conClaPair = 0;
    UINT32 countUnit = 0;
    do{
        shiftSingleCla(triAlg.vOnlyOutTriCmp, shift);
        if(triAlg.vOnlyOutTriCmp[0]->isConLit() &&triAlg.flagT){
            conClaPair = triAlg.realUnifyCounts(fol, triAlg.vOnlyOutTriCmp[0]->claPtr);
            if (conClaPair == 0) {
                if(triAlg.noCmpCla.find(triAlg.vOnlyOutTriCmp[0]->claPtr->uClaId) == triAlg.noCmpCla.end())
                    triAlg.noCmpCla.insert(triAlg.vOnlyOutTriCmp[0]->claPtr->uClaId);
                stable_sort(triAlg.vOnlyOutTriCmp.begin() , triAlg.vOnlyOutTriCmp.end(), SortRule::singleResetId); 
                ++shift;
                ++countUnit;
                if (shift >= size_Tri){
                    shift = 0;
                    if(countUnit == size_Tri)
                    {
                        break;
                    }
                }
            }
        }
    }while(triAlg.vOnlyOutTriCmp[0]->isConLit() && (conClaPair == 0) &&triAlg.flagT);
    startClause = triAlg.vOnlyOutTriCmp[0]->claPtr;
    if(triAlg.flagT)
    {
        for (UINT32 i = 0; i < triAlg.vOnlyOutTriCmp.size(); ++i) {
            UINT32 len = triAlg.vOnlyOutTriCmp[i]->getPairPredLst()->size();
            Literal* candLitP;
            Clause* candClaP;
            vector<Literal*>* vCandLit = triAlg.vOnlyOutTriCmp[i]->getPairPredLst();
            for (UINT32 pasLitInd = 0; pasLitInd < len; ++pasLitInd)
            {
                candLitP = vCandLit->at(pasLitInd); 
                candLitP->sDuct = candLitP->sDuct%50;
                candClaP = candLitP->claPtr; 
                candClaP->pdeductNum = candClaP->pdeductNum%30;
            }
        }
        if(StrategyParam::iterTimes == -1){
            if(triAlg.vOnlyOutTriCmp[0]->isConLit())
                loop = conClaPair;
            else
                loop = triAlg.vOnlyOutTriCmp[0]->getPairPredLst()->size();
            triAlg.total_Loop = loop;
        }
        else{
            loop = StrategyParam::iterTimes;
            triAlg.total_Loop = loop;
        }
        triAlg.flagT = false;
    }
    if ((triAlg.vOnlyOutTriCmp.size() > 2)) {
        if(StrategyParam::useSaturate){
            stable_sort(triAlg.vOnlyOutTriCmp.begin()+1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmp);
        }
        else{
            if (StrategyParam::unitSortRule == 0) {
                if(fol.uOrigClaSingleSize == 0){
                    stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.begin() + triAlg.vOnlyOutTriCmp.size(), SortRule::claSingCmp); 
                }
                else
                    stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.begin() + fol.uOrigClaSingleSize, SortRule::claSingCmp); 
            } else if (StrategyParam::unitSortRule == 1) {
                if(fol.uOrigClaSingleSize == 0){
                    stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.begin() + triAlg.vOnlyOutTriCmp.size(), SortRule::claSingCmp); 
                }
                else
                    stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.begin() + fol.uOrigClaSingleSize, SortRule::claSingCmp); 
                if (triAlg.vOnlyOutTriCmp.size() > fol.uOrigClaSingleSize)
                    stable_sort(triAlg.vOnlyOutTriCmp.begin() + fol.uOrigClaSingleSize, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmp); 
            } else if (StrategyParam::unitSortRule == 2) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpComplex); 
            } else  if (StrategyParam::unitSortRule == 3){
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingGenCmpNeg); 
            }else if (StrategyParam::unitSortRule == 4) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpPairSizeInc); 
            }else if (StrategyParam::unitSortRule == 5) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpPairSizeDec); 
            }else if (StrategyParam::unitSortRule == 6) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpVarSzie); 
            }else if (StrategyParam::unitSortRule == 7) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpVarSzieDec); 
            }
            else if (StrategyParam::unitSortRule == 8) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpGenCla);
            }
        }
    }
   if(chooseTri)
        loop--;
    if (StrategyParam::unitDeductTimesUpdate == 0) {
        for (UINT32 k = 0; k < size_Tri; k++) {
            if (StrategyParam::unitDeductTimes > 0)
            {
                if (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum > StrategyParam::unitDeductTimes) {
                    triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = 0;
                }
            }
            if (loop == 0) {
                triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum * 2) % StrategyParam::iterTimes;
            }
        }
    }
    else if(StrategyParam::unitDeductTimesUpdate == 1)
    {
        for (UINT32 k = 0; k < size_Tri; k++) {
                if (StrategyParam::unitDeductTimes > 0)
                {
                    if (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum > StrategyParam::unitDeductTimes) {
                        triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = 0;
                    }
                }
                 if (loop == 0)
                    triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum =  (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum * 3) % StrategyParam::iterTimes;
            }
    }
    else
    {
        for (UINT32 k = 0; k < size_Tri; k++) {
                if (StrategyParam::unitDeductTimes > 0)
                {
                    if (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum > StrategyParam::unitDeductTimes) {
                        triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = 0;
                    }
                }
                 if ((shift >= size_Tri -1) && (loop == 0))
                    triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum) % StrategyParam::iterTimes; 
            }
    }
    if (StrategyParam::unitReduntTimesUpdate == 0) {
        for (UINT32 k = 0; k < size_Tri; k++) {
            if (StrategyParam::unitReduntTimes > 0)
            {
                if (triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes > StrategyParam::unitReduntTimes) {
                    triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = 0;
                }
            }
            if (loop==0) {
                triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes % 200;
            }
        }
    }
    else if(StrategyParam::unitReduntTimesUpdate == 1)
    {
        for (UINT32 k = 0; k < size_Tri; k++) {
                if (StrategyParam::unitReduntTimes > 0)
                {
                    if (triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes > StrategyParam::unitReduntTimes) {
                        triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = 0;
                    }
                }
                 if (loop == 0)
                    triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes % 300;
            }
    }
    else
    {
        for (UINT32 k = 0; k < size_Tri; k++) {
                if (StrategyParam::unitReduntTimes > 0)
                {
                    if (triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes > StrategyParam::unitReduntTimes) {
                        triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = 0;
                    }
                }
                 if ((shift >= size_Tri -1) && (loop == 0))
                    triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes / 100; 
            }
    }
    if (StrategyParam::candDeductTimesUpdate == 0) {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if (loop == 0 ) {
                clapE->pdeductNum = (clapE->pdeductNum*2)%StrategyParam::iterTimes;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if (loop ==0 ) {
                clapE->pdeductNum = (clapE->pdeductNum*2)%StrategyParam::iterTimes;
            }
        }
    }
    else if (StrategyParam::candDeductTimesUpdate == 1) {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if (loop == 0) {
                clapE->pdeductNum = (clapE->pdeductNum*6)%StrategyParam::iterTimes;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if (loop == 0) {
                clapE->pdeductNum = (clapE->pdeductNum*6)%StrategyParam::iterTimes;
            }
        }
    }
    else
     {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if ((shift >= size_Tri -1) && (loop == 0)) {
                clapE->pdeductNum = clapE->pdeductNum %StrategyParam::iterTimes; 
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if ((shift >= size_Tri -1) && (loop == 0))  {
                clapE->pdeductNum = clapE->pdeductNum%StrategyParam::iterTimes;
            }
        }
    }
     if (StrategyParam::candReduntTimesUpdate == 0) {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if (loop == 0) {
                clapE->reduntTimes = clapE->reduntTimes%30;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
             if (loop ==0 ) {
                clapE->reduntTimes =clapE->reduntTimes%30;
            }
        }
    }
     else if (StrategyParam::candReduntTimesUpdate == 1) {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if (loop == 0) {
                clapE->reduntTimes = clapE->reduntTimes%15;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if (loop == 0) {
                clapE->reduntTimes = clapE->reduntTimes%15;
            }
        }
    }
    else
    {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if ((shift >= size_Tri -1) && (loop ==0)) {
                clapE->reduntTimes = clapE->reduntTimes %100;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if ((shift >= size_Tri -1) && (loop == 0)) {
                clapE->reduntTimes = clapE->reduntTimes %100; 
            }
        }
    }
    for (UINT32 k = 0; k < size_Tri; k++) {
        triAlg.vOnlyOutTri.push_back(triAlg.vOnlyOutTriCmp[k]);
        triAlg.vOutTri.push_back(triAlg.vOnlyOutTriCmp[k]);
    }
}

RESULT Resolution::InsertOutTriLR(TriAlg& triAlg,Formula& fol,UINT16& loop,bool chooseTri)
{
    string str;
    vector<Literal*> singleUintLit;
    singleUintLit.reserve(16);
    for(auto& clap:fol.setSingleCla)
    {
        if(clap->uLitNum > 1)
            return fail;
    }
    if(singleUintLit.size() > 0){
        for(UINT32 id = 0; id < singleUintLit.size(); ++ id){
            triAlg.vOnlyOutTriCmp.push_back(singleUintLit[id]);
        }
    }
    singleUintLit.clear();
    vector<Literal*>().swap(singleUintLit);
    UINT32 size_Tri = triAlg.vOnlyOutTriCmp.size();
    if (shift >= size_Tri){
            shift = 0;
            ringCount++;
            if(ringCount %10)
                triAlg.noCmpCla.clear();
            if(ringCount == StrategyParam::iterRing/100){
                StrategyParam::R_MAX_NUM += 1;
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/50){
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/25){
                StrategyParam::R_MAX_NUM += 1;
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/10){
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/5){
                StrategyParam::R_MAX_NUM += 1;
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
            else if(ringCount == StrategyParam::iterRing/2){
                StrategyParam::R_MAX_FUNCLAYER += 1;
            }
    }
    if(loop == 0) { 
        shift++;
        if (shift >= size_Tri)
            shift = 0;
        triAlg.flagT = true;       
    }
    UINT32 conClaPair = 0;
    UINT32 countUnit = 0;
    do{
        shiftSingleCla(triAlg.vOnlyOutTriCmp, shift);
        if(triAlg.vOnlyOutTriCmp[0]->isConLit() &&triAlg.flagT){
            conClaPair = triAlg.realUnifyCounts(fol, triAlg.vOnlyOutTriCmp[0]->claPtr);
            if (conClaPair == 0) {
                if(triAlg.noCmpCla.find(triAlg.vOnlyOutTriCmp[0]->claPtr->uClaId) == triAlg.noCmpCla.end())
                    triAlg.noCmpCla.insert(triAlg.vOnlyOutTriCmp[0]->claPtr->uClaId);
                stable_sort(triAlg.vOnlyOutTriCmp.begin() , triAlg.vOnlyOutTriCmp.end(), SortRule::singleResetId); 
                ++shift;
                ++countUnit;
                if (shift >= size_Tri){
                    shift = 0;
                    if(countUnit == size_Tri)
                    {
                        break;
                    }
                }
            }
        }
    }while(triAlg.vOnlyOutTriCmp[0]->isConLit() && (conClaPair == 0) &&triAlg.flagT);
    startClause = triAlg.vOnlyOutTriCmp[0]->claPtr;
    if(triAlg.flagT)
    {
        if(StrategyParam::iterTimes == -1){
            if(triAlg.vOnlyOutTriCmp[0]->isConLit())
                loop = conClaPair;
            else
                loop = triAlg.vOnlyOutTriCmp[0]->getPairPredLst()->size();
            triAlg.total_Loop = loop;
        }
        else{
            loop = StrategyParam::iterTimes;
            triAlg.total_Loop = loop;
        }
        triAlg.flagT = false;
    }
    if ((triAlg.vOnlyOutTriCmp.size() > 2)) {
         if(StrategyParam::useSaturate){
            stable_sort(triAlg.vOnlyOutTriCmp.begin()+1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmp);
        }
         else{
             if (StrategyParam::unitSortRule == 0) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.begin() + fol.uOrigClaSingleSize, SortRule::claSingCmp); 
            } else if (StrategyParam::unitSortRule == 1) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.begin() + fol.uOrigClaSingleSize, SortRule::claSingCmp); 
                if (triAlg.vOnlyOutTriCmp.size() > fol.uOrigClaSingleSize)
                    stable_sort(triAlg.vOnlyOutTriCmp.begin() + fol.uOrigClaSingleSize, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmp); 
            } else if (StrategyParam::unitSortRule == 2) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpComplex); 
            } else if (StrategyParam::unitSortRule == 3){
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingGenCmpNeg); 
            }else if (StrategyParam::unitSortRule == 4) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpPairSizeInc); 
            }else if (StrategyParam::unitSortRule == 5) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpPairSizeDec); 
            }else if (StrategyParam::unitSortRule == 6) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpVarSzie); 
            }else if (StrategyParam::unitSortRule == 7) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpVarSzieDec); 
            }
            else if (StrategyParam::unitSortRule == 8) {
                stable_sort(triAlg.vOnlyOutTriCmp.begin() + 1, triAlg.vOnlyOutTriCmp.end(), SortRule::claSingCmpGenCla); 
            }
         }
    }
   if(chooseTri)
        loop--;
    if (StrategyParam::unitDeductTimesUpdate == 0) {
        for (UINT32 k = 0; k < size_Tri; k++) {
            if (StrategyParam::unitDeductTimes > 0)
            {
                if (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum > StrategyParam::unitDeductTimes) {
                    triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = 0;
                }
            }
            if (loop == 0) {
                triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum % 500;
            }
        }
    }
    else if(StrategyParam::unitDeductTimesUpdate == 1)
    {
        for (UINT32 k = 0; k < size_Tri; k++) {
                if (StrategyParam::unitDeductTimes > 0)
                {
                    if (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum > StrategyParam::unitDeductTimes) {
                        triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = 0;
                    }
                }
                 if (loop == 0)
                    triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum =  triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum % 100;
            }
    }
    else
    {
        for (UINT32 k = 0; k < size_Tri; k++) {
                if (StrategyParam::unitDeductTimes > 0)
                {
                    if (triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum > StrategyParam::unitDeductTimes) {
                        triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = 0;
                    }
                }
                 if ((shift >= size_Tri -1) && (loop == 0))
                    triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum = triAlg.vOnlyOutTriCmp[k]->claPtr->adeductNum / size_Tri;
            }
    }
    if (StrategyParam::unitReduntTimesUpdate == 0) {
        for (UINT32 k = 0; k < size_Tri; k++) {
            if (StrategyParam::unitReduntTimes > 0)
            {
                if (triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes > StrategyParam::unitReduntTimes) {
                    triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = 0;
                }
            }
            if (loop==0) {
                triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes % 100;
            }
        }
    }
    else if(StrategyParam::unitReduntTimesUpdate == 1)
    {
        for (UINT32 k = 0; k < size_Tri; k++) {
                if (StrategyParam::unitReduntTimes > 0)
                {
                    if (triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes > StrategyParam::unitReduntTimes) {
                        triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = 0;
                    }
                }
                 if (loop == 0)
                    triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes % 100;
            }
    }
    else
    {
        for (UINT32 k = 0; k < size_Tri; k++) {
                if (StrategyParam::unitReduntTimes > 0)
                {
                    if (triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes > StrategyParam::unitReduntTimes) {
                        triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = 0;
                    }
                }
                 if ((shift >= size_Tri -1) && (loop == 0))
                    triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes = triAlg.vOnlyOutTriCmp[k]->claPtr->reduntTimes / size_Tri; 
            }
    }
    if (StrategyParam::candDeductTimesUpdate == 0) {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if (loop == 0 ) {
                clapE->pdeductNum = clapE->pdeductNum % 100;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if (loop ==0 ) {
                clapE->pdeductNum = clapE->pdeductNum % 100;
            }
        }
    }
    else if (StrategyParam::candDeductTimesUpdate == 1) {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if (loop == 0) {
                clapE->pdeductNum = clapE->pdeductNum % 100;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if (loop == 0) {
                clapE->pdeductNum = clapE->pdeductNum % 100;
            }
        }
    }
    else
     {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if ((shift >= size_Tri -1) && (loop == 0)) {
                clapE->pdeductNum = clapE->pdeductNum / size_Tri; 
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candDeductTimes > 0)
            {
                if (clapE->pdeductNum > StrategyParam::unitDeductTimes) {
                    clapE->pdeductNum = 0;
                }
            }
            if ((shift >= size_Tri -1) && (loop == 0))  {
                clapE->pdeductNum = clapE->pdeductNum / size_Tri; 
            }
        }
    }
     if (StrategyParam::candReduntTimesUpdate == 0) {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if (loop == 0) {
                clapE->reduntTimes = clapE->reduntTimes % 100;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
             if (loop ==0 ) {
                clapE->reduntTimes =clapE->reduntTimes % 100;
            }
        }
    }
     else if (StrategyParam::candReduntTimesUpdate == 1) {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if (loop == 0) {
                clapE->reduntTimes = clapE->reduntTimes % 100;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if (loop == 0) {
                clapE->reduntTimes = clapE->reduntTimes % 100;
            }
        }
    }
    else
    {
        for (auto& clapE : fol.setEnq) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if ((shift >= size_Tri -1) && (loop ==0)) {
                clapE->reduntTimes = clapE->reduntTimes / size_Tri;
            }
        }
        for (auto& clapE : fol.setCla) {
            if (StrategyParam::candReduntTimes > 0)
            {
                if (clapE->reduntTimes > StrategyParam::candReduntTimes) {
                    clapE->reduntTimes = 0;
                }
            }
            if ((shift >= size_Tri -1) && (loop == 0)) {
                clapE->reduntTimes = clapE->reduntTimes / size_Tri; 
            }
        }
    }
    for (UINT32 k = 0; k < size_Tri; k++) {
        triAlg.vOnlyOutTri.push_back(triAlg.vOnlyOutTriCmp[k]);
        triAlg.vOutTri.push_back(triAlg.vOnlyOutTriCmp[k]);
    }
}

int Resolution::JudgeLockResSimlifyNew(Formula& fol,TriAlg& triAlg,Literal* QLit,Literal* candLitp,vector<Literal*>& candLitPVec,UINT32& uNewClaID,UINT8 lockType,vector<Term*>& VarVec){
    Clause* actCla = QLit->claPtr;
    Clause* passiveCLa = candLitp->claPtr;
    vector<Literal*> vSimplyNewR;
    for(UINT16 i=0; i< actCla->uLitNum ; ++i){
        if(actCla->LitPtr[i] != QLit){
            vSimplyNewR.push_back(actCla->LitPtr[i]);
        }
    }
    for(UINT16 i=0; i< passiveCLa->uLitNum ; ++i){
        if(find(candLitPVec.begin(), candLitPVec.end(), passiveCLa->LitPtr[i]) == candLitPVec.end()){
            vSimplyNewR.push_back(passiveCLa->LitPtr[i]);
        }
    }
    if(vSimplyNewR.size() != 0) {
        for (size_t rInd = 0; rInd < vSimplyNewR.size() - 1; ++rInd) {
            Literal* rA = vSimplyNewR[rInd];
            for (size_t rIndB = rInd + 1; rIndB < vSimplyNewR.size();) {
                Literal* rB = vSimplyNewR[rIndB];
                if (rA->subTerm->iTermCode == rB->subTerm->iTermCode && triAlg.unify.CheckEqn(rA->subTerm, rB->subTerm)) {
                    swap(vSimplyNewR[rIndB], vSimplyNewR[vSimplyNewR.size() - 1]);
                    vSimplyNewR.pop_back();
                } else
                    ++rIndB;
            }
        }
    }
    Clause* newCla = nullptr;
    newCla = new Clause(uNewClaID, 0);
    size_t uRsize = vSimplyNewR.size();
    if (uRsize > 1)
        stable_sort(vSimplyNewR.begin(), vSimplyNewR.end(), SortRule::LitCmp);
    newCla->LitPtr = new Literal*[uRsize];
    for (UINT16 i = 0; i < vSimplyNewR.size(); ++i) {
        Term* litSubT = triAlg.unify.GetNewTerm(vSimplyNewR[i]->subTerm, newCla); 
        if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
            --uRsize;
            continue;
        }
        if (newCla->uFuncDepth < litSubT->uLevel)
            newCla->uFuncDepth = litSubT->uLevel;
        newCla->uVarCount += litSubT->uVarCount;		
        newCla->LitPtr[newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, newCla->uLitNum + 1), newCla, nullptr);
        newCla->LitPtr[newCla->uLitNum]->parentLitPtr = vSimplyNewR[i]; 
        ++(newCla->uLitNum);
    }
    
    if(triAlg.unify.isRedundancy(newCla)) {
        newCla->DelLits(false);
        delete(newCla);
        return 1;
    }
    if(lockType == 1){
        newCla->DelLits(false);
        delete(newCla);
        return 0;
    }
    vector<UINT32>invaildLitId;
    string rundMsg; 
    Subsumption subsump; 
    bool isRundacyR = subsump.GenCla(fol, newCla, invaildLitId, rundMsg);
    if (isRundacyR) {
        newCla->DelLits(false);
        delete(newCla);
        return 2;
    }
     if(lockType == 2){
        newCla->DelLits(false);
        delete(newCla);
        return 0;
    }
    if(1){
        vector<Literal*>* vCandSameLit;
        vector<Term*> chgVarTVec;
        vector<Term*> chgVarT;
        for(UINT16 i=0; i< newCla->uLitNum; ++i){
            Literal* noPair = newCla->LitPtr[i];
            vCandSameLit = noPair->getPredLst();
            Literal* candSameLit = nullptr;
            Clause* candSameCla = nullptr;
            UINT32 pasLitInd = 0;
            for (; pasLitInd < UINT32(vCandSameLit->size()); ++pasLitInd) {
                candSameLit = vCandSameLit->at(pasLitInd); 
                candSameCla = candSameLit->claPtr; 
                if(candSameCla->uLitNum > 2)
                    continue;
                vector<Literal*> candLeftLitVec;
                candLeftLitVec.reserve(2);
                if (triAlg.unify.litASmallLitB(noPair->subTerm, candSameLit->subTerm, chgVarT)) {
                    chgVarTVec.insert(chgVarTVec.end(), chgVarT.begin(), chgVarT.end());
                    chgVarT.clear();
                    for (UINT16 m = 0; m < candSameCla->uLitNum; ++m) {
                        if(candSameLit != candSameCla->LitPtr[m]){
                            continue;
                        }
                        bool leftFlag = true;
                        for(UINT16 n=0; n<newCla->uLitNum;++n) {
                            if (newCla->LitPtr[n]->isEqPred(candSameCla->LitPtr[m]) && triAlg.unify.litASmallLitB(newCla->LitPtr[n]->subTerm, candSameCla->LitPtr[m]->subTerm, chgVarT)) {
                                chgVarTVec.insert(chgVarTVec.end(), chgVarT.begin(), chgVarT.end());
                                chgVarT.clear();
                                leftFlag = false;
                                break;
                            } else
                                triAlg.unify.ClearVarBind(chgVarT);
                        }
                        if(leftFlag)
                            candLeftLitVec.push_back(candSameCla->LitPtr[m]);
                    }
                    if(candLeftLitVec.size() == 0){
                        triAlg.unify.ClearVarBind(chgVarTVec);
                        candLeftLitVec.clear();
                        newCla->DelLits(false);
                        delete(newCla);
                        return 0;
                    }
                    else if (lockType == 3) {
                        triAlg.unify.ClearVarBind(chgVarTVec);
                        candLeftLitVec.clear();
                        newCla->DelLits(false);
                        delete(newCla);
                        return 0;
                    }
                    else if (candLeftLitVec.size() == 1) {
                        uNewClaID++;
                        Clause* newClaPair = nullptr;
                        if(1){
                            newClaPair = new Clause(uNewClaID, 0);
                            size_t uRsize = candLeftLitVec.size();
                            if (uRsize > 1)
                                stable_sort(candLeftLitVec.begin(), candLeftLitVec.end(), SortRule::LitCmp);
                            newClaPair->LitPtr = new Literal*[uRsize];
                            for (UINT16 i = 0; i < candLeftLitVec.size(); ++i) {
                                Term* litSubT = triAlg.unify.GetNewTerm(candLeftLitVec[i]->subTerm, newClaPair);
                                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                                    --uRsize;
                                    continue;
                                }
                                if (newClaPair->uFuncDepth < litSubT->uLevel)
                                    newClaPair->uFuncDepth = litSubT->uLevel;
                                newClaPair->uVarCount += litSubT->uVarCount;		
                                newClaPair->LitPtr[newClaPair->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, newClaPair->uLitNum + 1), newClaPair, nullptr);
                                newClaPair->LitPtr[newClaPair->uLitNum]->parentLitPtr = candLeftLitVec[i];  
                                ++(newClaPair->uLitNum);
                            }
                            UINT32 uPredProp = GetCodeProp(newClaPair->LitPtr[0]->subTerm->iTermCode);
                            INT32 iPredCode = Get1stTermID(newClaPair->LitPtr[0]->subTerm->iTermCode);
                            if (uPredProp == TNegPred) {
                                newClaPair->LitPtr[0]->subTerm->iTermCode = Set1stTProp(iPredCode, TPred);
                            } else if (uPredProp == TPred) {
                                newClaPair->LitPtr[0]->subTerm->iTermCode = Set1stTProp(iPredCode, TNegPred);
                            } else if (uPredProp == TNegEqn) {
                                newClaPair->LitPtr[0]->subTerm->iTermCode = Set1stTProp(iPredCode, TEqn);
                            } else if (uPredProp == TEqn) {
                                newClaPair->LitPtr[0]->subTerm->iTermCode = Set1stTProp(iPredCode, TNegEqn);
                            }
                            for (UINT16 n = 0; n < newCla->uLitNum; ++n) {
                                candLeftLitVec.push_back(newCla->LitPtr[n]);
                            }
                        }
                        uNewClaID++;
                        Clause* newClaEnd = nullptr;
                        if(1){
                            newClaEnd = new Clause(uNewClaID, 0);
                            size_t uRsize = candLeftLitVec.size();
                            if (uRsize > 1)
                                stable_sort(candLeftLitVec.begin(), candLeftLitVec.end(), SortRule::LitCmp);
                            newClaEnd->LitPtr = new Literal*[uRsize];
                            for (UINT16 i = 0; i < candLeftLitVec.size(); ++i) {
                                Term* litSubT = triAlg.unify.GetNewTerm(candLeftLitVec[i]->subTerm, newClaEnd);
                                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                                    --uRsize;
                                    continue;
                                }
                                if (newClaEnd->uFuncDepth < litSubT->uLevel)
                                    newClaEnd->uFuncDepth = litSubT->uLevel;
                                newClaEnd->uVarCount += litSubT->uVarCount;		
                                newClaEnd->LitPtr[newClaEnd->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, newClaEnd->uLitNum + 1), newClaEnd, nullptr);
                                newClaEnd->LitPtr[newClaEnd->uLitNum]->parentLitPtr = candLeftLitVec[i];
                                ++(newClaEnd->uLitNum);
                            }
                            if (triAlg.unify.isRedundancy(newClaEnd)) {
                                triAlg.unify.ClearVarBind(chgVarTVec);
                                newClaEnd->DelLits(false);
                                delete(newClaEnd);
                                uNewClaID--;
                                newClaPair->DelLits(false);
                                delete(newClaPair);
                                uNewClaID--;
                                newCla->DelLits(false);
                                delete(newCla);
                                return 4; 
                            }
                            if(lockType == 4){
                                triAlg.unify.ClearVarBind(chgVarTVec);
                                newClaEnd->DelLits(false);
                                delete(newClaEnd);
                                uNewClaID--;
                                newClaPair->DelLits(false);
                                delete(newClaPair);
                                uNewClaID--;
                                newCla->DelLits(false);
                                delete(newCla);
                                return 0; 
                            }
                            vector<UINT32>invaildLitId; 
                            string rundMsg; 
                            Subsumption subsump; 
                            bool isRundacyR = subsump.GenCla(fol, newClaEnd, invaildLitId, rundMsg);
                            if (isRundacyR) {
                                triAlg.unify.ClearVarBind(chgVarTVec);
                                newClaEnd->DelLits(false);
                                delete(newClaEnd);
                                uNewClaID--;
                                newClaPair->DelLits(false);
                                delete(newClaPair);
                                uNewClaID--;
                                newCla->DelLits(false);
                                delete(newCla);
                                return 5; 
                            }
                        }
                        triAlg.unify.ClearVarBind(chgVarTVec);
                        newClaEnd->DelLits(false);
                        delete(newClaEnd);
                        uNewClaID--;
                        newClaPair->DelLits(false);
                        delete(newClaPair);
                        uNewClaID--;
                    }
                    else{
                        triAlg.unify.ClearVarBind(chgVarTVec);
                    }
                }
                else{
                    triAlg.unify.ClearVarBind(chgVarT);
                }
            }
        }
        triAlg.unify.ClearVarBind(chgVarT);
    }
    newCla->DelLits(false);
    delete(newCla);
    return 0;
}

int Resolution::JudgeLockResSimlify(Formula& fol,TriAlg& triAlg,Literal* QLit,Literal* candLitp,vector<Literal*>& candLitPVec,UINT32& uNewClaID,UINT8 lockType,vector<Term*>& VarVec){
    Clause* actCla = QLit->claPtr;
    Clause* passiveCLa = candLitp->claPtr;
    vector<Literal*> vSimplyNewR;
    for(UINT16 i=0; i< actCla->uLitNum ; ++i){
        if(actCla->LitPtr[i] != QLit){
            vSimplyNewR.push_back(actCla->LitPtr[i]);
        }
    }
    for(UINT16 i=0; i< passiveCLa->uLitNum ; ++i){
        if(find(candLitPVec.begin(), candLitPVec.end(), passiveCLa->LitPtr[i]) == candLitPVec.end()){
            vSimplyNewR.push_back(passiveCLa->LitPtr[i]);
        }
    }
    if(vSimplyNewR.size() != 0) {
        for (size_t rInd = 0; rInd < vSimplyNewR.size() - 1; ++rInd) {
            Literal* rA = vSimplyNewR[rInd];
            for (size_t rIndB = rInd + 1; rIndB < vSimplyNewR.size();) {
                Literal* rB = vSimplyNewR[rIndB];
                if (rA->subTerm->iTermCode == rB->subTerm->iTermCode && triAlg.unify.CheckEqn(rA->subTerm, rB->subTerm)) {
                    swap(vSimplyNewR[rIndB], vSimplyNewR[vSimplyNewR.size() - 1]);
                    vSimplyNewR.pop_back();
                } else
                    ++rIndB;
            }
        }
    }
    Clause* newCla = nullptr;
    newCla = new Clause(uNewClaID, 0);
    size_t uRsize = vSimplyNewR.size();
    if (uRsize > 1)
        stable_sort(vSimplyNewR.begin(), vSimplyNewR.end(), SortRule::LitCmp);
    newCla->LitPtr = new Literal*[uRsize];
    for (UINT16 i = 0; i < vSimplyNewR.size(); ++i) {
        Term* litSubT = triAlg.unify.GetNewTerm(vSimplyNewR[i]->subTerm, newCla);
        if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
            --uRsize;
            continue;
        }
        if (newCla->uFuncDepth < litSubT->uLevel)
            newCla->uFuncDepth = litSubT->uLevel;
        newCla->uVarCount += litSubT->uVarCount;		
        newCla->LitPtr[newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, newCla->uLitNum + 1), newCla, nullptr);
        newCla->LitPtr[newCla->uLitNum]->parentLitPtr = vSimplyNewR[i]; 
        ++(newCla->uLitNum);
    }
    
    if(triAlg.unify.isRedundancy(newCla)) {
        newCla->DelLits(false);
        delete(newCla);
        return 1;
    }
    if(lockType == 1){
        newCla->DelLits(false);
        delete(newCla);
        return 0;
    }
    vector<UINT32>invaildLitId; 
    string rundMsg; 
    Subsumption subsump; 
    bool isRundacyR = subsump.GenCla(fol, newCla, invaildLitId, rundMsg);
    if (isRundacyR) {
        newCla->DelLits(false);
        delete(newCla);
        return 2;
    }
     if(lockType == 2){
        newCla->DelLits(false);
        delete(newCla);
        return 0;
    }
    if(1){
        vector<Literal*>* vCandSameLit;
        vector<Term*> chgVarTVec;
        vector<Term*> chgVarT;
        for(UINT16 i=0; i< newCla->uLitNum; ++i){
            Literal* noPair = newCla->LitPtr[i];
            vCandSameLit = noPair->getPredLst();
            Literal* candSameLit = nullptr;
            Clause* candSameCla = nullptr;
            UINT32 pasLitInd = 0;
            for (; pasLitInd < UINT32(vCandSameLit->size()); ++pasLitInd) {
                candSameLit = vCandSameLit->at(pasLitInd); 
                candSameCla = candSameLit->claPtr;              
                vector<Literal*> candLeftLitVec;
                candLeftLitVec.reserve(2);
                if (triAlg.unify.litASmallLitB(noPair->subTerm, candSameLit->subTerm, chgVarT)) {
                    chgVarTVec.insert(chgVarTVec.end(), chgVarT.begin(), chgVarT.end());
                    chgVarT.clear();
                    for (UINT16 m = 0; m < candSameCla->uLitNum; ++m) {
                        if(candSameLit != candSameCla->LitPtr[m]){
                            continue;
                        }
                        bool leftFlag = true;
                        for(UINT16 n=0; n<newCla->uLitNum;++n) {
                            if (newCla->LitPtr[n]->isEqPred(candSameCla->LitPtr[m]) && triAlg.unify.litASmallLitB(newCla->LitPtr[n]->subTerm, candSameCla->LitPtr[m]->subTerm, chgVarT)) {
                                chgVarTVec.insert(chgVarTVec.end(), chgVarT.begin(), chgVarT.end());
                                chgVarT.clear();
                                leftFlag = false;
                                break;
                            } else
                                triAlg.unify.ClearVarBind(chgVarT);
                        }
                        if(leftFlag)
                            candLeftLitVec.push_back(candSameCla->LitPtr[m]);
                    }
                    if(candLeftLitVec.size() == 0){
                        triAlg.unify.ClearVarBind(chgVarTVec);
                        candLeftLitVec.clear();
                        newCla->DelLits(false);
                        delete(newCla);
                        return 3;
                    }
                    else if (lockType == 3) {
                        triAlg.unify.ClearVarBind(chgVarTVec);
                        candLeftLitVec.clear();
                        newCla->DelLits(false);
                        delete(newCla);
                        return 0;
                    }
                    else if (candLeftLitVec.size() == 1) {
                        uNewClaID++;
                        Clause* newClaPair = nullptr;
                        if(1){
                            newClaPair = new Clause(uNewClaID, 0);
                            size_t uRsize = candLeftLitVec.size();
                            if (uRsize > 1)
                                stable_sort(candLeftLitVec.begin(), candLeftLitVec.end(), SortRule::LitCmp);
                            newClaPair->LitPtr = new Literal*[uRsize];
                            for (UINT16 i = 0; i < candLeftLitVec.size(); ++i) {
                                Term* litSubT = triAlg.unify.GetNewTerm(candLeftLitVec[i]->subTerm, newClaPair);
                                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                                    --uRsize;
                                    continue;
                                }
                                if (newClaPair->uFuncDepth < litSubT->uLevel)
                                    newClaPair->uFuncDepth = litSubT->uLevel;
                                newClaPair->uVarCount += litSubT->uVarCount;	
                                newClaPair->LitPtr[newClaPair->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, newClaPair->uLitNum + 1), newClaPair, nullptr);
                                newClaPair->LitPtr[newClaPair->uLitNum]->parentLitPtr = candLeftLitVec[i];
                                ++(newClaPair->uLitNum);
                            }
                            UINT32 uPredProp = GetCodeProp(newClaPair->LitPtr[0]->subTerm->iTermCode);
                            INT32 iPredCode = Get1stTermID(newClaPair->LitPtr[0]->subTerm->iTermCode);
                            if (uPredProp == TNegPred) {
                                newClaPair->LitPtr[0]->subTerm->iTermCode = Set1stTProp(iPredCode, TPred);
                            } else if (uPredProp == TPred) {
                                newClaPair->LitPtr[0]->subTerm->iTermCode = Set1stTProp(iPredCode, TNegPred);
                            } else if (uPredProp == TNegEqn) {
                                newClaPair->LitPtr[0]->subTerm->iTermCode = Set1stTProp(iPredCode, TEqn);
                            } else if (uPredProp == TEqn) {
                                newClaPair->LitPtr[0]->subTerm->iTermCode = Set1stTProp(iPredCode, TNegEqn);
                            }
                            for (UINT16 n = 0; n < newCla->uLitNum; ++n) {
                                candLeftLitVec.push_back(newCla->LitPtr[n]);
                            }
                        }
                        uNewClaID++;
                        Clause* newClaEnd = nullptr;
                        if(1){
                            newClaEnd = new Clause(uNewClaID, 0);
                            size_t uRsize = candLeftLitVec.size();
                            if (uRsize > 1)
                                stable_sort(candLeftLitVec.begin(), candLeftLitVec.end(), SortRule::LitCmp);
                            newClaEnd->LitPtr = new Literal*[uRsize];
                            for (UINT16 i = 0; i < candLeftLitVec.size(); ++i) {
                                Term* litSubT = triAlg.unify.GetNewTerm(candLeftLitVec[i]->subTerm, newClaEnd); 
                                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                                    --uRsize;
                                    continue;
                                }
                                if (newClaEnd->uFuncDepth < litSubT->uLevel)
                                    newClaEnd->uFuncDepth = litSubT->uLevel;
                                newClaEnd->uVarCount += litSubT->uVarCount;		
                                newClaEnd->LitPtr[newClaEnd->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, newClaEnd->uLitNum + 1), newClaEnd, nullptr);
                                newClaEnd->LitPtr[newClaEnd->uLitNum]->parentLitPtr = candLeftLitVec[i];
                                ++(newClaEnd->uLitNum);
                            }
                            if (triAlg.unify.isRedundancy(newClaEnd)) {
                                triAlg.unify.ClearVarBind(chgVarTVec);
                                newClaEnd->DelLits(false);
                                delete(newClaEnd);
                                uNewClaID--;
                                newClaPair->DelLits(false);
                                delete(newClaPair);
                                uNewClaID--;
                                newCla->DelLits(false);
                                delete(newCla);
                                return 4;
                            }
                            if(lockType == 4){
                                triAlg.unify.ClearVarBind(chgVarTVec);
                                newClaEnd->DelLits(false);
                                delete(newClaEnd);
                                uNewClaID--;
                                newClaPair->DelLits(false);
                                delete(newClaPair);
                                uNewClaID--;
                                newCla->DelLits(false);
                                delete(newCla);
                                return 0;
                            }
                            vector<UINT32>invaildLitId; 
                            string rundMsg; 
                            Subsumption subsump; 
                            bool isRundacyR = subsump.GenCla(fol, newClaEnd, invaildLitId, rundMsg);
                            if (isRundacyR) {
                                triAlg.unify.ClearVarBind(chgVarTVec);
                                newClaEnd->DelLits(false);
                                delete(newClaEnd);
                                uNewClaID--;
                                newClaPair->DelLits(false);
                                delete(newClaPair);
                                uNewClaID--;
                                newCla->DelLits(false);
                                delete(newCla);
                                return 5; 
                            }
                        }
                        triAlg.unify.ClearVarBind(chgVarTVec);
                        newClaEnd->DelLits(false);
                        delete(newClaEnd);
                        uNewClaID--;
                        newClaPair->DelLits(false);
                        delete(newClaPair);
                        uNewClaID--;
                    }
                    else{
                        triAlg.unify.ClearVarBind(chgVarTVec);
                    }
                }
                else{
                    triAlg.unify.ClearVarBind(chgVarT);
                }
            }
        }
        triAlg.unify.ClearVarBind(chgVarT);
    }
    newCla->DelLits(false);
    delete(newCla);
    return 0;
}

bool Resolution::lockResSimlifyNew(Formula& fol,TriAlg& triAlg,Clause* cla,UINT32& uNewClaID){
    vector<Literal*>* vCandLit;   
    Clause* candClaP = nullptr;
    Literal* candLitP = nullptr;
    vector<Literal*> candLitPVec;
    candLitPVec.reserve(2);
    vector<Term*> chgVarT;
    vector<Term*> chgVarTVec;
    bool perFlag = true;
    bool pureLit = true;
    for(UINT16 i=0; i< cla->uLitNum ; ++i){
        perFlag = true;
        pureLit = true;
        Literal* QLit = cla->LitPtr[i];
        if(QLit->isOnlyOneAppeared() == false){
            continue;
        }
        vCandLit = QLit->getPairPredLst();
        UINT32 pasLitInd = 0;
        for (; pasLitInd < UINT32(vCandLit->size()); ++pasLitInd) {
            candLitP = vCandLit->at(pasLitInd); 
            candClaP = candLitP->claPtr; 
            if (triAlg.unify.LitMgu(QLit, candLitP, chgVarT)) {
                pureLit = false;
                chgVarTVec.insert(chgVarTVec.end(), chgVarT.begin(), chgVarT.end());
                candLitPVec.push_back(candLitP);
                chgVarT.clear();
                for (UINT16 m = 0; m < candClaP->uLitNum; ++m) {
                    if (find(candLitPVec.begin(), candLitPVec.end(), candClaP->LitPtr[m]) != candLitPVec.end()) {
                        continue;
                    }
                    if (QLit->isPairPred(candClaP->LitPtr[m]) && triAlg.unify.LitMgu(QLit, candClaP->LitPtr[m], chgVarT)) {
                        
                        chgVarTVec.insert(chgVarTVec.end(), chgVarT.begin(), chgVarT.end());
                        candLitPVec.push_back(candClaP->LitPtr[m]);
                        chgVarT.clear();
                    } else
                        triAlg.unify.ClearVarBind(chgVarT);
                }
                chgVarT.clear();
                int res = JudgeLockResSimlifyNew(fol, triAlg, QLit, candLitP, candLitPVec, uNewClaID,StrategyParam::lockType,chgVarTVec);
                if ( res== 0) {                                                                               
                    triAlg.unify.ClearVarBind(chgVarTVec);
                    perFlag = false;
                    break;
                }
                else{
                    candLitPVec.clear();
                    triAlg.unify.ClearVarBind(chgVarTVec);
                    continue;
                }
            }
            else{
                triAlg.unify.ClearVarBind(chgVarT);
            }
        }
        if(pureLit){
            candLitPVec.clear();
            triAlg.unify.ClearVarBind(chgVarTVec);
            lockLitVec.push_back(QLit);
            lockClaVec.push_back(cla);
            return true;
        }
        if(perFlag){
            candLitPVec.clear();
            triAlg.unify.ClearVarBind(chgVarTVec);
            lockLitVec.push_back(QLit);
            lockClaVec.push_back(cla);
            return false;
        }
        else{
            triAlg.unify.ClearVarBind(chgVarTVec);
            candLitPVec.clear();
        }
    }
    return false;
}

bool Resolution::lockResSimlify(Formula& fol,TriAlg& triAlg,Clause* cla,UINT32& uNewClaID){
    vector<Literal*>* vCandLit;   
    Clause* candClaP = nullptr;
    Literal* candLitP = nullptr;
    vector<Literal*> candLitPVec;
    candLitPVec.reserve(2);
    vector<Term*> chgVarT;
    vector<Term*> chgVarTVec;
    bool perFlag = true;
    for(UINT16 i=0; i< cla->uLitNum ; ++i){
        perFlag = true;
        Literal* QLit = cla->LitPtr[i];
        vCandLit = QLit->getPairPredLst();
        UINT32 pasLitInd = 0;
        for (; pasLitInd < UINT32(vCandLit->size()); ++pasLitInd) {
            candLitP = vCandLit->at(pasLitInd); 
            candClaP = candLitP->claPtr;
            if (triAlg.unify.LitMgu(QLit, candLitP, chgVarT)) {
                chgVarTVec.insert(chgVarTVec.end(), chgVarT.begin(), chgVarT.end());
                candLitPVec.push_back(candLitP);
                chgVarT.clear();
                for (UINT16 m = 0; m < candClaP->uLitNum; ++m) {
                    if (find(candLitPVec.begin(), candLitPVec.end(), candClaP->LitPtr[m]) != candLitPVec.end()) {
                        continue;
                    }
                    if (QLit->isPairPred(candClaP->LitPtr[m]) && triAlg.unify.LitMgu(QLit, candClaP->LitPtr[m], chgVarT)) {
                        
                        chgVarTVec.insert(chgVarTVec.end(), chgVarT.begin(), chgVarT.end());
                        candLitPVec.push_back(candClaP->LitPtr[m]);
                        chgVarT.clear();
                    } else
                        triAlg.unify.ClearVarBind(chgVarT);
                }
                chgVarT.clear();
                int res = JudgeLockResSimlify(fol, triAlg, QLit, candLitP, candLitPVec, uNewClaID,StrategyParam::lockType,chgVarTVec);
                if ( res== 0) {                                                                              
                    triAlg.unify.ClearVarBind(chgVarTVec);
                    perFlag = false;
                    break;
                }
                else{
                    candLitPVec.clear();
                    triAlg.unify.ClearVarBind(chgVarTVec);
                    continue;
                }
            }
            else{
                triAlg.unify.ClearVarBind(chgVarT);
            }
        }
        if(perFlag){
            candLitPVec.clear();
            triAlg.unify.ClearVarBind(chgVarTVec);
            lockLitVec.push_back(QLit);
            lockClaVec.push_back(cla);
            return true;
        }
        else{
            triAlg.unify.ClearVarBind(chgVarTVec);
            candLitPVec.clear();
        }
    }
    return false;
}

int Resolution::factorRes(Formula& fol,TriAlg& triAlg,Clause* cla,UINT32& uNewClaID,vector<Clause*>* preGenCla,bool& resDelFlag, bool fileNameFlag){
    if(cla->hasSamePredict() == false){
        resDelFlag = false;
        return 0;
    }
    else{
        vector<Term*> chgVarT; 
        chgVarT.reserve(6);
        vector<Literal*> vSimplyNewR;
        vSimplyNewR.reserve(6);
        vector<Term*> varBindT; 
        varBindT.reserve(6);
        bool flag = false;
        for(int i=0;i<cla->uLitNum-1;++i){
            for(int j=i+1;j<cla->uLitNum;++j){
                if(cla->LitPtr[i]->isEqPred(cla->LitPtr[j]) && ((cla->LitPtr[i]->noExistShareVar() && cla->LitPtr[j]->isConLit()) || (cla->LitPtr[i]->isConLit() && cla->LitPtr[j]->noExistShareVar()))){
                    if(triAlg.unify.LitMgu(cla->LitPtr[i],cla->LitPtr[j],chgVarT)){
                        flag = true;
                        varBindT.insert(varBindT.end(), chgVarT.begin(), chgVarT.end());
                        chgVarT.clear();
                    }
                    else{
                        triAlg.unify.ClearVarBind(chgVarT);
                    }
                }
                else{
                    continue;
                }
            }
        }
        if(flag){
            for (int i = 0; i < cla->uLitNum; ++i) {
                vSimplyNewR.push_back(cla->LitPtr[i]);
            }
            for (size_t rInd = 0; rInd < vSimplyNewR.size() - 1; ++rInd) {
                Literal* rA = vSimplyNewR[rInd];
                for (size_t rIndB = rInd + 1; rIndB < vSimplyNewR.size();) {
                    Literal* rB = vSimplyNewR[rIndB];
                    if (rA->subTerm->iTermCode == rB->subTerm->iTermCode && triAlg.unify.CheckEqn(rA->subTerm, rB->subTerm)) {
                        swap(vSimplyNewR[rIndB], vSimplyNewR[vSimplyNewR.size() - 1]);
                        vSimplyNewR.pop_back();
                    } else
                        ++rIndB;
                }
            }
            vector<string> strLit;
            strLit.reserve(6);
            string str;
            if (vSimplyNewR.size() > 1) {
                for (int m = 0; m < vSimplyNewR.size(); ++m) {
                    str = "";
                    triAlg.unify.TermToString(vSimplyNewR[m]->subTerm, str);
                    strLit.push_back(str);
                }
            }
            if (strLit.size() > 1) {
                if (RedundancyOp::CheckTautology(strLit)) {
                    triAlg.unify.ClearVarBind(varBindT);
                    vSimplyNewR.clear();
                    vector<Literal*>().swap(vSimplyNewR);
                    strLit.clear();
                    vector<string>().swap(strLit);
                    resDelFlag = false;
                    return 1;
                }
            }
            triAlg.newCla = nullptr;
            triAlg.newCla = new Clause(uNewClaID, 0);
            triAlg.newCla->isAxtom = true;
            size_t uRsize = vSimplyNewR.size();
            if (uRsize > 1)
                stable_sort(vSimplyNewR.begin(), vSimplyNewR.end(), SortRule::LitCmp);
            triAlg.newCla->LitPtr = new Literal*[uRsize];
            for (UINT16 i = 0; i < vSimplyNewR.size(); ++i) {
                Term* litSubT = triAlg.unify.GetNewTerm(vSimplyNewR[i]->subTerm, triAlg.newCla); 
                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                    --uRsize;
                    continue;
                }
                if (triAlg.newCla->uFuncDepth < litSubT->uLevel)
                    triAlg.newCla->uFuncDepth = litSubT->uLevel;
                triAlg.newCla->uVarCount += litSubT->uVarCount;		
                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, triAlg.newCla->uLitNum + 1), triAlg.newCla, nullptr);
                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->parentLitPtr = vSimplyNewR[i]; 
                if(triAlg.newCla ->deduceDeepth < (vSimplyNewR[i]->claPtr->deduceDeepth + 1)){
                    triAlg.newCla ->deduceDeepth = vSimplyNewR[i]->claPtr->deduceDeepth + 1;
                }
                 if (vSimplyNewR[i]->claPtr->isAxtom && triAlg.newCla->isAxtom)
                    triAlg.newCla->isAxtom = true;
                else
                    triAlg.newCla->isAxtom = false;
                if(triAlg.hasUnifGoalLit(fol,triAlg.newCla->LitPtr[triAlg.newCla->uLitNum])){
                    triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = 1;
                    if(triAlg.newCla->goaldeepth > 1)
                        triAlg.newCla->goaldeepth = 1;
                }
                else{
                    triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = vSimplyNewR[i]->goalDeepth;
                      if(triAlg.newCla->goaldeepth > vSimplyNewR[i]->goalDeepth)
                        triAlg.newCla->goaldeepth = vSimplyNewR[i]->goalDeepth;
                }
                ++(triAlg.newCla->uLitNum);
            }
            fol.initGenerateClaAttr(triAlg.newCla);
            uNewClaID++; 
            if(fileNameFlag == true)
                FileOp::CreateRunInfoFilePre(20000);
            else
                FileOp::CreateRunInfoFile(20000);
            *FileOp::ofRealInfoOut << "从编号为" + to_string(cla->uClaId) + "子句开始进行因子归结演绎" << endl;
            for (UINT32 k = 0; k < cla->uLitNum; k++) {
                triAlg.unify.WriteTriPath(cla->LitPtr[k]);
            }
            if(StrategyParam::orForCompition) {
                triAlg.newCla->addRForCompetiton();               
                string rStr = "";
                triAlg.newCla->toString(rStr);
                validPathOut[uNewClaID - 1].push_back(cla->uClaId);
                int begPos = rStr.find_first_of("[") + 1;
                int endPos = rStr.find_first_of("]");
                string rId = rStr.substr(begPos, endPos - begPos);
                string rContext = rStr.substr(rStr.find_first_of("]") + 1);
                string out = "cnf(" + rId + ",plain,\n" + "   ("+ rContext+"),\n"+"   inference(factoring_inference"+ "," + "[],"+ "["+to_string(cla->uClaId) +"]"+")).";               
                *FileOp::ofOutput << out << endl;
            }
            Clause* newCla = triAlg.newCla;
            if (StrategyParam::termRewriting != 2) {
                newCla->directEqn();
            }
            FileOp::WriteR(newCla);
            vector<UINT32>invaildLitId; 
            string rundMsg; 
            Subsumption subsump; 
            bool isRundacyR = subsump.GenCla(fol,newCla, invaildLitId, rundMsg);
            FileOp::RInvaildInfoOut(rundMsg, StartId);
            if (isRundacyR) {
                newCla->DelLits(false);
                delete(newCla);
                triAlg.unify.ClearVarBind(varBindT);
                vSimplyNewR.clear();
                vector<Literal*>().swap(vSimplyNewR);
                resDelFlag = false;
                return 0;
            } else {
                preGenCla->push_back(newCla);
                if (newCla->uLitNum == 1)
                    if (StrategyParam::R_Weight == -1)
                        newCla->adeductNum = 0;
                    else
                        newCla->adeductNum = StrategyParam::R_Weight;
                else
                    if (StrategyParam::R_Weight == -1)
                    newCla->pdeductNum = 0; 
                else
                    newCla->pdeductNum = StrategyParam::R_Weight;
                fol.InsertCla(newCla);
                fol.setCla.insert(newCla);
                if (StrategyParam::termRewriting != 2){
                    fol.judgeInsertSuper(newCla);
                }
                if(1){
                    insertNoPairCla(fol, newCla);
                }
                triAlg.unify.ClearVarBind(varBindT);
                vSimplyNewR.clear();
                vector<Literal*>().swap(vSimplyNewR);
                resDelFlag = true;
                return 2;
            }       
        }
        else{
            triAlg.unify.ClearVarBind(varBindT);
            resDelFlag = false;
            return 0;
        }
    }
}

int Resolution::negEqnRes(Formula& fol,TriAlg& triAlg,Clause* cla,UINT32& uNewClaID,vector<Clause*>* preGenCla,bool fileNameFlag){
    if(cla->hasOnlyNegEqn()){
        vector<Term*> chgVarT; 
        chgVarT.reserve(6);
        bool flag = false;
        vector<Literal*> vSimplyNewR;
        vSimplyNewR.reserve(6);
        for(int i=0;i<cla->uLitNum;++i){
            Term* term = cla->LitPtr[i]->subTerm;
            if(term->isNegEqnTerm()){
                if(term->subTermPtr[0]->isFuncTerm() &&term->subTermPtr[1]->isFuncTerm()){
                    if(triAlg.unify.pairFuncTermMgu(term->subTermPtr[0],term->subTermPtr[1], chgVarT)){
                        flag = true;
                    }
                    else{
                        triAlg.unify.ClearVarBind(chgVarT);
                        vSimplyNewR.clear();
                        vector<Literal*>().swap(vSimplyNewR);
                        return 0;
                    }
                }
                else{
                    if(triAlg.unify.TermMgu(term->subTermPtr[0],term->subTermPtr[1] , chgVarT)){
                        flag = true;
                    }
                    else{
                        triAlg.unify.ClearVarBind(chgVarT);
                        vSimplyNewR.clear();
                        vector<Literal*>().swap(vSimplyNewR);
                        return 0;
                    }
                }
            }
            vSimplyNewR.push_back(cla->LitPtr[i]);
        }
        if (flag) {
            vector<string> strLit;
            string str;
            if(vSimplyNewR.size()>1) {
                for (int m = 0; m < vSimplyNewR.size(); ++m) {
                    str = "";
                    triAlg.unify.TermToString(vSimplyNewR[m]->subTerm, str);
                    strLit.push_back(str);
                }
            }
            if(strLit.size()>1){
                if(RedundancyOp::CheckTautology(strLit)){
                    triAlg.unify.ClearVarBind(chgVarT);
                     vSimplyNewR.clear();
                    vector<Literal*>().swap(vSimplyNewR);
                    return 1;
                }
            }
            triAlg.newCla = nullptr;
            triAlg.newCla = new Clause(uNewClaID, 0);
            triAlg.newCla->isAxtom = true;
            size_t uRsize = vSimplyNewR.size();
            if (uRsize > 1)
                stable_sort(vSimplyNewR.begin(), vSimplyNewR.end(), SortRule::LitCmp);
            triAlg.newCla->LitPtr = new Literal*[uRsize];
            for (UINT16 i = 0; i < vSimplyNewR.size(); ++i) {
                Term* litSubT = triAlg.unify.GetNewTerm(vSimplyNewR[i]->subTerm, triAlg.newCla);
                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                    --uRsize;
                    continue;
                }
                if (triAlg.newCla->uFuncDepth < litSubT->uLevel)
                    triAlg.newCla->uFuncDepth = litSubT->uLevel;
                triAlg.newCla->uVarCount += litSubT->uVarCount;
                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, triAlg.newCla->uLitNum + 1), triAlg.newCla, nullptr);
                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->parentLitPtr = vSimplyNewR[i];
                if(triAlg.newCla ->deduceDeepth < (vSimplyNewR[i]->claPtr->deduceDeepth + 1)){
                    triAlg.newCla ->deduceDeepth = vSimplyNewR[i]->claPtr->deduceDeepth + 1;
                }
                 if (vSimplyNewR[i]->claPtr->isAxtom && triAlg.newCla->isAxtom)
                    triAlg.newCla->isAxtom = true;
                else
                    triAlg.newCla->isAxtom = false;
                if(triAlg.hasUnifGoalLit(fol,triAlg.newCla->LitPtr[triAlg.newCla->uLitNum])){
                    triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = 1;
                    if(triAlg.newCla->goaldeepth > 1)
                        triAlg.newCla->goaldeepth = 1;
                }
                else{
                    triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = vSimplyNewR[i]->goalDeepth;
                      if(triAlg.newCla->goaldeepth > vSimplyNewR[i]->goalDeepth)
                        triAlg.newCla->goaldeepth = vSimplyNewR[i]->goalDeepth;
                }
                ++(triAlg.newCla->uLitNum);
            }
            fol.initGenerateClaAttr(triAlg.newCla);
            uNewClaID++;
            if(fileNameFlag)
                FileOp::CreateRunInfoFilePre(30000);
            else
                FileOp::CreateRunInfoFile(30000);
            *FileOp::ofRealInfoOut << "从编号为" + to_string(cla->uClaId) + "子句开始进行等词归结演绎" << endl;
            for (UINT32 k = 0; k < vSimplyNewR.size(); k++) {
                triAlg.unify.WriteTriPath(vSimplyNewR[k]);
            }
            if (StrategyParam::orForCompition) {
                triAlg.newCla->addRForCompetiton();               
                string rStr = "";
                triAlg.newCla->toString(rStr);
                validPathOut[uNewClaID - 1].push_back(cla->uClaId);
                int begPos = rStr.find_first_of("[") + 1;
                int endPos = rStr.find_first_of("]");
                string rId = rStr.substr(begPos, endPos - begPos);
                string rContext = rStr.substr(rStr.find_first_of("]") + 1);
                string out = "cnf(" + rId + ",plain,\n" + "   ("+ rContext+"),\n"+ "   inference(equality_inference,[]," + "["+to_string(cla->uClaId) +"]"+ ")).";
                *FileOp::ofOutput << out << endl;
            }

            Clause* newCla = triAlg.newCla;
            if (StrategyParam::termRewriting != 2){
                    newCla->directEqn();
            }
            FileOp::WriteR(newCla);
            vector<UINT32>invaildLitId;
            string rundMsg; 
            Subsumption subsump; 
            bool isRundacyR = subsump.GenCla(fol,newCla, invaildLitId, rundMsg);
            FileOp::RInvaildInfoOut(rundMsg, StartId);
            if (isRundacyR) {
                newCla->DelLits(false);
                delete(newCla);
                triAlg.unify.ClearVarBind(chgVarT);
                vSimplyNewR.clear();
                vector<Literal*>().swap(vSimplyNewR);
                return 1;
            } else {
                preGenCla->push_back(newCla);
                if (newCla->uLitNum == 1)
                    if (StrategyParam::R_Weight == -1)
                        newCla->adeductNum = 0; 
                    else
                        newCla->adeductNum = StrategyParam::R_Weight;
                else
                    if (StrategyParam::R_Weight == -1)
                    newCla->pdeductNum = 0; 
                else
                    newCla->pdeductNum = StrategyParam::R_Weight; 
                fol.InsertCla(newCla);
                fol.setCla.insert(newCla);              
                if (StrategyParam::termRewriting != 2)
                    fol.judgeInsertSuper(newCla);
                if (1){
                    insertNoPairCla(fol, newCla);
                }
            }
            triAlg.unify.ClearVarBind(chgVarT);
            vSimplyNewR.clear();
            vector<Literal*>().swap(vSimplyNewR);
            return 2;
        }
    }
    else
        return 0;
} 

int Resolution::demodulatorClaSet(Formula& fol,TriAlg& triAlg,UINT32& uNewClaID,vector<Clause*>* preGenCla,UINT32 simplyIdStart,UINT32 simplyIdEnd){
    if(fol.eqnDemod.size() == 0){
        return 0;
    }
    set<Clause*>::iterator claP_it = fol.setCla.begin();
    for (; claP_it != fol.setCla.end();) {
         if(((*claP_it)->uClaId <= simplyIdEnd) && ((*claP_it)->uClaId >= simplyIdStart)) {
            if ((*claP_it)->isUintEqn()) {
                ++claP_it;
                continue;
            }
            int res = demodulatorCla(fol, triAlg, *claP_it, uNewClaID, preGenCla);
            set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
            if (delIt != fol.setOriClas.end()) {
                fol.setOriClas.erase(delIt);
            }
            set<Clause*> ::iterator delItNoPair = fol.noPairSingleCla.find(*claP_it);
            if (delItNoPair != fol.noPairSingleCla.end()) {
                fol.noPairSingleCla.erase(delItNoPair);
            }
            string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]被调解后删除";
            FileOp::LogOut(out);
            (*claP_it)->DelLits();
            delete((*claP_it));
            claP_it = fol.setCla.erase(claP_it);
         }
         else
             ++claP_it;
    }
    return 1;
}

int Resolution::demodulatorCla(Formula& fol,TriAlg& triAlg,Clause* cla,UINT32& uNewClaID,vector<Clause*>* preGenCla){
    bool flag = false;
    varBindTerm.clear();
    vector<Clause*> eqnLit;
    eqnLit.reserve(2);
    for(UINT32 i=0; i<cla->uLitNum; ++i){
        if(cla->LitPtr[i]->subTerm->isEqnTerm() == false){
            continue;
        }
        if(cla->LitPtr[i]->isConLit() == false){
            continue;
        }
        if(demodulatorLit(fol, triAlg,cla->LitPtr[i], uNewClaID, &eqnLit)){
            flag = true;
        }
    }
    if (!flag){
        triAlg.unify.ClearVarBind(varBindTerm);
        return 0;
    }
    else {
        vector<Literal*> vInsertNewR;
        vInsertNewR.reserve(6);
        for (UINT32 i = 0; i < cla->uLitNum; ++i)
            vInsertNewR.push_back(cla->LitPtr[i]);
        for (size_t rInd = 0; rInd < vInsertNewR.size() - 1; ++rInd) {
            Literal* rA = vInsertNewR[rInd];
            for (size_t rIndB = rInd + 1; rIndB < vInsertNewR.size();) {
                Literal* rB = vInsertNewR[rIndB];
                if (rA->subTerm->iTermCode == rB->subTerm->iTermCode && triAlg.unify.CheckEqn(rA->subTerm, rB->subTerm)) {
                    swap(vInsertNewR[rIndB], vInsertNewR[vInsertNewR.size() - 1]);
                    vInsertNewR.pop_back();
                } else
                    ++rIndB;
            }
        }
        Clause* newCla = nullptr;
        if (!vInsertNewR.empty()) {
            newCla = new Clause(uNewClaID, 0);
            newCla->isAxtom = true;
            size_t uRsize = vInsertNewR.size();
            if (uRsize > 1)
                stable_sort(vInsertNewR.begin(), vInsertNewR.end(), SortRule::LitCmp);
            newCla->LitPtr = new Literal*[uRsize];
            for (UINT16 i = 0; i < vInsertNewR.size(); ++i) {
                Term* litSubT = triAlg.unify.GetNewTermSuper(vInsertNewR[i]->subTerm, newCla); 
                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                    --uRsize;
                    continue;
                }
                if (newCla->uFuncDepth < litSubT->uLevel)newCla->uFuncDepth = litSubT->uLevel;
                newCla->uVarCount += litSubT->uVarCount;	
                newCla->LitPtr[newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, newCla->uLitNum + 1), newCla, nullptr);
                newCla->LitPtr[newCla->uLitNum]->parentLitPtr = vInsertNewR[i]; 
                if(newCla ->deduceDeepth < (vInsertNewR[i]->claPtr->deduceDeepth + 1)){
                    newCla ->deduceDeepth = vInsertNewR[i]->claPtr->deduceDeepth + 1;
                }
                if (vInsertNewR[i]->claPtr->isAxtom && newCla->isAxtom)
                    newCla->isAxtom = true;
                else
                    newCla->isAxtom = false;
                if (triAlg.hasUnifGoalLit(fol, newCla->LitPtr[newCla->uLitNum])) {
                    newCla->LitPtr[newCla->uLitNum]->goalDeepth = 1;
                    if (newCla->goaldeepth > 1)
                        newCla->goaldeepth = 1;
                } else {
                    newCla->LitPtr[newCla->uLitNum]->goalDeepth = vInsertNewR[i]->goalDeepth;
                    if (newCla->goaldeepth > vInsertNewR[i]->goalDeepth)
                        newCla->goaldeepth = vInsertNewR[i]->goalDeepth;
                }  
                ++(newCla->uLitNum);
            }
            fol.initGenerateClaAttr(newCla);
            ++uNewClaID;
            if (StrategyParam::termRewriting != 2) {
                newCla->directEqn();
            }
            vector<UINT32>invaildLitId;
            string rundMsg;
            Subsumption subsump; 
            bool isRundacyR = subsump.GenCla(fol,newCla, invaildLitId, rundMsg);
            if (isRundacyR) {
                newCla->DelLits(false);
                delete(newCla);
                triAlg.unify.ClearVarBind(varBindTerm);
                vInsertNewR.clear();
                vector<Literal*>().swap(vInsertNewR);
                return 1;
            } else {
                FileOp::CreateRunInfoFilePre(10000);
                *FileOp::ofRealInfoOut << "从编号为" + to_string(cla->uClaId) + "子句开始构建三角形" << endl;
                for (UINT8 k = 0; k < eqnLit.size(); ++k) {
                    triAlg.unify.WriteTriPath(eqnLit[k]->LitPtr[0]);
                }
                for (UINT32 i = 0; i < cla->uLitNum; ++i) {
                    triAlg.unify.WriteTriPath(cla->LitPtr[i]);
                }
                if (StrategyParam::orForCompition) {
                    newCla->addRForCompetiton();
                    
                    string rStr = "";
                    newCla->toString(rStr);
                    
                    string outClauses = "";
                    for (UINT8 k = 0; k < eqnLit.size(); ++k){
                            outClauses += to_string(eqnLit[k]->uClaId) + ",";
                            validPathOut[uNewClaID - 1].push_back(eqnLit[k]->uClaId);
                    }
                    outClauses += to_string(cla->uClaId);
                    validPathOut[uNewClaID - 1].push_back(cla->uClaId);
                    int begPos = rStr.find_first_of("[") + 1;
                    int endPos = rStr.find_first_of("]");
                    string rId = rStr.substr(begPos, endPos - begPos);
                    string rContext = rStr.substr(rStr.find_first_of("]") + 1);
                    string out = "cnf(" + rId + ",plain,\n" + "   ("+ rContext+"),\n" + "   inference(rewriting,[]," + "[" + outClauses +"]" + ")).";
                    *FileOp::ofOutput << out << endl;
                }
                FileOp::WriteR(newCla);
                rundMsg = "|[Y]";
                FileOp::RInvaildInfoOut(rundMsg, StartId);
                preGenCla->push_back(newCla);
                if (newCla->uLitNum == 1)
                    if (StrategyParam::R_Weight == -1)
                        newCla->adeductNum = 0; 
                    else
                        newCla->adeductNum = StrategyParam::R_Weight;
                else
                    if (StrategyParam::R_Weight == -1)
                       newCla->pdeductNum = 0; 
                   else
                       newCla->pdeductNum = StrategyParam::R_Weight;
                fol.InsertCla(newCla);
                fol.setCla.insert(newCla);
                if (StrategyParam::termRewriting != 2)
                    fol.judgeInsertSuper(newCla);
            }
            triAlg.unify.ClearVarBind(varBindTerm);
            vInsertNewR.clear();
            vector<Literal*>().swap(vInsertNewR);
            return 2;
        }
    }   
}

int Resolution::demodulatorClaSaturate(Formula& fol,TriAlg& triAlg,Clause* cla,UINT32& uNewClaID,vector<Clause*>* preGenCla){
    bool flag = false;
    varBindTerm.clear();
    vector<Clause*> eqnLit;
    eqnLit.reserve(2);
    for(UINT32 i=0; i<cla->uLitNum; ++i){
        
        if((StrategyParam::termRewriting == 0)&&(cla->LitPtr[i]->subTerm->isPLitTerm()))
            continue;
        if(demodulatorLitSaturate(fol, triAlg,cla->LitPtr[i], uNewClaID, &eqnLit)){
            flag = true;
        }
    }
    if (!flag){
        triAlg.unify.ClearVarBind(varBindTerm);
        return 0;
    }
    else {
        vector<Literal*> vInsertNewR;
        vInsertNewR.reserve(6);
        for (UINT32 i = 0; i < cla->uLitNum; ++i)
            vInsertNewR.push_back(cla->LitPtr[i]);
        for (size_t rInd = 0; rInd < vInsertNewR.size() - 1; ++rInd) {
            Literal* rA = vInsertNewR[rInd];
            for (size_t rIndB = rInd + 1; rIndB < vInsertNewR.size();) {
                Literal* rB = vInsertNewR[rIndB];
                if (rA->subTerm->iTermCode == rB->subTerm->iTermCode && triAlg.unify.CheckEqn(rA->subTerm, rB->subTerm)) {
                    swap(vInsertNewR[rIndB], vInsertNewR[vInsertNewR.size() - 1]);
                    vInsertNewR.pop_back();
                } else
                    ++rIndB;
            }
        }
        Clause* newCla = nullptr;
        if (!vInsertNewR.empty()) {
            newCla = new Clause(uNewClaID, 0);
            newCla->isAxtom = true;
            size_t uRsize = vInsertNewR.size();
            if (uRsize > 1)
                stable_sort(vInsertNewR.begin(), vInsertNewR.end(), SortRule::LitCmp);
            newCla->LitPtr = new Literal*[uRsize];
            for (UINT16 i = 0; i < vInsertNewR.size(); ++i) {
                Term* litSubT = triAlg.unify.GetNewTermSuper(vInsertNewR[i]->subTerm, newCla);
                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                    --uRsize;
                    continue;
                }
                if (newCla->uFuncDepth < litSubT->uLevel)newCla->uFuncDepth = litSubT->uLevel;
                newCla->uVarCount += litSubT->uVarCount;	
                newCla->LitPtr[newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, newCla->uLitNum + 1), newCla, nullptr);
                newCla->LitPtr[newCla->uLitNum]->parentLitPtr = vInsertNewR[i]; 
                if(newCla ->deduceDeepth < (vInsertNewR[i]->claPtr->deduceDeepth + 1)){
                    newCla ->deduceDeepth = vInsertNewR[i]->claPtr->deduceDeepth + 1;
                }
                if (vInsertNewR[i]->claPtr->isAxtom && newCla->isAxtom)
                    newCla->isAxtom = true;
                else
                    newCla->isAxtom = false;
                if (triAlg.hasUnifGoalLit(fol, newCla->LitPtr[newCla->uLitNum])) {
                    newCla->LitPtr[newCla->uLitNum]->goalDeepth = 1;
                    if (newCla->goaldeepth > 1)
                        newCla->goaldeepth = 1;
                } else {
                    newCla->LitPtr[newCla->uLitNum]->goalDeepth = vInsertNewR[i]->goalDeepth;
                    if (newCla->goaldeepth > vInsertNewR[i]->goalDeepth)
                        newCla->goaldeepth = vInsertNewR[i]->goalDeepth;
                }
                ++(newCla->uLitNum);
            }
            fol.initGenerateClaAttr(newCla);
            ++uNewClaID;
            if (StrategyParam::termRewriting != 2) {
                newCla->directEqn();
            }
            vector<UINT32>invaildLitId;
            string rundMsg; 
            Subsumption subsump; 
            bool isRundacyR = subsump.GenCla(fol,newCla, invaildLitId, rundMsg);
            if (isRundacyR) {
                newCla->DelLits(false);
                delete(newCla);
                triAlg.unify.ClearVarBind(varBindTerm);
                vInsertNewR.clear();
                vector<Literal*>().swap(vInsertNewR);
                return 1;
            } else {
                FileOp::CreateRunInfoFilePre(10000);
                *FileOp::ofRealInfoOut << "从编号为" + to_string(cla->uClaId) + "子句开始进行项调解" << endl;
                for (UINT8 k = 0; k < eqnLit.size(); ++k) {
                    triAlg.unify.WriteTriPath(eqnLit[k]->LitPtr[0]);
                }
                for (UINT32 i = 0; i < cla->uLitNum; ++i) {
                    triAlg.unify.WriteTriPath(cla->LitPtr[i]);
                }
                if (StrategyParam::orForCompition) {
                    newCla->addRForCompetiton();                   
                    string rStr = "";
                    newCla->toString(rStr);
                    string outClauses = "";
                    for (UINT8 k = 0; k < eqnLit.size(); ++k){
                            outClauses += to_string(eqnLit[k]->uClaId) + ",";
                            validPathOut[uNewClaID - 1].push_back(eqnLit[k]->uClaId);
                    }
                    outClauses += to_string(cla->uClaId);
                    validPathOut[uNewClaID - 1].push_back(cla->uClaId);
                    int begPos = rStr.find_first_of("[") + 1;
                    int endPos = rStr.find_first_of("]");
                    string rId = rStr.substr(begPos, endPos - begPos);
                    string rContext = rStr.substr(rStr.find_first_of("]") + 1);
                    string out = "cnf(" + rId + ",plain,\n" + "   ("+ rContext+"),\n" + "   inference(rewriting,[]," + "[" + outClauses +"]" + ")).";
                    *FileOp::ofOutput << out << endl;
                }
                FileOp::WriteR(newCla);
                rundMsg = "|[Y]";
                FileOp::RInvaildInfoOut(rundMsg, StartId);
                preGenCla->push_back(newCla);
                if (newCla->uLitNum == 1)
                    if (StrategyParam::R_Weight == -1)
                        newCla->adeductNum = 0;
                    else
                        newCla->adeductNum = StrategyParam::R_Weight;
                else
                    if (StrategyParam::R_Weight == -1)
                       newCla->pdeductNum = 0; 
                   else
                       newCla->pdeductNum = StrategyParam::R_Weight; 
                fol.InsertCla(newCla);
                fol.setCla.insert(newCla);
                if (StrategyParam::termRewriting != 2)
                    fol.judgeInsertSuper(newCla);
            }
            triAlg.unify.ClearVarBind(varBindTerm);
            vInsertNewR.clear();
            vector<Literal*>().swap(vInsertNewR);
            return 2;
        }
    }   
}

int Resolution::demodulatorLit(Formula& fol,TriAlg& triAlg,Literal* lit,UINT32& uNewClaID,vector<Clause*>* eqnLit){
    bool flag = false;
    vector<Term*> termVec;
    termVec.reserve(6);
    lit->subTerm->recordTerm(&termVec);
    set<Clause*>::iterator claP_it = fol.eqnDemod.begin();
    assert(termVec.size() >0);
    UINT8 funCount = 0;
    for(UINT32 i=1; i< termVec.size(); ++i) {
        if(termVec[i]->isVar())
            continue;
        else if(termVec[i]->vBinding){
            if(termVec[i]->isFuncTerm()){
                funCount = termVec[i]->uSubTermNum;
            }
        }
        else if(termVec[i]->isFuncTerm()){
            if(demodulatorTerm(fol, triAlg,termVec[i], uNewClaID, eqnLit)){
                flag = true;
                funCount = termVec[i]->uSubTermNum;
            }
        }
        else{
            if(demodulatorTerm(fol, triAlg,termVec[i], uNewClaID, eqnLit)){
                flag = true;
            }
        }
        if(funCount != 0){
            i += funCount;
            funCount = 0;
        }
    }
    if(flag)
        return 1;
    else
        return 0;
}

int Resolution::demodulatorLitSaturate(Formula& fol,TriAlg& triAlg,Literal* lit,UINT32& uNewClaID,vector<Clause*>* eqnLit){
    bool flag = false;
    vector<Term*> termVec;
    termVec.reserve(6);
    lit->subTerm->recordTerm(&termVec);
    assert(termVec.size() >0);
    UINT8 funCount = 0;
    for(UINT32 i=1; i< termVec.size(); ++i) {
        if(termVec[i]->isVar())
            continue;
        else if(termVec[i]->vBinding){
            if(termVec[i]->isFuncTerm()){
                funCount = termVec[i]->uSubTermNum;
            }
        }
        else if(termVec[i]->isFuncTerm()){
            if(demodulatorTermSaturate(fol, triAlg,termVec[i], uNewClaID, eqnLit)){
                flag = true;
                funCount = termVec[i]->uSubTermNum;
            }
        }
        else{
            continue;
        }
        if(funCount != 0){
            i += funCount;
            funCount = 0;
        }
    }
    if(flag)
        return 1;
    else
        return 0;
}

int Resolution::demodulatorTerm(Formula& fol,TriAlg& triAlg,Term* term,UINT32& uNewClaID,vector<Clause*>* eqnLit){
    bool flag = false; 
    vector<Term*> chgVarT;
    vector<Term*> chgVarTEqn;
    chgVarT.reserve(6);
    chgVarTEqn.reserve(6);
    vector<Literal*> vSimplyNewR;
    vSimplyNewR.reserve(6);
    UINT32 termCount = 0;
    UINT32 demodulCount = 0;
    term->countTerm(termCount);
    if(term->isFuncTerm()){
        set<Clause*>::iterator claP_it = fol.eqnDemod.begin();
        for (; claP_it != fol.eqnDemod.end();) {
            Term* demodulGoal = (*claP_it)->LitPtr[0]->subTerm->subTermPtr[0];
            demodulCount = 0;
            demodulGoal->countTerm(demodulCount);
            if(demodulGoal->isVar()){
                if(triAlg.unify.TermMguEqn(term,demodulGoal,chgVarT,chgVarTEqn)) {
                    if (triAlg.unify.GetBindingT(demodulGoal) == ((*claP_it)->LitPtr[0]->subTerm->subTermPtr[1])) {
                        triAlg.unify.ClearVarBind(chgVarT);
                        triAlg.unify.ClearVarBind(chgVarTEqn);
                    }
                    else
                    {
                        if (find(eqnLit->begin(), eqnLit->end(), *claP_it) == eqnLit->end())
                            eqnLit->push_back(*claP_it);
                        varBindTerm.insert(varBindTerm.end(), chgVarT.begin(), chgVarT.end());
                        varBindTerm.insert(varBindTerm.end(), chgVarTEqn.begin(), chgVarTEqn.end());
                        chgVarT.clear();
                        term->vBinding = (*claP_it)->LitPtr[0]->subTerm->subTermPtr[1];
                        varBindTerm.push_back(term);
                        flag = true;
                        break;
                    }
                }
                else{
                    triAlg.unify.ClearVarBind(chgVarT);
                    triAlg.unify.ClearVarBind(chgVarTEqn);
                }
            }
            else if((term->iTermCode == demodulGoal->iTermCode) &&(termCount == demodulCount ) && term->similarTerm(demodulGoal)){
                if(triAlg.unify.pairFuncTermMguEqn(term,demodulGoal,chgVarT,chgVarTEqn)) {
                    if (triAlg.unify.GetBindingT(demodulGoal) == ((*claP_it)->LitPtr[0]->subTerm->subTermPtr[1])) {
                        triAlg.unify.ClearVarBind(chgVarT);
                        triAlg.unify.ClearVarBind(chgVarTEqn);
                    } else {
                        if (find(eqnLit->begin(), eqnLit->end(), *claP_it) == eqnLit->end())
                            eqnLit->push_back(*claP_it);
                        varBindTerm.insert(varBindTerm.end(), chgVarT.begin(), chgVarT.end());
                        varBindTerm.insert(varBindTerm.end(), chgVarTEqn.begin(), chgVarTEqn.end());
                        chgVarT.clear();
                        term->vBinding = (*claP_it)->LitPtr[0]->subTerm->subTermPtr[1];
                        varBindTerm.push_back(term);
                        flag = true;
                        break;
                    }
                } 
                else{
                    triAlg.unify.ClearVarBind(chgVarT);
                    triAlg.unify.ClearVarBind(chgVarTEqn);
                }
            }
            ++claP_it;
        }
    }
    else{
        set<Clause*>::iterator claP_it = fol.eqnDemod.begin();
        for (; claP_it != fol.eqnDemod.end();) {
            Term* demodulGoal = (*claP_it)->LitPtr[0]->subTerm->subTermPtr[0];
            if(demodulGoal->isVar()){
                if(triAlg.unify.TermMguEqn(term,demodulGoal,chgVarT,chgVarTEqn)) {
                    if (triAlg.unify.GetBindingT(demodulGoal) == ((*claP_it)->LitPtr[0]->subTerm->subTermPtr[1])) {
                        triAlg.unify.ClearVarBind(chgVarT);
                        triAlg.unify.ClearVarBind(chgVarTEqn);
                    } else {
                        if (find(eqnLit->begin(), eqnLit->end(), *claP_it) == eqnLit->end())
                            eqnLit->push_back(*claP_it);
                        varBindTerm.insert(varBindTerm.end(), chgVarT.begin(), chgVarT.end());
                        varBindTerm.insert(varBindTerm.end(), chgVarTEqn.begin(), chgVarTEqn.end());
                        chgVarT.clear();
                        term->vBinding = (*claP_it)->LitPtr[0]->subTerm->subTermPtr[1];
                        varBindTerm.push_back(term);
                        flag = true;
                        break;
                    }
                }
                else{
                    triAlg.unify.ClearVarBind(chgVarT);
                    triAlg.unify.ClearVarBind(chgVarTEqn);
                }
            }
            else if((term->iTermCode == demodulGoal->iTermCode)  &&(termCount == demodulCount )&& term->similarTerm(demodulGoal)){
                if(triAlg.unify.TermMguEqn(term,demodulGoal,chgVarT,chgVarTEqn)) {
                    if (triAlg.unify.GetBindingT(demodulGoal) == ((*claP_it)->LitPtr[0]->subTerm->subTermPtr[1])) {
                        triAlg.unify.ClearVarBind(chgVarT);
                        triAlg.unify.ClearVarBind(chgVarTEqn);
                    } else {
                        if (find(eqnLit->begin(), eqnLit->end(), *claP_it) == eqnLit->end())
                            eqnLit->push_back(*claP_it);
                        varBindTerm.insert(varBindTerm.end(), chgVarT.begin(), chgVarT.end());
                        varBindTerm.insert(varBindTerm.end(), chgVarTEqn.begin(), chgVarTEqn.end());
                        chgVarT.clear();
                        term->vBinding = (*claP_it)->LitPtr[0]->subTerm->subTermPtr[1];
                        varBindTerm.push_back(term);
                        flag = true;
                        break;
                    }
                }  
                else{
                    triAlg.unify.ClearVarBind(chgVarT);
                    triAlg.unify.ClearVarBind(chgVarTEqn);
                }
            }
            ++claP_it;
        }
    }
    if(flag)
        return 1;
    else
        return 0;
}

int Resolution::demodulatorTermSaturate(Formula& fol,TriAlg& triAlg,Term* term,UINT32& uNewClaID,vector<Clause*>* eqnLit){
    bool flag = false;   
    vector<Term*> chgVarT;
    vector<Term*> chgVarTEqn;
    chgVarT.reserve(6);
    chgVarTEqn.reserve(6);
    vector<Literal*> vSimplyNewR;
    vSimplyNewR.reserve(6);
    UINT32 termCount = 0;
    UINT32 varTermCount = 0;
    term->countTerm(termCount);
    term->countVarTerm(varTermCount);
    vector<INT32> saveTerm;
    saveTerm.reserve(6);
    UINT32 varUqTermCount = 0;
    term->countUqVarTerm(varUqTermCount,saveTerm);
    saveTerm.clear();
    vector<INT32>().swap(saveTerm);
    if((termCount == 1))
        return 0;
    set<Clause*>::iterator claP_it = fol.eqnDemod.begin();
    for (; claP_it != fol.eqnDemod.end();) {
        if((*claP_it)->LitPtr == nullptr){
             ++claP_it;
            continue;
        }
        Term* demodulGoal = (*claP_it)->LitPtr[0]->subTerm->subTermPtr[0];
        UINT32 termGoalCount = 0;
        UINT32 varTermGoalCount = 0;
        demodulGoal->countTerm(termGoalCount);
        demodulGoal->countVarTerm(varTermGoalCount);
        vector<INT32> saveDemodulGoal;
        saveDemodulGoal.reserve(6);
        UINT32 varUqDemodulGoalCount = 0;
        demodulGoal->countUqVarTerm(varUqDemodulGoalCount, saveDemodulGoal);
        saveDemodulGoal.clear();
        vector<INT32>().swap(saveDemodulGoal);
        if((term->isFuncTerm()) &&(demodulGoal->isFuncTerm()) && (term->iTermCode != demodulGoal->iTermCode))
        {
            ++claP_it;
            continue;
        }     
        if(termCount < termGoalCount)
        {
            ++claP_it;
            continue;
        }
        bool demod = true;
        if(varTermCount == 0){
            demod = true;
        }
        else if(varUqTermCount > varUqDemodulGoalCount){
            demod = false;
        }
        if (demod && term->similarTerm(demodulGoal)) {
            if (triAlg.unify.pairFuncTermMguEqn(term, demodulGoal, chgVarT, chgVarTEqn)) {
                if (triAlg.unify.GetBindingT(demodulGoal) == ((*claP_it)->LitPtr[0]->subTerm->subTermPtr[1])) {
                    triAlg.unify.ClearVarBind(chgVarT);
                    triAlg.unify.ClearVarBind(chgVarTEqn);
                } else {
                    if (find(eqnLit->begin(), eqnLit->end(), *claP_it) == eqnLit->end())
                        eqnLit->push_back(*claP_it);
                    varBindTerm.insert(varBindTerm.end(), chgVarT.begin(), chgVarT.end());
                    varBindTerm.insert(varBindTerm.end(), chgVarTEqn.begin(), chgVarTEqn.end());
                    chgVarT.clear();
                    chgVarTEqn.clear();
                    term->vBinding = (*claP_it)->LitPtr[0]->subTerm->subTermPtr[1];
                    varBindTerm.push_back(term);
                    flag = true;
                    break;
                }
            }
            else {
                triAlg.unify.ClearVarBind(chgVarT);
                triAlg.unify.ClearVarBind(chgVarTEqn);
            }
        }
        ++claP_it;
    }  
    if(flag)
        return 1;
    else
        return 0;
}

double Resolution::timeConsume(TriAlg& triAlg){
    clock_t endNoPair = clock();
    double time_sNoPair = (double) (endNoPair - triAlg.start) / (double) CLOCKS_PER_SEC;
    return time_sNoPair;
}

RESULT Resolution::BaseAlgByRecodePath(Formula& fol, UINT64 &triTotalTimes) {
    shift = 0;
    TriAlg triAlg;
    UINT32 uNewClaID = fol.uOrigClaSize + 1;
    triAlg.singleClaNum = fol.uOrigClaSize;
    triAlg.flagT = true;
    string str = "";
    BACKTYPE backType = NOBACK;
    vector<Clause*> preGenCla;
    preGenCla.reserve(6);
    vector<Clause*> chgCla;
    chgCla.reserve(6);
    triAlg.start = StrategyParam::start;
    Clause* givenClaP = nullptr;
    vector<Literal*> unifyLit;
    unifyLit.reserve(16);
    set<UINT32> saveClaId;
    if((StrategyParam::lockType != 0) && 0){
        set<Clause*>::iterator claP_it = fol.setCla.begin();
        string lockOutTotal;
        for (; claP_it != fol.setCla.end();) {           
            if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) > 30) {
                break;
            }
            bool pureLit = lockResSimlifyNew(fol, triAlg, *claP_it, uNewClaID);
            string lockOut;
            if((lockClaVec.size())>0 &&(fol.setOriClas.size() >1)){
                if(pureLit)
                    lockOut = "纯文字;";
                else {
                    if (StrategyParam::lockType == 1)
                        lockOut = "锁子句处理;";
                    else if (StrategyParam::lockType == 2)
                        lockOut = "锁+包含处理;";
                    else if (StrategyParam::lockType == 3)
                        lockOut = "锁+包含2处理;";
                    else if (StrategyParam::lockType == 4)
                        lockOut = "锁+不对称文字+恒真子句处理;";
                    else if (StrategyParam::lockType == 5)
                        lockOut = "锁+不对称文字+包含子句处理;";
                }
                lockOut = lockOut + "子句编号为:";
                for (int m = 0; m < lockClaVec.size(); ++m) {
                    lockOut = lockOut + to_string(lockClaVec[m]->uClaId) + "\n";
                    set<Clause*> ::iterator delIt = fol.setOriClas.find(lockClaVec[m]);
                    if (delIt != fol.setOriClas.end()) {
                        fol.setOriClas.erase(delIt);
                    }
                    (lockClaVec[m])->DelLits();
                    delete(lockClaVec[m]);
                    claP_it = fol.setCla.erase(claP_it);                   
                    string out = "#子句编号[" + to_string(lockClaVec[m]->uClaId) + "]为锁子句被删除";
                    FileOp::LogOut(out);
                }
                lockClaVec.clear();
                lockOutTotal += lockOut;
            }
            else
                ++claP_it;
        }
        *FileOp::ofJudgeAllFileResult << FileOp::JudgeFileName << lockOutTotal << endl;
    }
    if (StrategyParam::delPureCla && StrategyParam::problemDifficult && !StrategyParam::preprocess) {
        set<Clause*>::iterator claP_it = fol.setCla.begin();
        for (; claP_it != fol.setCla.end();) {
            if (triAlg.claHasPureLit(fol, (*claP_it))) {
                set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
                if (delIt != fol.setOriClas.end()) {
                    fol.setOriClas.erase(delIt);
                }
                string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]含纯文字被删除";
                FileOp::LogOut(out);
                (*claP_it)->DelLits();
                delete((*claP_it));
                claP_it = fol.setCla.erase(claP_it);
            } 
            else{
                if((*claP_it)->isGoal){
                    for(int i=0;i<(*claP_it)->uLitNum;++i){
                        unifyLit.push_back((*claP_it)->LitPtr[i]);
                    }
                    saveClaId.insert((*claP_it)->uClaId);
                    fol.saveGoalClaSet.insert(*claP_it);  
                    insertNoPairCla(fol, *claP_it);
                }
                ++claP_it;
            }
        }
    }
    else if(StrategyParam::problemDifficult){
        set<Clause*>::iterator claP_it = fol.setCla.begin();
        for (; claP_it != fol.setCla.end();) {
            if ((*claP_it)->isGoal) {
                for (int i = 0; i < (*claP_it)->uLitNum; ++i) {
                    unifyLit.push_back((*claP_it)->LitPtr[i]);
                }
                saveClaId.insert((*claP_it)->uClaId);
                fol.saveGoalClaSet.insert(*claP_it);
                insertNoPairCla(fol, *claP_it);
            }
            ++claP_it;
        }
    }
    if(StrategyParam::goalCla && StrategyParam::problemDifficult && (unifyLit.size()>0)){
        if(StrategyParam::preprocess){
            StrategyParam::start = clock();
        }            
        vector<Literal*>* vCandLit;
        Literal* candLitP = nullptr;
        Clause* candClaP = nullptr;
        vector<Term*> chgVarTemp;
        chgVarTemp.reserve(6);
        int length = unifyLit.size();
        bool flag = true;
        int resLen = 0;
        int reslength = 0;
        for(int i=0;i<unifyLit.size();++i){        
            unifyLit[i]->goalDeepth = reslength;
            if(unifyLit[i]->claPtr->goaldeepth > reslength){
                unifyLit[i]->claPtr->goaldeepth = reslength;
            }
            if(flag){
                flag = false;
                resLen = unifyLit.size();
            }
            if(i == resLen){
                ++reslength;
                resLen = unifyLit.size();
                 if(reslength == StrategyParam::goalDeepth){
                     break;
                 }
            }
            if(saveClaId.size() == (fol.setCla.size() + fol.setEnq.size())){
                break;
            }
            vCandLit = unifyLit[i]->getPairPredLst();
            for (UINT32 pasLitInd = 0; pasLitInd < UINT32(vCandLit->size()); ++pasLitInd)
            {
                candLitP = vCandLit->at(pasLitInd);
                candClaP = candLitP->claPtr;
                if (saveClaId.find(candClaP->uClaId) != saveClaId.end())
                    continue;
                if (triAlg.unify.LitMgu(unifyLit[i], candLitP, chgVarTemp)) {
                    triAlg.unify.ClearVarBind(chgVarTemp);                
                    for(int m=0;m<candClaP->uLitNum;++m){
                        if(find(unifyLit.begin(),unifyLit.end(),candClaP->LitPtr[m]) ==unifyLit.end()){
                            unifyLit.push_back(candClaP->LitPtr[m]);
                        }
                    }
                    if (saveClaId.find(candClaP->uClaId) == saveClaId.end())
                        saveClaId.insert(candClaP->uClaId);
                }
                else
                    triAlg.unify.ClearVarBind(chgVarTemp);
            }
            triAlg.unify.ClearVarBind(chgVarTemp);
        }
        triAlg.unify.ClearVarBind(chgVarTemp);
        vector<Term*>().swap(chgVarTemp);
        unifyLit.clear();
        vector<Literal*>().swap(unifyLit);
        set<Clause*>::iterator claP_it = fol.setCla.begin();
        for (; claP_it != fol.setCla.end();) {
            if(saveClaId.find((*claP_it)->uClaId) == saveClaId.end()){
                set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
                if (delIt != fol.setOriClas.end()) {
                    fol.setOriClas.erase(delIt);
                }
                string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]目标子句约简被删除";
                FileOp::LogOut(out);
                (*claP_it)->DelLits();
                delete((*claP_it));
                claP_it = fol.setCla.erase(claP_it);
            }
            else{
                if (StrategyParam::termRewriting != 2) {
                    if (((*claP_it)->isUintEqn()) && ((*claP_it)->LitPtr[0]->subTerm->suitSuper())) {
                        (*claP_it)->demodulator = true;
                        fol.eqnDemod.insert(*claP_it); 
                    }
                }               
                ++claP_it;
            }
        }
        set<Clause*>::iterator claPE_it = fol.setEnq.begin();
        for (; claPE_it != fol.setEnq.end();) {
            if (saveClaId.find((*claPE_it)->uClaId) == saveClaId.end()) {
                set<Clause*> ::iterator delIt = fol.setOriClas.find(*claPE_it);
                if (delIt != fol.setOriClas.end()) {
                    fol.setOriClas.erase(delIt);
                }
                string out = "#子句编号[" + to_string((*claPE_it)->uClaId) + "]目标子句约简被删除";
                FileOp::LogOut(out);
                (*claPE_it)->DelLits();
                delete((*claPE_it));
                claPE_it = fol.setEnq.erase(claPE_it);
            } else{
                if (StrategyParam::termRewriting != 2) {
                    if (((*claPE_it)->isUintEqn()) && ((*claPE_it)->LitPtr[0]->subTerm->suitSuper())) {
                        (*claPE_it)->demodulator = true;
                        fol.eqnDemod.insert(*claPE_it); 
                    }
                } 
                ++claPE_it;
            }
        }           
    }
    saveClaId.clear();
    if(StrategyParam::preprocess){
        clock_t endNoPair = clock();
        double time_sNoPair = (double) (endNoPair - StrategyParam::start) / (double) CLOCKS_PER_SEC;
        if (time_sNoPair > StrategyParam::timeLimit * 60)
        {
            FileOp::outReleaseOf();
            string str = "";
            fol.uOrigClaSingleSize = 0;
            str += fol.ToSingleString();
            FileOp::PrintINFOEndRSinglePre(str); 
            FileOp::LogOut("超出时间" + to_string(StrategyParam::timeLimit) + "分钟限制,预处理失败！");
            return UNKNOWN;
        }
        else{
            FileOp::outReleaseOf();
            string str = "";
            fol.uOrigClaSingleSize = 0;
            str += fol.ToSingleString();
            FileOp::PrintINFOEndRSinglePre(str);
            FileOp::LogOut("使用时间" + to_string(time_sNoPair) + "秒,预处理成功！");
            return UNKNOWN;
        }
    }
    UINT16 loop = 10;
    bool chooseTri = true;
    bool firstSwitch = true;
    int maxUintClaId = 0;
    int maxUnitId = 0;
    bool readStrategySwitch1 = true;
    bool readStrategySwitch2 = true;
    bool readStrategySwitch3 = true;
    bool readStrategySwitch4 = true;
    UINT32 simplyIdStart = 0;
    UINT32 simplyIdEnd = uNewClaID - 1;
    bool insertNoUseCla = true;
    triAlg.start = clock();
    bool readSecondStrategy = true;
    bool readThirdStrategy = true; 
    while(1) {
        triAlg.vOnlyOutTri.clear();
        triAlg.vOutTri.clear();
        triAlg.vOnlyOutTriCmp.clear();
        triAlg.delSaveCopyCla(fol);
        triAlg.BackPathClear();
        fol.setSingleCla.clear();
        simplyIdEnd= uNewClaID - 1;
        if((FileOp::strategyMode != 3)&&(FileOp::strategyMode != 4))
        if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::timeLimit *12) {
            StrategyParam::funcFixDeepth = StrategyParam::funcOrgFixDeepthMax + 10;
        } 
        else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::timeLimit *24) {
            StrategyParam::funcFixDeepth = StrategyParam::funcOrgFixDeepthMax + StrategyParam::funcOrgFixDeepthLessMax + 10;
        } 
        else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::timeLimit *36) {
            StrategyParam::funcFixDeepth = StrategyParam::funcOrgFixDeepthMax + 20;
        } 
        else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::timeLimit *48) {
            StrategyParam::funcFixDeepth = StrategyParam::funcOrgFixDeepthMax * 2 + 20;
        } 
        else
            StrategyParam::funcFixDeepth = 100;
        if (StrategyParam::findStr == false) {
            if ((FileOp::strategyMode == 1) && 0) {
                int timeUsed = (int) timeConsume(triAlg);
                if (timeUsed > (StrategyParam::timeLimitStrategies * 12)) {
                    if (timeUsed < (StrategyParam::timeLimitStrategies * 24)) {
                        if (readSecondStrategy) {
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[1]);
                            shift = 0;
                            if (StrategyParam::noUnitRes == false) {
                                chooseTri = true;
                            }
                            StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                            readSecondStrategy = false;
                        }
                    } else if (timeUsed < (StrategyParam::timeLimitStrategies * 36)) {
                        if (readThirdStrategy) {
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[2]);
                            shift = 0;
                            if (StrategyParam::noUnitRes == false) {
                                chooseTri = true;
                            }
                            StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                            readThirdStrategy = false;
                        }

                    } else if ((timeUsed > 1) && (timeUsed % 5 == 0)) {
                        if ((timeUsed % 5) < 150) {
                            if (timeUsed / 5 >= FileOp::StrategyfileName.size())
                                FileOp::readBatchStrategy(FileOp::StrategyfileName[FileOp::StrategyfileName.size() - 1]);
                            else
                                FileOp::readBatchStrategy(FileOp::StrategyfileName[timeUsed / 5]);
                        } else
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[0]);
                        shift = 0;
                        if (StrategyParam::noUnitRes == false) {
                            chooseTri = true;
                        }
                        StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                    }
                }
            } else if ((FileOp::strategyMode == 1) && 0) {
                int timeUsed = (int) timeConsume(triAlg);
                if ((timeUsed > 1) && (timeUsed % 5 == 0) && readSecondStrategy && readThirdStrategy) {
                    if ((timeUsed % 5) < 150)
                        if (timeUsed / 5 >= FileOp::StrategyfileName.size())
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[FileOp::StrategyfileName.size() - 1]);
                        else
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[timeUsed / 5]);
                    else
                        FileOp::readBatchStrategy(FileOp::StrategyfileName[0]);
                    shift = 0;
                    if (StrategyParam::noUnitRes == false) {
                        chooseTri = true;
                    }
                    StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                }
                if (timeUsed > (StrategyParam::timeLimitStrategies * 24)) {
                    if (timeUsed < (StrategyParam::timeLimitStrategies * 36)) {
                        if (readSecondStrategy) {
                            if (0) {
                                fol.DelSubSet(fol.uOrigClaSize);
                                FileOp::delRes();
                                uNewClaID = fol.uOrigClaSize + 1;
                                StrategyParam::timeLimit = 5;
                                firstSwitch = true;
                                fol.noPairSingleCla.clear();
                                triAlg.noCmpCla.clear();
                                triAlg.flagT = true;
                                maxUintClaId = 0;
                                maxUnitId = 0;
                                simplyIdStart = 0;
                                simplyIdEnd = uNewClaID - 1;
                            }
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[1]);
                            shift = 0;
                            if (StrategyParam::noUnitRes == false) {
                                chooseTri = true;
                            }
                            StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                            readSecondStrategy = false;
                        }
                    } else if (timeUsed < (StrategyParam::timeLimitStrategies * 48)) {
                        if (readThirdStrategy) {
                            if (0) {
                                fol.DelSubSet(fol.uOrigClaSize);
                                FileOp::delRes();
                                uNewClaID = fol.uOrigClaSize + 1;
                                StrategyParam::timeLimit = 5;
                                firstSwitch = true;
                                fol.noPairSingleCla.clear();
                                triAlg.noCmpCla.clear();
                                triAlg.flagT = true;
                                maxUintClaId = 0;
                                maxUnitId = 0;
                                simplyIdStart = 0;
                                simplyIdEnd = uNewClaID - 1;
                            }
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[2]);
                            shift = 0;
                            if (StrategyParam::noUnitRes == false) {
                                chooseTri = true;
                            }
                            StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                            readThirdStrategy = false;
                        }
                    } else if ((timeUsed > 1) && (timeUsed % 5 == 0)) {
                        if ((timeUsed % 5) < 150)
                            if (timeUsed / 5 >= FileOp::StrategyfileName.size())
                                FileOp::readBatchStrategy(FileOp::StrategyfileName[FileOp::StrategyfileName.size() - 1]);
                            else
                                FileOp::readBatchStrategy(FileOp::StrategyfileName[timeUsed / 5]);
                        else
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[0]);
                        shift = 0;
                        if (StrategyParam::noUnitRes == false) {
                            chooseTri = true;
                        }
                        StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                    }
                }
            } else if ((FileOp::strategyMode == 1) && 0) {
                int timeUsed = (int) timeConsume(triAlg);
                if ((timeUsed > 1) && (timeUsed % 5 == 0) && readSecondStrategy && readThirdStrategy) {
                    if ((timeUsed % 5) < 150)
                        if (timeUsed / 5 >= FileOp::StrategyfileName.size())
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[FileOp::StrategyfileName.size() - 1]);
                        else
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[timeUsed / 5]);
                    else
                        FileOp::readBatchStrategy(FileOp::StrategyfileName[0]);
                    shift = 0;
                    if (StrategyParam::noUnitRes == false) {
                        chooseTri = true;
                    }
                    StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                }
                if (timeUsed > (StrategyParam::timeLimitStrategies * 6)) {
                    if (timeUsed < (StrategyParam::timeLimitStrategies * 9)) {
                        if (readSecondStrategy) {
                            if (0) {
                                fol.DelSubSet(fol.uOrigClaSize);
                                FileOp::delRes();
                                uNewClaID = fol.uOrigClaSize + 1;
                                StrategyParam::timeLimit = 5;
                                firstSwitch = true;
                                fol.noPairSingleCla.clear();
                                triAlg.noCmpCla.clear();
                                triAlg.flagT = true;
                                maxUintClaId = 0;
                                maxUnitId = 0;
                                simplyIdStart = 0;
                                simplyIdEnd = uNewClaID - 1;
                            }
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[1]);
                            shift = 0;
                            if (StrategyParam::noUnitRes == false) {
                                chooseTri = true;
                            }
                            StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                            readSecondStrategy = false;
                        }
                    } else if (timeUsed < (StrategyParam::timeLimitStrategies * 12)) {
                        if (readThirdStrategy) {
                            if (0) {
                                fol.DelSubSet(fol.uOrigClaSize);
                                FileOp::delRes();
                                uNewClaID = fol.uOrigClaSize + 1;
                                StrategyParam::timeLimit = 5;
                                firstSwitch = true;
                                fol.noPairSingleCla.clear();
                                triAlg.noCmpCla.clear();
                                triAlg.flagT = true;
                                maxUintClaId = 0;
                                maxUnitId = 0;
                                simplyIdStart = 0;
                                simplyIdEnd = uNewClaID - 1;
                            }
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[2]);
                            shift = 0;
                            if (StrategyParam::noUnitRes == false) {
                                chooseTri = true;
                            }
                            StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                            readThirdStrategy = false;
                        }
                    } else if ((timeUsed > 1) && (timeUsed % 5 == 0)) {
                        if ((timeUsed % 5) < 150)
                            if (timeUsed / 5 >= FileOp::StrategyfileName.size())
                                FileOp::readBatchStrategy(FileOp::StrategyfileName[FileOp::StrategyfileName.size() - 1]);
                            else
                                FileOp::readBatchStrategy(FileOp::StrategyfileName[timeUsed / 5]);
                        else
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[0]);
                        shift = 0;
                        if (StrategyParam::noUnitRes == false) {
                            chooseTri = true;
                        }
                        StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                    }
                }
            } else if ((FileOp::strategyMode == 1) && 1) {
                int timeUsed = (int) timeConsume(triAlg);
                if (timeUsed > (StrategyParam::timeLimitStrategies * 12)) {
                    if (timeUsed < (StrategyParam::timeLimitStrategies * 24)) {
                        if (readSecondStrategy) {
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[1]);
                            shift = 0;
                            if (StrategyParam::noUnitRes == false) {
                                chooseTri = true;
                            }
                            StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                            readSecondStrategy = false;
                        }
                    } else if (timeUsed < (StrategyParam::timeLimitStrategies * 36)) {
                        if (readThirdStrategy) {
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[2]);
                            shift = 0;
                            if (StrategyParam::noUnitRes == false) {
                                chooseTri = true;
                            }
                            StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                            readThirdStrategy = false;
                        }
                    } else if ((timeUsed > 1) && (timeUsed % ((int) (StrategyParam::timeLimitStrategies) * 5) == 0)) {
                        if ((timeUsed / ((int) (StrategyParam::timeLimitStrategies) * 5)) < 120)
                            if ((timeUsed / ((int) (StrategyParam::timeLimitStrategies) * 5)) >= FileOp::StrategyfileName.size())
                                FileOp::readBatchStrategy(FileOp::StrategyfileName[FileOp::StrategyfileName.size() - 1]);
                            else
                                FileOp::readBatchStrategy(FileOp::StrategyfileName[timeUsed / ((int) (StrategyParam::timeLimitStrategies) * 5)]);
                        else
                            FileOp::readBatchStrategy(FileOp::StrategyfileName[0]);
                        shift = 0;
                        if (StrategyParam::noUnitRes == false) {
                            chooseTri = true;
                        }
                        StrategyParam::timeLimit = StrategyParam::timeLimitStrategies;
                    }
                }
            }
        }
        if((FileOp::strategyMode == 1) && 0){
            if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < 75) {
                StrategyParam::timeLimit = 5;
            } 
            else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < 130) {
                if(readStrategySwitch1){
                    shift = 0;
                    readStrategySwitch1= false;
                    FileOp::readBatchStrategy(FileOp::StrategyfileName[1]);                  
                    StrategyParam::timeLimit = 5;
                    if(StrategyParam::noUnitRes == false){
                        chooseTri = true;
                    }
                    if(StrategyParam::strategyMode ==0){
                        fol.DelSubSet(fol.uOrigClaSize);
                        FileOp::delRes();
                        uNewClaID = fol.uOrigClaSize + 1;
                        StrategyParam::timeLimit = 5;
                        firstSwitch = true;
                        fol.noPairSingleCla.clear();
                        triAlg.noCmpCla.clear();
                        triAlg.flagT = true;
                        maxUintClaId = 0;
                        maxUnitId = 0;
                        simplyIdStart = 0;
                        simplyIdEnd= uNewClaID - 1;
                    }
                }
            } else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < 190) {
                 if(readStrategySwitch2){
                    shift = 0;
                    readStrategySwitch2= false;
                    FileOp::readBatchStrategy(FileOp::StrategyfileName[2]);
                    StrategyParam::timeLimit = 5;
                    if(StrategyParam::noUnitRes == false){
                        chooseTri = true;
                    }
                    if(StrategyParam::strategyMode ==0){
                        fol.DelSubSet(fol.uOrigClaSize);
                        FileOp::delRes();
                        uNewClaID = fol.uOrigClaSize + 1;
                        StrategyParam::timeLimit = 5;
                        firstSwitch = true;
                        fol.noPairSingleCla.clear();
                        triAlg.noCmpCla.clear();
                        triAlg.flagT = true;
                        maxUintClaId = 0;
                        maxUnitId = 0;
                        simplyIdStart = 0;
                        simplyIdEnd= uNewClaID - 1;
                    }
                }
            } else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < 250) {
                 if(readStrategySwitch3){
                    shift = 0;
                    readStrategySwitch3= false;
                    FileOp::readBatchStrategy(FileOp::StrategyfileName[3]);
                    StrategyParam::timeLimit = 5;
                    if(StrategyParam::noUnitRes == false){
                        chooseTri = true;
                    }
                    if(StrategyParam::strategyMode ==0){
                        fol.DelSubSet(fol.uOrigClaSize);
                        FileOp::delRes();
                        uNewClaID = fol.uOrigClaSize + 1;
                        StrategyParam::timeLimit = 5;
                        firstSwitch = true;
                        fol.noPairSingleCla.clear();
                        triAlg.noCmpCla.clear();
                        triAlg.flagT = true;
                        maxUintClaId = 0;
                        maxUnitId = 0;
                        simplyIdStart = 0;
                        simplyIdEnd= uNewClaID - 1;
                    }
                }
            } else{
                 if(readStrategySwitch4){
                    shift = 0;
                    readStrategySwitch4= false;
                    FileOp::readBatchStrategy(FileOp::StrategyfileName[4]);
                    StrategyParam::timeLimit = 5;
                    if(StrategyParam::noUnitRes == false){
                        chooseTri = true;
                    }
                    if(StrategyParam::strategyMode ==0){
                        fol.DelSubSet(fol.uOrigClaSize);
                        FileOp::delRes();
                        uNewClaID = fol.uOrigClaSize + 1;
                        StrategyParam::timeLimit = 5;
                        firstSwitch = true;
                        fol.noPairSingleCla.clear();
                        triAlg.noCmpCla.clear();
                        triAlg.flagT = true;
                        maxUintClaId = 0;
                        maxUnitId = 0;
                        simplyIdStart = 0;
                        simplyIdEnd= uNewClaID - 1;
                    }
                }
            }
        }
        if((StrategyParam::delPureCla || StrategyParam::termReRedund) && StrategyParam::problemDifficult){
            set<Clause*>::iterator claP_it = fol.setCla.begin();
            for (; claP_it != fol.setCla.end();) {
                if ((*claP_it)->uClaId >= simplyIdStart && (*claP_it)->uClaId <= simplyIdEnd) {
                    if (triAlg.claHasPureLit(fol, (*claP_it)) || triAlg.judgeDemodRedundent(fol, (*claP_it), uNewClaID)) {
                        set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
                        if (delIt != fol.setOriClas.end()) {
                            fol.setOriClas.erase(delIt);
                        }
                        set<Clause*> ::iterator delItNoPair = fol.noPairSingleCla.find(*claP_it);
                        if (delItNoPair != fol.noPairSingleCla.end()) {
                            fol.noPairSingleCla.erase(delItNoPair);
                        }
                        set<Clause*> ::iterator delItEqnDemod = fol.eqnDemod.find(*claP_it);
                        if (delItEqnDemod != fol.eqnDemod.end()) {
                            fol.eqnDemod.erase(delItEqnDemod);
                        }                       
                        set<Clause*> ::iterator delItGoal = fol.saveGoalClaSet.find(*claP_it);
                        if (delItGoal != fol.saveGoalClaSet.end()) {
                            fol.saveGoalClaSet.erase(delItGoal);
                        }                        
                        string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]冗余被删除";
                        FileOp::LogOut(out);
                        (*claP_it)->DelLits();
                        delete((*claP_it));
                        claP_it = fol.setCla.erase(claP_it);
                        continue;
                    }
                }
                ++claP_it;
            }  
        }
        preGenCla.clear();
        if (StrategyParam::factorRes && StrategyParam::problemDifficult) {
            set<Clause*>::iterator claP_it = fol.setCla.begin();
            for (; claP_it != fol.setCla.end();) {
                if(((*claP_it)->uClaId <= simplyIdEnd) && ((*claP_it)->uClaId >= simplyIdStart)){
                    bool delFlag = true;
                    int res = factorRes(fol, triAlg, (*claP_it), uNewClaID, &preGenCla, delFlag, false);
                    if (delFlag) {
                        set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
                        if (delIt != fol.setOriClas.end()) {
                            fol.setOriClas.erase(delIt);
                        }
                        set<Clause*> ::iterator delItNoPair = fol.noPairSingleCla.find(*claP_it);
                        if (delItNoPair != fol.noPairSingleCla.end()) {
                            fol.noPairSingleCla.erase(delItNoPair);
                        }
                        set<Clause*> ::iterator delItEqnDemod = fol.eqnDemod.find(*claP_it);
                        if (delItEqnDemod != fol.eqnDemod.end()) {
                            fol.eqnDemod.erase(delItEqnDemod);
                        }
                        set<Clause*> ::iterator delItGoal = fol.saveGoalClaSet.find(*claP_it);
                        if (delItGoal != fol.saveGoalClaSet.end()) {
                            fol.saveGoalClaSet.erase(delItGoal);
                        }
                        string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]被因子归结删除";
                        FileOp::LogOut(out);
                        (*claP_it)->DelLits();
                        delete((*claP_it));
                        claP_it = fol.setCla.erase(claP_it);
                    } else
                        ++claP_it;
                }
                else
                   ++claP_it; 
            }
        }
        if (preGenCla.size() > 0) {
            for (int i = 0; i < preGenCla.size(); ++i) {
                fol.BackwardSubsump(*preGenCla[i]);
            }
        }
        preGenCla.clear();
        if (StrategyParam::eqnRes && StrategyParam::problemDifficult) {
            set<Clause*>::iterator claP_it = fol.setCla.begin();
            for (; claP_it != fol.setCla.end();) {
                
                if (((*claP_it)->uClaId <= simplyIdEnd) && ((*claP_it)->uClaId >= simplyIdStart)) {
                    int res = negEqnRes(fol, triAlg, (*claP_it), uNewClaID, &preGenCla,false);
                    ++claP_it;
                } else
                    ++claP_it;
            }
        }
       if (preGenCla.size() > 0) {
            for (int i = 0; i < preGenCla.size(); ++i) {
                fol.BackwardSubsump(*preGenCla[i]); 
            }
        }
        preGenCla.clear();       
        if(StrategyParam::termRewriting != 2) {
            set<Clause*>::iterator claP_it = fol.setCla.begin();
            for (; claP_it != fol.setCla.end();) {
                if(((*claP_it)->uClaId >= simplyIdStart))
                {
                    if(fol.eqnDemod.find(*claP_it)!=fol.eqnDemod.end()){
                        ++claP_it;
                        continue;
                    }
                    int res =demodulatorClaSaturate(fol,triAlg,(*claP_it),uNewClaID,&preGenCla);
                    if((res==1) || (res==2)){
                        vector<Clause*>::iterator delVec = find(preGenCla.begin(),preGenCla.end(),(*claP_it));
                        if( delVec!= preGenCla.end()){
                            swap((*delVec),(*(preGenCla.end() -1)));
                            preGenCla.pop_back();
                        }
                        set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
                        if (delIt != fol.setOriClas.end()) {
                            fol.setOriClas.erase(delIt);
                        }
                        set<Clause*> ::iterator delItNoPair = fol.noPairSingleCla.find(*claP_it);
                        if (delItNoPair != fol.noPairSingleCla.end()) {
                            fol.noPairSingleCla.erase(delItNoPair);
                        }
                        set<Clause*> ::iterator delItEqnDemod = fol.eqnDemod.find(*claP_it);
                        if (delItEqnDemod != fol.eqnDemod.end()) {
                            fol.eqnDemod.erase(delItEqnDemod);
                        }
                        set<Clause*> ::iterator delItGoal = fol.saveGoalClaSet.find(*claP_it);
                        if (delItGoal != fol.saveGoalClaSet.end()) {
                            fol.saveGoalClaSet.erase(delItGoal);
                        }
                        string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]被单元等词子句重写删除";
                        FileOp::LogOut(out);
                        (*claP_it)->DelLits();
                        delete((*claP_it));
                        claP_it = fol.setCla.erase(claP_it);
                    }
                    else 
                        ++claP_it;
                }
                else
                    ++claP_it;
            }
            if (preGenCla.size() > 0) {
                for (int i = 0; i < preGenCla.size(); ++i) {
                    if (preGenCla[i]->LitPtr)
                        fol.BackwardSubsump(*preGenCla[i]); 
                }
            }
            preGenCla.clear();
        }
        if (firstSwitch){
            fol.resetSingleClaNum();
        }
        maxUintClaId = maxUnitId; 
        InsertSingleSet(fol,triAlg,maxUnitId);
        if ((fol.setOriClas.size() < 2) && (fol.setCla.size() < 2)) {	
            FileOp::LogOut("得出结论:可满足！");
            cout<< "得出结论:可满足"<<endl;
            return SAT;
        }
        if(fol.setSingleCla.size() == 0){
            StrategyParam::noUnitRes = true;
        }
        if (fol.setSingleCla.size() > 0) {
            if(!StrategyParam::deductMode)
                InsertOutTri(triAlg, fol, loop, chooseTri); 
            else{
                InsertOutTriLR(triAlg, fol, loop, chooseTri); 
            }
            givenClaP = triAlg.vOnlyOutTri[0]->claPtr;
            StartId = triAlg.vOnlyOutTri[0]->claPtr->uClaId;
            triAlg.startCluId = StartId;
            vector<Term*> chgVarT;
            chgVarT.reserve(6);
            UINT32 lens = triAlg.vOnlyOutTri.size();
            if (firstSwitch) {
                for (UINT32 i = 0; i < lens - 1; i++) {
                    for (UINT32 j = i + 1; j < lens; j++) {
                        if (triAlg.vOnlyOutTri[i]->isPairPred(triAlg.vOnlyOutTri[j]) && triAlg.unify.LitMgu(triAlg.vOnlyOutTri[i], triAlg.vOnlyOutTri[j], chgVarT)) {
                            string str;                      
                            FileOp::CreateRunInfoFile(FileOp::runFileId);
                            *FileOp::ofRealInfoOut << "从编号为" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "子句开始构建三角形" << endl;
                            triAlg.unify.WriteTriPath(triAlg.vOnlyOutTri[i]);
                            triAlg.unify.WriteTriPath(triAlg.vOnlyOutTri[j]);
                            *FileOp::ofRealInfoOut << "R:空子句" << endl;
                            FileOp::LogOut("# ==子句" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "起步，结论正确(UNSAT)！");
                            if(!StrategyParam::orForCompition && (StrategyParam::problemDiv == 0))
                                cout << "得出结论:不可满足" << endl;
                            if(StrategyParam::orForCompition) {
                                validPathOut[0].push_back(triAlg.vOnlyOutTri[i]->claPtr->uClaId);
                                validPathOut[0].push_back(triAlg.vOnlyOutTri[j]->claPtr->uClaId);
                                string out = "cnf($" + to_string(uNewClaID) + ",plain,\n" + "   ($false),\n";
                                out +="   inference(scs_inference,[],[" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "," + to_string(triAlg.vOnlyOutTri[j]->claPtr->uClaId) +"]" + "),\n" + "   ['proof']"+ ").";
                                *FileOp::ofOutput << out << endl;
                            }
                            return UNSAT;
                        }
                        triAlg.unify.ClearVarBind(chgVarT);
                    }
                }
                triAlg.unify.ClearVarBind(chgVarT);
                for (int k = 0; k < fol.setSingleEnq.size(); ++k) {
                    for (int i = 0; i < lens; ++i) {
                        if (triAlg.vOnlyOutTri[i]->isPairPred(fol.setSingleEnq[k]) && triAlg.unify.LitMgu(triAlg.vOnlyOutTri[i], fol.setSingleEnq[k], chgVarT)) {
                            string str;
                            FileOp::CreateRunInfoFile(FileOp::runFileId);
                            *FileOp::ofRealInfoOut << "从编号为" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "子句开始构建三角形" << endl;
                            triAlg.unify.WriteTriPath(triAlg.vOnlyOutTri[i]);
                            triAlg.unify.WriteTriPath(fol.setSingleEnq[k]);
                            *FileOp::ofRealInfoOut << "R:空子句" << endl;
                            FileOp::LogOut("# ==子句" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "起步，结论正确(UNSAT)！");
                            if(!StrategyParam::orForCompition && (StrategyParam::problemDiv == 0))
                                cout << "得出结论:不可满足" << endl;
                            if(StrategyParam::orForCompition) {
                                validPathOut[0].push_back(triAlg.vOnlyOutTri[i]->claPtr->uClaId);
                                validPathOut[0].push_back(fol.setSingleEnq[k]->claPtr->uClaId);
                                string out = "cnf($" + to_string(uNewClaID) + ",plain,\n" + "   ($false),\n";
                                out += "   inference(scs_inference,[],[" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "," + to_string(fol.setSingleEnq[k]->claPtr->uClaId) +"]" + "),\n" + "   ['proof']" + ").";
                                *FileOp::ofOutput << out << endl;
                            }
                            return UNSAT;
                        }
                        triAlg.unify.ClearVarBind(chgVarT);
                    }
                }
                triAlg.unify.ClearVarBind(chgVarT);
            } 
            else if (maxUintClaId != maxUnitId)
            {
                for (UINT32 i = 0; i < lens; i++) {
                    if (triAlg.vOnlyOutTri[i]->claPtr->uClaId < maxUintClaId)
                        continue;
                    triAlg.unify.ClearVarBind(chgVarT);
                    for (UINT32 j = 0; j < lens; j++) {
                        if (i == j)
                            continue;
                        if (triAlg.vOnlyOutTri[i]->isPairPred(triAlg.vOnlyOutTri[j]) && triAlg.unify.LitMgu(triAlg.vOnlyOutTri[i], triAlg.vOnlyOutTri[j], chgVarT)) {
                            string str;
                            FileOp::CreateRunInfoFile(FileOp::runFileId);
                            *FileOp::ofRealInfoOut << "从编号为" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "子句开始构建三角形" << endl;
                            triAlg.unify.WriteTriPath(triAlg.vOnlyOutTri[i]);                        
                            triAlg.unify.WriteTriPath(triAlg.vOnlyOutTri[j]);
                            *FileOp::ofRealInfoOut << "R:空子句" << endl;
                            FileOp::LogOut("# ==子句" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "起步，结论正确(UNSAT)！");
                            if(!StrategyParam::orForCompition && (StrategyParam::problemDiv == 0))
                                cout << "得出结论:不可满足" << endl;
                            if(StrategyParam::orForCompition) {
                                validPathOut[0].push_back(triAlg.vOnlyOutTri[i]->claPtr->uClaId);
                                validPathOut[0].push_back(triAlg.vOnlyOutTri[j]->claPtr->uClaId);
                                string out = "cnf($" + to_string(uNewClaID) + ",plain,\n" + "   ($false),\n";
                                out +="   inference(scs_inference,[],[" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "," + to_string(triAlg.vOnlyOutTri[j]->claPtr->uClaId) +"]" + "),\n" + "   ['proof']"+ ").";
                                *FileOp::ofOutput << out << endl;
                            }
                            return UNSAT;
                        }
                        triAlg.unify.ClearVarBind(chgVarT);
                    }
                    triAlg.unify.ClearVarBind(chgVarT);
                    for (int k = 0; k < fol.setSingleEnq.size(); ++k) {
                        if (triAlg.vOnlyOutTri[i]->isPairPred(fol.setSingleEnq[k]) && triAlg.unify.LitMgu(triAlg.vOnlyOutTri[i], fol.setSingleEnq[k], chgVarT)) {
                            string str;
                            FileOp::CreateRunInfoFile(FileOp::runFileId);
                            *FileOp::ofRealInfoOut << "从编号为" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "子句开始构建三角形" << endl;
                            triAlg.unify.WriteTriPath(triAlg.vOnlyOutTri[i]);                           
                            triAlg.unify.WriteTriPath(fol.setSingleEnq[k]);
                            *FileOp::ofRealInfoOut << "R:空子句" << endl;
                            FileOp::LogOut("# ==子句" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "起步，结论正确(UNSAT)！");
                            if(!StrategyParam::orForCompition && (StrategyParam::problemDiv == 0))
                                cout << "得出结论:不可满足" << endl;
                            if(StrategyParam::orForCompition) {
                                validPathOut[0].push_back(triAlg.vOnlyOutTri[i]->claPtr->uClaId);
                                validPathOut[0].push_back(fol.setSingleEnq[k]->claPtr->uClaId);
                                string out = "cnf($" + to_string(uNewClaID) + ",plain,\n" + "   ($false),\n";
                                out += "   inference(scs_inference,[],[" + to_string(triAlg.vOnlyOutTri[i]->claPtr->uClaId) + "," + to_string(fol.setSingleEnq[k]->claPtr->uClaId) +"]" + "),\n" + "   ['proof']"+ ").";
                                *FileOp::ofOutput << out << endl;
                            }
                            return UNSAT;
                        }
                        triAlg.unify.ClearVarBind(chgVarT);
                    }
                    triAlg.unify.ClearVarBind(chgVarT);
                }
            }
            triAlg.unify.ClearVarBind(chgVarT);
            chgVarT.clear();
            if (firstSwitch && StrategyParam::problemDifficult) {
                for (auto& aClap : fol.setCla)
                {
                    if ((aClap->uLitNum == 1)) {
                        continue;
                    }
                    vector<Term*> varBindT;
                    varBindT.reserve(6);
                    UINT16 pairCount = 0; 
                    vector<Literal*> isOutPut; 
                    isOutPut.reserve(6);
                    if (StrategyParam::noUnitRes) {
                        for (UINT16 ActLitInd = 0; ActLitInd < aClap->uLitNum; ++ActLitInd) {
                            Literal* QLit = aClap->LitPtr[ActLitInd];
                            for (UINT32 i = 0; i < lens; ++i) {
                                if (QLit->isPairPred(triAlg.vOnlyOutTri[i]) && triAlg.unify.LitMgu(QLit, triAlg.vOnlyOutTri[i], chgVarT)) {
                                    aClap->clast = PiredSingle; 
                                    varBindT.insert(varBindT.end(), chgVarT.begin(), chgVarT.end());
                                    chgVarT.clear();
                                    pairCount++;
                                    isOutPut.push_back(triAlg.vOnlyOutTri[i]); 
                                    break;
                                } else
                                    triAlg.unify.ClearVarBind(chgVarT);
                            }
                        }
                        if (pairCount == aClap->uLitNum)
                        {
                            FileOp::CreateRunInfoFile(FileOp::runFileId);
                            *FileOp::ofRealInfoOut << "从编号为" + to_string(aClap->uClaId) + "子句开始构建三角形" << endl;
                            for (UINT16 ActLitInd = 0; ActLitInd < aClap->uLitNum; ++ActLitInd) {
                                isOutPut.push_back(aClap->LitPtr[ActLitInd]);
                            }
                            for (UINT32 k = 0; k < isOutPut.size(); k++) {
                                triAlg.unify.WriteTriPath(isOutPut[k]);
                            }
                            *FileOp::ofRealInfoOut << "R:空子句" << endl;
                            FileOp::LogOut("# ==子句" + to_string(aClap->uClaId) + "起步，结论正确(UNSAT)！");
                            if(!StrategyParam::orForCompition && (StrategyParam::problemDiv == 0))
                                cout << "得出结论:不可满足" << endl;
                            if (StrategyParam::orForCompition) {
                                
                                string outClauses = "";
                                for (UINT8 k = 0; k < isOutPut.size(); ++k) {
                                    if(find(validPathOut[0].begin(),validPathOut[0].end(),isOutPut[k]-> claPtr->uClaId) == validPathOut[0].end()){
                                        validPathOut[0].push_back(isOutPut[k]-> claPtr->uClaId);
                                        outClauses += to_string(isOutPut[k]-> claPtr->uClaId) + ",";
                                    }
                                }
                                if(find(validPathOut[0].begin(),validPathOut[0].end(),aClap->uClaId) == validPathOut[0].end()){
                                    validPathOut[0].push_back(aClap->uClaId);
                                    outClauses += to_string(aClap->uClaId);
                                }
                                if(outClauses[outClauses.size() -1] == ','){
                                    outClauses.erase(outClauses.size() -1, 1);
                                }
                                string out = "cnf($" + to_string(uNewClaID) + ",plain,\n"+ "   ($false),\n"+"   inference(scs_inference,[]," + "[" + outClauses +"]" + "),\n" + "   ['proof']" + ").";
                                
                                *FileOp::ofOutput << out << endl;
                            }

                            return UNSAT;
                        } else if (pairCount == 0) {
                            aClap->clast = noPiredSingle; 
                            if (fol.noPairSingleCla.find(aClap) == fol.noPairSingleCla.end()) {
                                fol.noPairSingleCla.insert(aClap);
                            }
                        }
                        triAlg.unify.ClearVarBind(varBindT);
                        triAlg.unify.ClearVarBind(chgVarT);
                    }
                    vector<Term*>().swap(varBindT); 
                    vector<Literal*>().swap(isOutPut); 
                }
                triAlg.unify.ClearVarBind(chgVarT);
                triAlg.unify.ClearVarBind(chgVarT);
            }
            triAlg.unify.ClearVarBind(chgVarT);
            chgVarT.clear();
            if (0 &&(StrategyParam::singleGoalCount <= 3) && givenClaP->isGoal) {
                vector<Literal*>* vCandLitVec; 
                Clause* candClaP = nullptr; 
                Literal* candLitP = nullptr; 
                vector<Literal*> vSimplyNewR;
                vSimplyNewR.reserve(6);
                Literal* actLit = givenClaP->LitPtr[0];
                vCandLitVec = actLit->getPairPredLst();
                if ((vCandLitVec->size() == 0) && (givenClaP->uLitNum == 1)) {
                    if (triAlg.noCmpCla.find(givenClaP->uClaId) == triAlg.noCmpCla.end())
                        triAlg.noCmpCla.insert(givenClaP->uClaId);
                    continue;
                }
                for (UINT32 pasLitInd = 0; pasLitInd < UINT32(vCandLitVec->size()); ++pasLitInd)
                {
                    candLitP = vCandLitVec->at(pasLitInd); 
                    candClaP = candLitP->claPtr; 
                    if (triAlg.unify.LitMgu(actLit, candLitP, chgVarT)) {
                        for (UINT16 k = 0; k < candClaP->uLitNum; ++k) {
                            if (candClaP->LitPtr[k] != candLitP)
                                vSimplyNewR.push_back(candClaP->LitPtr[k]);
                        }
                        triAlg.newCla = nullptr;
                        triAlg.newCla = new Clause(uNewClaID, 0);
                        triAlg.newCla->isAxtom = true;
                        size_t uRsize = vSimplyNewR.size();
                        if (uRsize > 1)
                            stable_sort(vSimplyNewR.begin(), vSimplyNewR.end(), SortRule::LitCmp);
                        triAlg.newCla->LitPtr = new Literal*[uRsize];
                        for (UINT16 i = 0; i < vSimplyNewR.size(); ++i) {
                            Term* litSubT = triAlg.unify.GetNewTerm(vSimplyNewR[i]->subTerm, triAlg.newCla);
                            if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                                    && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                                --uRsize;
                                continue;
                            }
                            if (triAlg.newCla->uFuncDepth < litSubT->uLevel)
                                triAlg.newCla->uFuncDepth = litSubT->uLevel;
                            triAlg.newCla->uVarCount += litSubT->uVarCount;	
                            triAlg.newCla->LitPtr[triAlg.newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, triAlg.newCla->uLitNum + 1), triAlg.newCla, nullptr);
                            triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->parentLitPtr = vSimplyNewR[i];
                            if (triAlg.newCla ->deduceDeepth < (vSimplyNewR[i]->claPtr->deduceDeepth + 1)) {
                                triAlg.newCla ->deduceDeepth = vSimplyNewR[i]->claPtr->deduceDeepth + 1;
                            }
                            if (vSimplyNewR[i]->claPtr->isAxtom && triAlg.newCla->isAxtom)
                                triAlg.newCla->isAxtom = true;
                            else
                                triAlg.newCla->isAxtom = false;
                            if (triAlg.hasUnifGoalLit(fol, triAlg.newCla->LitPtr[triAlg.newCla->uLitNum])) {
                                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = 1;
                                if (triAlg.newCla->goaldeepth > 1)
                                    triAlg.newCla->goaldeepth = 1;
                            } else {
                                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = vSimplyNewR[i]->goalDeepth;
                                if (triAlg.newCla->goaldeepth > vSimplyNewR[i]->goalDeepth)
                                    triAlg.newCla->goaldeepth = vSimplyNewR[i]->goalDeepth;
                            }
                            ++(triAlg.newCla->uLitNum);
                        }
                        fol.initGenerateClaAttr(triAlg.newCla);
                        uNewClaID++; 
                        FileOp::CreateRunInfoFile(FileOp::runFileId);
                        *FileOp::ofRealInfoOut << "从编号为" + to_string(givenClaP->uClaId) + "子句开始构建三角形" << endl;
                        for (UINT32 k = 0; k < givenClaP->uLitNum; k++) {
                            triAlg.unify.WriteTriPath(givenClaP->LitPtr[k]);
                        }
                        for (UINT32 k = 0; k < candClaP->uLitNum; k++) {
                            triAlg.unify.WriteTriPath(candClaP->LitPtr[k]);
                        }
                        Clause* newCla = triAlg.newCla;
                        if (StrategyParam::termRewriting != 2) {
                            newCla->directEqn();
                        }
                        FileOp::WriteR(newCla);
                        triAlg.unify.ClearVarBind(chgVarT); 
                        vector<UINT32>invaildLitId; 
                        string rundMsg; 
                        Subsumption subsump; 
                        bool isRundacyR = subsump.GenCla(fol,newCla, invaildLitId, rundMsg);
                        FileOp::RInvaildInfoOut(rundMsg, StartId);
                        if (isRundacyR) {
                            newCla->DelLits(false);
                            delete(newCla);
                        } else {
                            fol.BackwardSubsump(*newCla); 
                            if (newCla->uLitNum == 1)
                                if (StrategyParam::R_Weight == -1)
                                    newCla->adeductNum = 0; 
                                else
                                    newCla->adeductNum = StrategyParam::R_Weight;
                            else
                                if (StrategyParam::R_Weight == -1)
                                    newCla->pdeductNum = 0; 
                                else
                                    newCla->pdeductNum = StrategyParam::R_Weight; 
                            fol.InsertCla(newCla);
                            fol.setCla.insert(newCla);
                            if (StrategyParam::termRewriting != 2)
                                fol.judgeInsertSuper(newCla);
                        }
                        vSimplyNewR.clear();
                        continue;
                    } else {
                        triAlg.unify.ClearVarBind(chgVarT);
                        continue;
                    }
                }
                if (StrategyParam::singleUseUintCla&& (triAlg.noCmpCla.find(givenClaP->uClaId) == triAlg.noCmpCla.end()))
                    triAlg.noCmpCla.insert(givenClaP->uClaId);
                triAlg.unify.ClearVarBind(chgVarT);
                continue;
            }
            int singleInClaSet = 0;
            vector<Clause*> simplfyCla; 
            simplfyCla.reserve(6);
            vector<Clause*> simplfyClaDel; 
            simplfyClaDel.reserve(6);
            triAlg.unify.ClearVarBind(chgVarT);
            if (StrategyParam::noUnitSimplify && StrategyParam::problemDifficult) {
                for (auto& aClap : fol.setCla)
                {
                    if ((aClap->uClaId <= simplyIdEnd) && (aClap->uClaId >= simplyIdStart)) {
                        continue;
                    }
                    if ((aClap->uLitNum == 1))
                        continue;
                    vector<Term*> varBindT; 
                    varBindT.reserve(6);
                    UINT8 pairCount = 0; 
                    vector<Literal*> isOutPut; 
                    isOutPut.reserve(6);
                    bool noShareVar = false;
                    isOutPut.clear();
                    chgVarT.clear();
                    if (aClap->hasConLit()) {
                        bool simplyOut = false;
                        vector<Literal*> vSimplyNewR;
                        vSimplyNewR.reserve(6);
                        for (UINT16 ActLitInd = 0; ActLitInd < aClap->uLitNum; ++ActLitInd) {
                            Literal* QLit = aClap->LitPtr[ActLitInd];
                            QLit->isDel = 0;
                            if (QLit->subTerm->IsShareVarLit())
                                continue;
                            else {
                                for (UINT32 i = 0; i < lens; ++i) {
                                    if (QLit->isPairPred(triAlg.vOnlyOutTri[i]) && triAlg.unify.LitMgu(QLit, triAlg.vOnlyOutTri[i], chgVarT)) {
                                        QLit->isDel = 1;
                                        varBindT.insert(varBindT.end(), chgVarT.begin(), chgVarT.end());
                                        chgVarT.clear();
                                        isOutPut.push_back(triAlg.vOnlyOutTri[i]);
                                        simplyOut = true;
                                        break;
                                    } else
                                        triAlg.unify.ClearVarBind(chgVarT);
                                }
                            }
                        }
                        if (simplyOut) {
                            for (UINT16 ActLitInd = 0; ActLitInd < aClap->uLitNum; ++ActLitInd) {
                                Literal* QLit = aClap->LitPtr[ActLitInd];
                                isOutPut.push_back(QLit);
                                if (!QLit->isDel)
                                    vSimplyNewR.push_back(QLit);
                            }
                            FileOp::CreateRunInfoFile(FileOp::runFileId);
                            *FileOp::ofRealInfoOut << "从编号为" + to_string(aClap->uClaId) + "子句开始构建三角形" << endl;
                            for (UINT32 k = 0; k < isOutPut.size(); k++) {
                                triAlg.unify.WriteTriPath(isOutPut[k]);
                            }
                            triAlg.newCla = nullptr;
                            triAlg.newCla = new Clause(uNewClaID, 0);
                            triAlg.newCla->isAxtom = true;
                            size_t uRsize = vSimplyNewR.size();
                            if (uRsize > 1)
                                stable_sort(vSimplyNewR.begin(), vSimplyNewR.end(), SortRule::LitCmp);
                            triAlg.newCla->LitPtr = new Literal*[uRsize];
                            for (UINT16 i = 0; i < vSimplyNewR.size(); ++i) {
                                Term* litSubT = triAlg.unify.GetNewTerm(vSimplyNewR[i]->subTerm, triAlg.newCla);
                                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                                    --uRsize;
                                    continue;
                                }
                                if (triAlg.newCla->uFuncDepth < litSubT->uLevel)
                                    triAlg.newCla->uFuncDepth = litSubT->uLevel;
                                triAlg.newCla->uVarCount += litSubT->uVarCount;		
                                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, triAlg.newCla->uLitNum + 1), triAlg.newCla, nullptr);
                                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->parentLitPtr = vSimplyNewR[i]; 
                                if (triAlg.newCla ->deduceDeepth < (vSimplyNewR[i]->claPtr->deduceDeepth + 1)) {
                                    triAlg.newCla ->deduceDeepth = vSimplyNewR[i]->claPtr->deduceDeepth + 1;
                                }
                                if (vSimplyNewR[i]->claPtr->isAxtom && triAlg.newCla->isAxtom)
                                    triAlg.newCla->isAxtom = true;
                                else
                                    triAlg.newCla->isAxtom = false;
                                 if (triAlg.hasUnifGoalLit(fol, triAlg.newCla->LitPtr[triAlg.newCla->uLitNum])) {
                                    triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = 1;
                                    if (triAlg.newCla->goaldeepth > 1)
                                        triAlg.newCla->goaldeepth = 1;
                                } else {
                                    triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = vSimplyNewR[i]->goalDeepth;
                                    if (triAlg.newCla->goaldeepth > vSimplyNewR[i]->goalDeepth)
                                        triAlg.newCla->goaldeepth = vSimplyNewR[i]->goalDeepth;
                                }  
                                ++(triAlg.newCla->uLitNum);
                            }
                            fol.initGenerateClaAttr(triAlg.newCla);
                            uNewClaID++;
                            Clause* newCla = triAlg.newCla;
                            if (StrategyParam::termRewriting != 2) {
                                newCla->directEqn();
                            }
                            FileOp::WriteR(newCla);                          
                            if (StrategyParam::orForCompition) {
                                newCla->addRForCompetiton();                             
                                string rStr = "";
                                newCla->toString(rStr);
                                string outClauses = "";
                                for (UINT8 k = 0; k < isOutPut.size(); ++k) {
                                    if(find(validPathOut[uNewClaID-1].begin(),validPathOut[uNewClaID-1].end(),isOutPut[k]-> claPtr->uClaId) == validPathOut[uNewClaID-1].end()){
                                        validPathOut[uNewClaID-1].push_back(isOutPut[k]-> claPtr->uClaId);
                                        outClauses += to_string(isOutPut[k]-> claPtr->uClaId) + ",";
                                    }
                                }
                                if(find(validPathOut[uNewClaID-1].begin(),validPathOut[uNewClaID-1].end(),aClap->uClaId) == validPathOut[uNewClaID-1].end()){
                                    validPathOut[uNewClaID-1].push_back(aClap->uClaId);
                                    outClauses += to_string(aClap->uClaId);
                                }
                                if(outClauses[outClauses.size() -1] == ','){
                                    outClauses.erase(outClauses.size() -1, 1);
                                }
                                int begPos = rStr.find_first_of("[") + 1;
                                int endPos = rStr.find_first_of("]");
                                string rId = rStr.substr(begPos, endPos - begPos);
                                string rContext = rStr.substr(rStr.find_first_of("]") + 1);
                                string out = "cnf(" + rId + ",plain,\n" + "   ("+ rContext+"),\n" + "   inference(scs_inference,[]," + "[" + outClauses +"]"+ ")).";
                                *FileOp::ofOutput << out << endl;
                            }
                            if (1){
                                insertNoPairCla(fol, newCla);
                            }
                            string vaildInf = "|[Y]";
                            FileOp::RInvaildInfoOut(vaildInf, StartId);
                            ++triTotalTimes; 
                            simplfyCla.push_back(newCla);
                            simplfyClaDel.push_back(aClap);
                            if (newCla->uLitNum == 1)
                                if (StrategyParam::R_Weight == -1)
                                    newCla->adeductNum = 0;
                                else
                                    newCla->adeductNum = StrategyParam::R_Weight;
                            else
                                if (StrategyParam::R_Weight == -1)
                                newCla->pdeductNum = 0; 
                            else
                                newCla->pdeductNum = StrategyParam::R_Weight;
                        }
                        vSimplyNewR.clear();
                        vector<Literal*>().swap(vSimplyNewR);
                    }
                    triAlg.unify.ClearVarBind(varBindT);
                    vector<Term*>().swap(varBindT);
                    isOutPut.clear();
                    vector<Literal*>().swap(isOutPut);
                    triAlg.unify.ClearVarBind(chgVarT);
                    chgVarT.clear();
                }
                if (!simplfyCla.empty()) {
                    for (int m = 0; m < simplfyCla.size(); ++m) {
                        fol.BackwardSubsumpDirect(*(simplfyCla[m]), *(simplfyClaDel[m]));
                        fol.InsertCla(simplfyCla[m]);
                        fol.setCla.insert(simplfyCla[m]);
                        if (StrategyParam::termRewriting != 2)
                            fol.judgeInsertSuper(simplfyCla[m]);
                    }
                    simplfyCla.clear();
                    vector<Clause*>().swap(simplfyCla);
                    simplfyClaDel.clear();
                    vector<Clause*>().swap(simplfyClaDel);
                }
            }
            triAlg.unify.ClearVarBind(chgVarT);
            chgVarT.clear();
            vector<Term*>().swap(chgVarT);
             if (!givenClaP)
                continue;
        }
        else{
            chooseTri = false;
            givenClaP = nullptr;
            if(fol.setSingleCla.size() == 0){
                for (auto& aClap : fol.setCla) {
                    if (aClap->uLitNum == 1) {
                        continue;
                    }
                    insertNoPairCla(fol, aClap);
                }
            }
        }
        chgCla.clear();    
        triAlg.allNoPair = false;     
        simplyIdStart = simplyIdEnd+1;
        firstSwitch = false;
        UINT8 resolutionRes = 0;
        if(chooseTri)
        {
            if(!StrategyParam::deductMode){
                FileOp::CreateRunInfoFile(FileOp::runFileId);
                resolutionRes = triAlg.GenerateTriByRecodePath(fol,&*givenClaP, uNewClaID, backType,chgCla,loop);
            }
            else{
                FileOp::CreateRunInfoFile(FileOp::runFileId);
                vector<Clause*> candidateClaSet;
                candidateClaSet.reserve(16);
                for (auto& aClap : fol.setCla) {
                    if(aClap->uLitNum != 1){
                        candidateClaSet.push_back(aClap);
                    }
                }
                for (auto& aClap : fol.setEnq){
                    if(aClap->uLitNum != 1){
                        candidateClaSet.push_back(aClap);
                    }
                }
               if((StrategyParam::unitSortRule == 0) || (StrategyParam::unitSortRule == 1)){
                   stable_sort(candidateClaSet.begin(), candidateClaSet.end(), SortRule::PoslitCmpProRL);
                }
                else
                    stable_sort(candidateClaSet.begin(), candidateClaSet.end(), SortRule::PoslitCmpPriorLR);
                if(StrategyParam::cseRunMode == 0){
                    resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                }
                else if(StrategyParam::cseRunMode == 1){
                    resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                }
                else if(StrategyParam::cseRunMode == 2){
                    if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::distriTime * 12) {
                        resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol, candidateClaSet, uNewClaID, backType, chgCla, loop);
                    }
                    else
                        resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                }
                else if((StrategyParam::cseRunMode == 3)){
                    if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::distriTime * 12) {
                        resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                    }
                    else
                        resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol, candidateClaSet, uNewClaID, backType, chgCla, loop);
                }   
                else if((StrategyParam::cseRunMode == 4)){
                    if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::distriTime * 12) {
                        resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                    }
                    else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::distriTime * 36) {
                        resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol, candidateClaSet, uNewClaID, backType, chgCla, loop);
                    }
                    else
                        resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                } 
                else if((StrategyParam::cseRunMode == 5)){
                    if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::distriTime * 12) {
                        resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol, candidateClaSet, uNewClaID, backType, chgCla, loop);
                    }
                    else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::distriTime * 36) {
                        resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                    }
                    else
                        resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol, candidateClaSet, uNewClaID, backType, chgCla, loop);
                }                              
                candidateClaSet.clear();
                vector<Clause*>().swap(candidateClaSet);
            }
            ++triTotalTimes;
            if(fol.noPairSingleCla.size()>0 &&StrategyParam::noUnitRes &&(triTotalTimes%triAlgTimes == 0))
                chooseTri = false;
        }
        else if(StrategyParam::noUnitRes && !chooseTri){
            givenClaP = nullptr;
            vector<Clause*> claNoPairVec;
            claNoPairVec.reserve(8);
            if((fol.noPairSingleCla.size()>0) || insertNoUseCla){
                if(insertNoUseCla)
{
                    for (auto& claP : fol.setCla) {
                        insertNoPairCla(fol, claP);
                    }
                    for (auto& claP : fol.setEnq) {
                        insertNoPairCla(fol, claP);
                    }
                    insertNoUseCla = false;
                }
                for (auto& claP : fol.noPairSingleCla) {
                    if(claP->uLitNum == 1){
                        continue;
                    }
                    claNoPairVec.push_back(claP);
                }
                if((fol.noPairSingleCla.size() == 0) ||(claNoPairVec.size() == 0)){
                    chooseTri = true;
                    continue;
                }
                if(claNoPairVec.size() != 0){
                    stable_sort(claNoPairVec.begin(), claNoPairVec.end(), SortRule::noPairClaCmp);
                }
                if(claNoPairVec.size()>= noPairInd+1){
                    for(UINT32 m=0; m<=noPairInd;++m){
                        if(claNoPairVec[m]->noPairResFiFg == false){
                            swap(claNoPairVec[m],claNoPairVec[noPairInd]);
                            break;
                        }
                    }
                }               
                if (noPairInd >= claNoPairVec.size())
                    noPairInd = 0;
                givenClaP = claNoPairVec[noPairInd];
                UINT32 countSize = 0;
                while(givenClaP->uLitNum >= litSize){
                    ++noPairCla;
                    if (noPairInd >= claNoPairVec.size())
                        noPairInd = 0;
                     givenClaP = claNoPairVec[noPairInd];
                    ++countSize;
                    if(countSize == claNoPairVec.size())
                        break;
                }
                noPairCla = givenClaP;
            }
            
            if (givenClaP) {
                triAlg.vOutTri.clear();
                FileOp::CreateRunInfoFile(FileOp::runFileId);
                resolutionRes = triAlg.GenerateTriByRecodePathNoUnit(fol, &*givenClaP, uNewClaID, backType, chgCla); 
                ++triTotalTimes;
            }
            chooseTri = true;
            claNoPairVec.clear();
            vector<Clause*>().swap(claNoPairVec);
        }
        if (!givenClaP){
            ++StrategyParam::noUintCla;
            if(StrategyParam::noUintCla > 8)
                StrategyParam::noUintCla = 0;
            continue;
        }
        if(triAlg.allNoPair)
        {
            if(resolutionRes == 33){
                FileOp::delCreateRunInfoFile(FileOp::runFileId);
                for (auto& claP : fol.noPairSingleCla) {
                    claP->noUnitDeductTimes = 0;
                }
                givenClaP->noPairResFiFg = true;
                ++noPairInd;
            }
            else if((resolutionRes == 22) ||(resolutionRes == 222) ){
                FileOp::delCreateRunInfoFile(FileOp::runFileId);
                triAlgTimes = 1;
                if(fol.noPairSingleCla.size() == 0){
                    fol.initNoPairCla();
                }
                StrategyParam::noUnitRes = true;
                if((FileOp::strategyMode != 3)&&(FileOp::strategyMode != 4))
                    ++StrategyParam::R_MAX_FUNCLAYER;
                if(StrategyParam::R_MAX_FUNCLAYER == 21)
                    StrategyParam::R_MAX_FUNCLAYER = StrategyParam::R_MAX_FUNCLAYER_TMP;
                else if(StrategyParam::R_MAX_FUNCLAYER > 100)
                    StrategyParam::R_MAX_FUNCLAYER = 16;
                if(StrategyParam::R_MAX_FUNCLAYER == (StrategyParam::R_INIT_MAX_FUNCLAYER*2 +1)){
                    StrategyParam::R_MAX_FUNCLAYER = StrategyParam::R_MAX_FUNCLAYER_TMP;
                }
                if((FileOp::strategyMode != 3)&&(FileOp::strategyMode != 4))
                    ++StrategyParam::R_MAX_NUM;
                if(StrategyParam::R_MAX_NUM == StrategyParam::R_INIT_MAX_NUM){
                    StrategyParam::R_MAX_NUM = StrategyParam::R_MAX_NUM_TMP;
                }
                else if(StrategyParam::R_MAX_NUM > 50){
                    StrategyParam::R_MAX_NUM = 16;
                }
            }
            else if(resolutionRes == 211){
                loop = 0;
            }
            long leftMem = FileOp::getMem();
            if (leftMem < 200000) {
                chgCla.clear();
                vector<Clause*>().swap(chgCla);
                preGenCla.clear();
                vector<Clause*>().swap(preGenCla);
                FileOp::LogOut("内存超出限制,无法判定！剩余内存:" + to_string(leftMem));
                string fileMemOut = FileOp::g_OutputPath + "/"  + "memleft.log";
                string cmdMem = "ps -aux | sort -k4nr | head -10 >> " + fileMemOut;
                int res = system(const_cast<char*> (cmdMem.c_str()));
                if (!StrategyParam::orForCompition)
                    cout << "无法判定,内存超出限制" << endl;
                else
                    cout << "TMO-Non" << endl;
                triAlg.delSaveCopyCla(fol);
                FileOp::outReleaseOf();
                string str = "";
                if (!StrategyParam::orForCompition && (FileOp::runMode != 0) && (FileOp::runMode != 1)) {
                    fol.uOrigClaSingleSize = 0;
                    str += fol.ToSingleString();
                    FileOp::PrintINFOEndRSinglePre(str); 
                }
                return UNKNOWN;
            }
            clock_t endNoPair = clock();
            double time_sNoPair = (double) (endNoPair - StrategyParam::start) / (double) CLOCKS_PER_SEC;
            double breakTimeA = 0.0;
            if((FileOp::runMode != 0 )&&(FileOp::runMode != 1)){
                breakTimeA = StrategyParam::timeForceLimit - 10;
            }
            else if((FileOp::runMode == 0 )||(FileOp::runMode == 1)){
                breakTimeA = StrategyParam::timeForceLimit - 10;
            }
            StrategyParam::timeLimit = StrategyParam::firstTimeLimit;
            if (time_sNoPair > StrategyParam::timeLimit *60)
            {
                triAlg.delSaveCopyCla(fol);
                FileOp::outReleaseOf();
                string str = "";
                if (!StrategyParam::orForCompition && (FileOp::runMode != 0) && (FileOp::runMode != 1)) {
                    fol.uOrigClaSingleSize = 0;
                    str += fol.ToSingleString();
                    FileOp::PrintINFOEndRSinglePre(str); 
                }
                
                if(!StrategyParam::orForCompition) {
                    str = "";
                    str += fol.ToUnUseString();
                    FileOp::PrintINFONoUsePre(str); 
                    FileOp::LogOut("超出时间" + to_string(StrategyParam::timeLimit) +"分钟限制,无法判定！");
                }              
                if ((!StrategyParam::orForCompition) && (FileOp::strategyMode != 3))
                    cout << "无法判定" << endl;
                else
                    cout << "TMO-Non" << endl;
                chgCla.clear();
                vector<Clause*>().swap(chgCla);
                preGenCla.clear();
                vector<Clause*>().swap(preGenCla);
                return UNKNOWN;
            }
            if (StrategyParam::genLitClause && (loop != 0) && (triAlg.total_Loop / loop < 2)) {
                --loop;
            }
            else if (((fol.noPairSingleCla.size()>0) &&StrategyParam::noUnitRes &&!chooseTri )||(chooseTri &&(!StrategyParam::noUnitRes || (fol.noPairSingleCla.size() == 0)) )) {
                loop = 0;
            }
            FileOp::outReleaseOf();
            continue;
        }
        Clause* newCla = triAlg.newCla;
        bool rFilter = false;
        if((chgCla.size() == 0) || (triAlg.vNewR.empty() || triAlg.newCla->uLitNum == 0)){
            rFilter = false;
        }
        else{
            UINT32 midLens = chgCla.size();
            if(chgCla[midLens-1]->uLitNum != newCla->uLitNum){
                rFilter = false;
            }
            else{
                UINT32 countRSize = 0;
                for(UINT32 pos=0; pos < newCla->uLitNum; ++pos){
                    Literal* litR = newCla->LitPtr[pos];
                    Literal* midR = chgCla[midLens-1]->LitPtr[pos];
                    if(litR->isEqPred(midR) && triAlg.unify.CheckEqnR(litR->subTerm,midR->subTerm)){
                        ++countRSize;
                        continue;
                    }
                    else
                        rFilter = false;
                }
                if(countRSize == newCla->uLitNum)
                    rFilter = true;
            }
        }
        if(!rFilter) {	
            if (resolutionRes == 44){
                *FileOp::ofRealInfoOut << "从编号为" + to_string(givenClaP->uClaId) + "子句开始非单元子句演绎" << endl;
                triAlg.PrintTriTwoPath();
            }
            else{
                *FileOp::ofRealInfoOut << "从编号为" + to_string(givenClaP->uClaId) + "子句开始构建三角形" << endl;
                triAlgTimes = 16;
                UINT32 realStartId = triAlg.PrintTriPath();
                if(realStartId != givenClaP->uClaId){
                    loop = 1;
                }
            }
            if (triAlg.vNewR.empty() || triAlg.newCla->uLitNum == 0) {
                FileOp::WriteR(nullptr);
                FileOp::LogOut("# ==子句" + to_string(givenClaP->uClaId) + "起步，结论正确(UNSAT)！");
                chgCla.clear();
                vector<Clause*>().swap(chgCla);
                preGenCla.clear();
                vector<Clause*>().swap(preGenCla);
                if (StrategyParam::orForCompition) {                   
                    string outClauses = triAlg.PrintCompetitionOut(0);
                    string out = "cnf($" + to_string(uNewClaID) + ",plain,\n"+ "   ($false),\n"+"   inference(scs_inference,[]," + "[" + outClauses +"]" + "),\n" + "   ['proof']"+ ").";                  
                    *FileOp::ofOutput << out << endl;
                }
                return UNSAT;
            }
            if (StrategyParam::termRewriting != 2){
                    newCla->directEqn();
            }
            FileOp::WriteR(newCla);
            triAlg.BackPathClear();
            UINT16 rLen = newCla->uLitNum;
            vector<UINT32>invaildLitId; 
            string rundMsg; 
            Subsumption subsump; 
            bool isRundacyR = subsump.GenCla(fol,newCla, invaildLitId, rundMsg);
            bool RFactorFlag = false;
            FileOp::RInvaildInfoOut(rundMsg, StartId);
            if (isRundacyR) {
                newCla->DelLits(false);
                delete(newCla);
            } else {
                if (StrategyParam::orForCompition) {
                    newCla->addRForCompetiton();                   
                    string rStr = "";
                    newCla->toString(rStr);
                    string outClauses = triAlg.PrintCompetitionOut(uNewClaID - 1);
                    int begPos = rStr.find_first_of("[") + 1;
                    int endPos = rStr.find_first_of("]");
                    string rId = rStr.substr(begPos, endPos - begPos);
                    string rContext = rStr.substr(rStr.find_first_of("]") + 1);
                    string out = "cnf(" + rId + ",plain,\n" + "   ("+ rContext+"),\n";
                    out += "   inference(scs_inference,[],[" + outClauses +"]" + ")).";
                    *FileOp::ofOutput << out << endl;
                }
                bool resFlag = true;
                if (resFlag) {
                    fol.BackwardSubsumpClaSet(*newCla,chgCla,false); 
                    if (newCla->uLitNum == 1)
                        if (StrategyParam::R_Weight == -1)
                            newCla->adeductNum = 0; 
                        else
                            newCla->adeductNum = StrategyParam::R_Weight;
                    else
                        if (StrategyParam::R_Weight == -1)
                        newCla->pdeductNum = 0; 
                    else
                        newCla->pdeductNum = StrategyParam::R_Weight;
                    fol.InsertCla(newCla);
                    fol.setCla.insert(newCla);
                    if (StrategyParam::termRewriting != 2)
                        fol.judgeInsertSuper(newCla);
                    if (1) {
                        insertNoPairCla(fol, newCla);
                    }
                }
            }
        }
        else {
            newCla->DelLits(false);
            delete(newCla);
            --uNewClaID;
        }
        for(int m=0;m<chgCla.size();m++)
        {
            if (1) {
                if (!StrategyParam::divideTri) {
                    *FileOp::ofRealInfoOut << "R[" + to_string(chgCla[m]->uClaId) + "]" << "[" + to_string(chgCla[m]->LitPtr[0]->ParentLitRow()) + "_" + to_string(chgCla[m]->LitPtr[0]->ParentLitCol()) + "]" << chgCla[m]->LitPtr[0]->subTerm->ToString();
                    *FileOp::ofRealInfoOut << "|[Y]" << endl;
                }
                bool resFlag = true;
                if(resFlag) 
                {
                    fol.BackwardSubsumpClaSet(*chgCla[m],chgCla,true); 
                    if (chgCla[m]->uLitNum == 1) {
                        if (StrategyParam::R_Weight == -1)
                            chgCla[m]->adeductNum = 0;
                        else
                            chgCla[m]->adeductNum = StrategyParam::R_Weight;
                    } else {
                        if (StrategyParam::R_Weight == -1)
                            chgCla[m]->pdeductNum = 0;
                        else
                            chgCla[m]->pdeductNum = StrategyParam::R_Weight;
                    }
                    if (StrategyParam::termRewriting != 2)
                        fol.judgeInsertSuper(chgCla[m]);
                    if (1){
                        insertNoPairCla(fol, chgCla[m]);
                    }
                }
                
                continue;
            }
        }
        long leftMem = FileOp::getMem();
        if (leftMem < 200000) {
            chgCla.clear();
            vector<Clause*>().swap(chgCla);
            preGenCla.clear();
            vector<Clause*>().swap(preGenCla);
            FileOp::LogOut("内存超出限制,无法判定！剩余内存:" + to_string(leftMem));
            string fileMemOut = FileOp::g_OutputPath + "/" + "memleft.log";
            string cmdMem = "ps -aux | sort -k4nr | head -10 >> " + fileMemOut;
            int res = system(const_cast<char*> (cmdMem.c_str()));
            if (!StrategyParam::orForCompition)
                cout << "无法判定,内存超出限制" << endl;
            else
                cout << "TMO-Non" << endl;
            triAlg.delSaveCopyCla(fol);
            FileOp::outReleaseOf();
            string str = "";
            if (!StrategyParam::orForCompition && (FileOp::runMode != 0) && (FileOp::runMode != 1)) {
                fol.uOrigClaSingleSize = 0;
                str += fol.ToSingleString();
                FileOp::PrintINFOEndRSinglePre(str);
            }
            return UNKNOWN;
        }
        clock_t end = clock();
        double time_s = (double)(end - StrategyParam::start)/(double)CLOCKS_PER_SEC;  
        double breakTimeB = 0.0;
        if ((FileOp::runMode != 0)&&(FileOp::runMode != 1)) {
            breakTimeB = StrategyParam::timeForceLimit - 10;
        } else if ((FileOp::runMode == 0) || (FileOp::runMode == 1)) {
            breakTimeB = StrategyParam::timeForceLimit - 10;
        }
        StrategyParam::timeLimit = StrategyParam::firstTimeLimit;
        if(time_s  > StrategyParam::timeLimit *60)
        {
            triAlg.delSaveCopyCla(fol);
            FileOp::outReleaseOf();
            string str = "";
            if (!StrategyParam::orForCompition && (FileOp::runMode != 0)&&(FileOp::runMode != 1)) {
                fol.uOrigClaSingleSize = 0;
                str += fol.ToSingleString();
                FileOp::PrintINFOEndRSinglePre(str); 
            }         
            if(!StrategyParam::orForCompition) {
                str = "";
                str += fol.ToUnUseString();
                FileOp::PrintINFONoUsePre(str); 
                FileOp::LogOut("超出时间"+ to_string(StrategyParam::timeLimit) +"分钟限制,无法判定！");
            }           
            if ((!StrategyParam::orForCompition) && (FileOp::strategyMode != 3))
                cout << "无法判定" << endl;
            else
                cout << "TMO-Non" << endl;
            chgCla.clear();
            vector<Clause*>().swap(chgCla);
            preGenCla.clear();
            vector<Clause*>().swap(preGenCla);
            return UNKNOWN;
        }     
    }
    return UNKNOWN;
}

void Resolution::insertNoPairCla(Formula& fol, Clause* cla){
    UINT16 ulitN = cla->uLitNum;
    if(ulitN == 1)
        return;
    switch(StrategyParam::noUintCla){
        case 0:
            if (ulitN == 2) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
        case 1:
            if (ulitN <= 3) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
          case 2:
            if (ulitN <= 4) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
          case 3:
            if (ulitN <= 5) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
          case 4:
            if ((ulitN == 2) || cla->isGoal) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
          case 5:
            if ((ulitN <= 3) || cla->isGoal ) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
          case 6:
            if ((ulitN <= 4) || cla->isGoal) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
          case 7:
            if ((ulitN <= 5) || cla->isGoal) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
          case 8:
            if (1) {
                if (fol.noPairSingleCla.find(cla) == fol.noPairSingleCla.end()) {
                    fol.noPairSingleCla.insert(cla);
                }
            }
            break;
    }
}
 RESULT Resolution::BaseAlgByRecodePathSaturate(Formula& fol, UINT64 &triTotalTimes) {
    shift = 0;
    TriAlg triAlg;
    UINT32 uNewClaID = fol.uOrigClaSize + 1;
    triAlg.singleClaNum = fol.uOrigClaSize;
    triAlg.flagT = true;
    string str = "";
    BACKTYPE backType = NOBACK;
    vector<Clause*> preGenCla;
    preGenCla.reserve(6);
    vector<Clause*> chgCla;
    chgCla.reserve(6);
    triAlg.start = clock();
    Clause* givenClaP = nullptr;
    vector<Literal*> unifyLit;
    unifyLit.reserve(16);
    set<UINT32> saveClaId;
    if(StrategyParam::lockType != 0){
        set<Clause*>::iterator claP_it = fol.setCla.begin();
        string lockOutTotal;
        for (; claP_it != fol.setCla.end();) {           
            if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) > 40) {
                break;
            }
            bool pureLit = lockResSimlifyNew(fol, triAlg, *claP_it, uNewClaID);
            string lockOut;
            if((lockClaVec.size())>0 &&(fol.setOriClas.size() >1)){
                if(pureLit)
                    lockOut = "纯文字;";
                else {
                    if (StrategyParam::lockType == 1)
                        lockOut = "锁子句处理;";
                    else if (StrategyParam::lockType == 2)
                        lockOut = "锁+包含处理;";
                    else if (StrategyParam::lockType == 3)
                        lockOut = "锁+包含2处理;";
                    else if (StrategyParam::lockType == 4)
                        lockOut = "锁+不对称文字+恒真子句处理;";
                    else if (StrategyParam::lockType == 5)
                        lockOut = "锁+不对称文字+包含子句处理;";
                }
                lockOut = lockOut + "子句编号为:";
                for (int m = 0; m < lockClaVec.size(); ++m) {
                    lockOut = lockOut + to_string(lockClaVec[m]->uClaId) + "\n";
                    set<Clause*> ::iterator delIt = fol.setOriClas.find(lockClaVec[m]);
                    if (delIt != fol.setOriClas.end()) {
                        fol.setOriClas.erase(delIt);
                    }
                    (lockClaVec[m])->DelLits();
                    delete(lockClaVec[m]);
                    claP_it = fol.setCla.erase(claP_it);                   
                    string out = "#子句编号[" + to_string(lockClaVec[m]->uClaId) + "]为锁子句被删除";
                    FileOp::LogOut(out);
                }
                lockClaVec.clear();
                lockOutTotal += lockOut;
            }
            else
                ++claP_it;          
        }
        *FileOp::ofJudgeAllFileResult << FileOp::JudgeFileName << lockOutTotal << endl;
    }
    if (StrategyParam::delPureCla) {
        set<Clause*>::iterator claP_it = fol.setCla.begin();
        for (; claP_it != fol.setCla.end();) {
            if (triAlg.claHasPureLit(fol, (*claP_it))) {
                set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
                if (delIt != fol.setOriClas.end()) {
                    fol.setOriClas.erase(delIt);
                }
                string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]含纯文字被删除";
                FileOp::LogOut(out);
                (*claP_it)->DelLits();
                delete((*claP_it));
                claP_it = fol.setCla.erase(claP_it);
            } 
            else{
                if((*claP_it)->isGoal){
                    for(int i=0;i<(*claP_it)->uLitNum;++i){
                        unifyLit.push_back((*claP_it)->LitPtr[i]);
                    }
                    saveClaId.insert((*claP_it)->uClaId);
                    fol.saveGoalClaSet.insert(*claP_it);
                    if ((*claP_it)->uLitNum > 1) {
                        insertNoPairCla(fol, *claP_it);
                    }             
                }
                ++claP_it;
            }
        }
        set<Clause*>::iterator claPE_it = fol.setEnq.begin();
        for (; claPE_it != fol.setEnq.end();) {
            if (triAlg.claHasPureLit(fol, (*claPE_it))) {
                set<Clause*> ::iterator delIt = fol.setOriClas.find(*claPE_it);
                if (delIt != fol.setOriClas.end()) {
                    fol.setOriClas.erase(delIt);
                }               
                string out = "#子句编号[" + to_string((*claPE_it)->uClaId) + "]含纯文字被删除";
                FileOp::LogOut(out);
                (*claPE_it)->DelLits();
                delete((*claPE_it));
                claPE_it = fol.setEnq.erase(claPE_it);
            } 
            else
                ++claPE_it;
        }
    }
    else{
        set<Clause*>::iterator claP_it = fol.setCla.begin();
        for (; claP_it != fol.setCla.end();) {
            if ((*claP_it)->isGoal) {
                for (int i = 0; i < (*claP_it)->uLitNum; ++i) {
                    unifyLit.push_back((*claP_it)->LitPtr[i]);
                }
                saveClaId.insert((*claP_it)->uClaId);
                fol.saveGoalClaSet.insert(*claP_it);
                insertNoPairCla(fol, *claP_it);
            }
            ++claP_it;
        }
    }
    if(StrategyParam::goalCla && (unifyLit.size()>0)){
        vector<Literal*>* vCandLit;
        Literal* candLitP = nullptr;
        Clause* candClaP = nullptr;
        vector<Term*> chgVarTemp;
        chgVarTemp.reserve(6);
        int length = unifyLit.size();
        bool flag = true;
        int resLen = 0;
        int reslength = 0;
        for(int i=0;i<unifyLit.size();++i){         
            unifyLit[i]->goalDeepth = reslength;
            if(unifyLit[i]->claPtr->goaldeepth > reslength){
                unifyLit[i]->claPtr->goaldeepth = reslength;
            }
            if(flag){
                flag = false;
                resLen = unifyLit.size();
            }
            if(i == (resLen -1)){
                ++reslength;
                resLen = unifyLit.size();
                 if(reslength == StrategyParam::goalDeepth){
                     break;
                 }
            }
            if(saveClaId.size() == (fol.setCla.size() + fol.setEnq.size())){
                break;
            }
            vCandLit = unifyLit[i]->getPairPredLst();
            for (UINT32 pasLitInd = 0; pasLitInd < UINT32(vCandLit->size()); ++pasLitInd)
            {
                candLitP = vCandLit->at(pasLitInd);
                candClaP = candLitP->claPtr;
                if (saveClaId.find(candClaP->uClaId) != saveClaId.end())
                    continue;
                if (triAlg.unify.LitMgu(unifyLit[i], candLitP, chgVarTemp)) {
                    triAlg.unify.ClearVarBind(chgVarTemp);                
                    for(int m=0;m<candClaP->uLitNum;++m){
                        if(find(unifyLit.begin(),unifyLit.end(),candClaP->LitPtr[m]) ==unifyLit.end()){
                            unifyLit.push_back(candClaP->LitPtr[m]);
                        }
                    }
                    if (saveClaId.find(candClaP->uClaId) == saveClaId.end())
                        saveClaId.insert(candClaP->uClaId);
                }
                else
                    triAlg.unify.ClearVarBind(chgVarTemp);
            }
            triAlg.unify.ClearVarBind(chgVarTemp);
        }
        triAlg.unify.ClearVarBind(chgVarTemp);
        vector<Term*>().swap(chgVarTemp);
        unifyLit.clear();
        vector<Literal*>().swap(unifyLit);
        set<Clause*>::iterator claP_it = fol.setCla.begin();
        for (; claP_it != fol.setCla.end();) {
            if(saveClaId.find((*claP_it)->uClaId) == saveClaId.end()){
                set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
                if (delIt != fol.setOriClas.end()) {
                    fol.setOriClas.erase(delIt);
                }
                string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]目标子句约简被删除";
                FileOp::LogOut(out);
                (*claP_it)->DelLits();
                delete((*claP_it));
                claP_it = fol.setCla.erase(claP_it);
            }
            else{
                (*claP_it)->deduceDeepth = 0;
                ++claP_it;
            }
        }
        set<Clause*>::iterator claPE_it = fol.setEnq.begin();
        for (; claPE_it != fol.setEnq.end();) {
            if (saveClaId.find((*claPE_it)->uClaId) == saveClaId.end()) {
                set<Clause*> ::iterator delIt = fol.setOriClas.find(*claPE_it);
                if (delIt != fol.setOriClas.end()) {
                    fol.setOriClas.erase(delIt);
                }
                string out = "#子句编号[" + to_string((*claPE_it)->uClaId) + "]目标子句约简被删除";
                FileOp::LogOut(out);
                (*claPE_it)->DelLits();
                delete((*claPE_it));
                claPE_it = fol.setEnq.erase(claPE_it);
            } else{
                (*claPE_it)->deduceDeepth = 0;
                ++claPE_it;
            }
        }           
    }
    saveClaId.clear();
    if (StrategyParam::preprocess) {
        clock_t endNoPair = clock();
        double time_sNoPair = (double) (endNoPair - StrategyParam::start) / (double) CLOCKS_PER_SEC;
        if (time_sNoPair > StrategyParam::timeLimit * 60) {
            FileOp::outReleaseOf();
            string str = "";
            fol.uOrigClaSingleSize = 0;
            str += fol.ToSingleString();
            FileOp::PrintINFOEndRSinglePre(str);
            str = "";
            str += fol.ToUnUseString();
            FileOp::PrintINFONoUsePre(str);
            FileOp::LogOut("超出时间" + to_string(StrategyParam::timeLimit) + "分钟限制,无法判定！");
            if (!StrategyParam::orForCompition)
                cout << "无法判定" << endl;
            else
                cout << "TMO-Non" << endl;
            return UNKNOWN;
        }
    }
    UINT16 loop = 10;
    bool chooseTri = true;
    bool firstSwitch = true;
    bool readStrategySwitch1 = true;
    bool readStrategySwitch2 = true;
    bool readStrategySwitch3 = true;
    bool readStrategySwitch4 = true;
    UINT32 noPairIt = 0;
    vector<Clause*> saveCurrentClaSet;
    saveCurrentClaSet.reserve(6);
    UINT32 startId = 0;
    UINT32 endId = uNewClaID - 1;
    bool insertNoUseCla = true;
    while(1) {
        endId  = uNewClaID - 1;
        saveCurrentClaSet.clear();
        triAlg.vOnlyOutTri.clear();
        triAlg.vOutTri.clear();
        triAlg.vOnlyOutTriCmp.clear();
        fol.setSingleCla.clear();
        if(FileOp::strategyMode == 1){
            if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < 75) {
                StrategyParam::timeLimit = 5;
            } 
            else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < 130) {
                if(readStrategySwitch1){
                    shift = 0;
                    readStrategySwitch1= false;
                    FileOp::readBatchStrategy(FileOp::StrategyfileName[1]);                  
                    StrategyParam::timeLimit = 5;
                    if(StrategyParam::noUnitRes == false){
                        chooseTri = true;
                    }
                    if(StrategyParam::strategyMode ==0){
                        fol.DelSubSet(fol.uOrigClaSize);
                        FileOp::delRes();
                        uNewClaID = fol.uOrigClaSize + 1;
                        StrategyParam::timeLimit = 5;
                        firstSwitch = true;
                        fol.noPairSingleCla.clear();
                        fol.setDeductCla.clear();
                        fol.eqnDemod.clear();
                        fol.DelSubSetSuper(fol.uOrigClaSize);
                        triAlg.noCmpCla.clear();                       
                        triAlg.flagT = true;
                        startId = 0;
                        endId  = uNewClaID - 1;
                    }
                }
            } else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < 190) {
                 if(readStrategySwitch2){
                    shift = 0;
                    readStrategySwitch2= false;
                    FileOp::readBatchStrategy(FileOp::StrategyfileName[2]);
                    StrategyParam::timeLimit = 5;
                    if(StrategyParam::noUnitRes == false){
                        chooseTri = true;
                    }
                    if(StrategyParam::strategyMode ==0){
                        fol.DelSubSet(fol.uOrigClaSize);
                        FileOp::delRes();
                        uNewClaID = fol.uOrigClaSize + 1;
                        StrategyParam::timeLimit = 5;
                        firstSwitch = true;
                        fol.noPairSingleCla.clear();
                        fol.setDeductCla.clear();
                        fol.eqnDemod.clear();
                        fol.DelSubSetSuper(fol.uOrigClaSize);
                        triAlg.noCmpCla.clear();
                        triAlg.flagT = true;
                        startId = 0;
                        endId  = uNewClaID - 1;
                    }
                }
            } else if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < 250) {
                 if(readStrategySwitch3){
                    shift = 0;
                    readStrategySwitch3= false;
                    FileOp::readBatchStrategy(FileOp::StrategyfileName[3]);
                    StrategyParam::timeLimit = 5;
                    if(StrategyParam::noUnitRes == false){
                        chooseTri = true;
                    }
                    if(StrategyParam::strategyMode ==0){
                        fol.DelSubSet(fol.uOrigClaSize);
                        FileOp::delRes();
                        uNewClaID = fol.uOrigClaSize + 1;
                        StrategyParam::timeLimit = 5;
                        firstSwitch = true;
                        fol.noPairSingleCla.clear();
                        fol.setDeductCla.clear();
                        fol.eqnDemod.clear();
                        fol.DelSubSetSuper(fol.uOrigClaSize);
                        triAlg.noCmpCla.clear();                    
                        triAlg.flagT = true;
                        startId = 0;
                        endId  = uNewClaID - 1;
                    }
                }
            } else{
                 if(readStrategySwitch4){
                    shift = 0;
                    readStrategySwitch4= false;
                    FileOp::readBatchStrategy(FileOp::StrategyfileName[4]);
                    StrategyParam::timeLimit = 5;
                    if(StrategyParam::noUnitRes == false){
                        chooseTri = true;
                    }
                    if(StrategyParam::strategyMode ==0){
                        fol.DelSubSet(fol.uOrigClaSize);
                        FileOp::delRes();
                        uNewClaID = fol.uOrigClaSize + 1;
                        StrategyParam::timeLimit = 5;
                        firstSwitch = true;
                        fol.noPairSingleCla.clear();
                        fol.setDeductCla.clear();
                        fol.eqnDemod.clear();
                        fol.DelSubSetSuper(fol.uOrigClaSize);
                        triAlg.noCmpCla.clear();
                        triAlg.flagT = true;
                        startId = 0;
                        endId  = uNewClaID - 1;
                    }
                }
            }
        }
        if(1){
            set<Clause*>::iterator claP_it = fol.setCla.begin();
            for (; claP_it != fol.setCla.end();) {
                if ((*claP_it)->uClaId >= startId && (*claP_it)->uClaId <= endId) {
                    if (triAlg.claHasPureLit(fol, (*claP_it)) || triAlg.judgeDemodRedundent(fol, (*claP_it), uNewClaID)) {
                        set<Clause*> ::iterator delIt = fol.setOriClas.find(*claP_it);
                        if (delIt != fol.setOriClas.end()) {
                            fol.setOriClas.erase(delIt);
                        }
                        set<Clause*> ::iterator delItNoPair = fol.noPairSingleCla.find(*claP_it);
                        if (delItNoPair != fol.noPairSingleCla.end()) {
                            fol.noPairSingleCla.erase(delItNoPair);
                        }
                        set<Clause*> ::iterator delItEqnDemod = fol.eqnDemod.find(*claP_it);
                        if (delItEqnDemod != fol.eqnDemod.end()) {
                            fol.eqnDemod.erase(delItEqnDemod);
                        }
                        set<Clause*> ::iterator delItGoal = fol.saveGoalClaSet.find(*claP_it);
                        if (delItGoal != fol.saveGoalClaSet.end()) {
                            fol.saveGoalClaSet.erase(delItGoal);
                        }
                        string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]冗余被删除";
                        FileOp::LogOut(out);
                        (*claP_it)->DelLits();
                        delete((*claP_it));
                        claP_it = fol.setCla.erase(claP_it);
                        continue;
                    }
                }
                ++claP_it;
            }  
        }
        preGenCla.clear();
        if(StrategyParam::useSaturate){
            RESULT res = fol.chooseCurrentCLaSet(&saveCurrentClaSet, 6,triAlg,startId,endId,uNewClaID);
            startId = endId;
            if(res == UNSAT)
                return UNSAT;
            set<Clause*>::iterator claP_it = fol.setCla.begin();
            for (; claP_it != fol.setCla.end();) {
                if (find(saveCurrentClaSet.begin(), saveCurrentClaSet.end(), (*claP_it)) != saveCurrentClaSet.end()) {
                    string out = "#子句编号[" + to_string((*claP_it)->uClaId) + "]加入到演绎集合中";
                    FileOp::LogOut(out);
                    claP_it = fol.setCla.erase(claP_it);
                } else {
                    ++claP_it;
                }
            }
            set<Clause*>::iterator claPE_it = fol.setEnq.begin();
            for (; claPE_it != fol.setEnq.end();) {
                if (find(saveCurrentClaSet.begin(), saveCurrentClaSet.end(), (*claPE_it)) != saveCurrentClaSet.end()) {
                    string out = "#子句编号[" + to_string((*claPE_it)->uClaId) + "]加入到演绎集合中";
                    FileOp::LogOut(out);
                    claPE_it = fol.setEnq.erase(claPE_it);
                } else {
                    ++claPE_it;
                }
            }
            for(UINT32 i=0; i<saveCurrentClaSet.size(); ++i){
                if (StrategyParam::termRewriting == 2) {
                    fol.judgeInsertSuper(saveCurrentClaSet[i]);
                    fol.setDeductCla.insert(saveCurrentClaSet[i]);
                } else {
                    fol.judgeInsertSuper(saveCurrentClaSet[i]);
                    if (fol.eqnDemod.find(saveCurrentClaSet[i]) != fol.eqnDemod.end()) {
                        continue;
                    }
                    int res = demodulatorClaSaturate(fol, triAlg, saveCurrentClaSet[i], uNewClaID, &preGenCla);
                    if ((res == 1) || (res == 2)) {
                        swap(saveCurrentClaSet[i], saveCurrentClaSet[saveCurrentClaSet.size() - 1]);
                        saveCurrentClaSet.pop_back();
                        --i;
                    } else{
                        fol.setDeductCla.insert(saveCurrentClaSet[i]);
                    }
                }
            }
        }
        if (preGenCla.size() > 0) {
            for (int i = 0; i < preGenCla.size(); ++i) {
                fol.BackwardSubsumpSuper(*preGenCla[i],saveCurrentClaSet);
            }
        }
        preGenCla.clear();
        if(StrategyParam::factorRes) {
            for (UINT32 i = 0; i < saveCurrentClaSet.size(); ++i) {
                bool delFlag = true;
                factorRes(fol, triAlg, saveCurrentClaSet[i], uNewClaID, &preGenCla, delFlag, false);
                if (delFlag) {
                    set<Clause*> ::iterator delItDeduct = fol.setDeductCla.find(saveCurrentClaSet[i]);
                    if (delItDeduct != fol.setDeductCla.end()) {
                        fol.setDeductCla.erase(delItDeduct);
                    }
                    set<Clause*> ::iterator delItNoPair = fol.noPairSingleCla.find(saveCurrentClaSet[i]);
                    if (delItNoPair != fol.noPairSingleCla.end()) {
                        fol.noPairSingleCla.erase(delItNoPair);
                    }
                    set<Clause*> ::iterator delItEqnDemod = fol.eqnDemod.find(saveCurrentClaSet[i]);
                    if (delItEqnDemod != fol.eqnDemod.end()) {
                        fol.eqnDemod.erase(delItEqnDemod);
                    }
                    set<Clause*> ::iterator delItGoal = fol.saveGoalClaSet.find(saveCurrentClaSet[i]);
                    if (delItGoal != fol.saveGoalClaSet.end()) {
                        fol.saveGoalClaSet.erase(delItGoal);
                    }
                    string out = "#子句编号[" + to_string((saveCurrentClaSet[i])->uClaId) + "]被因子归结删除";
                    FileOp::LogOut(out);
                    saveCurrentClaSet[i]->DelLits();
                    delete(saveCurrentClaSet[i]);
                    swap(saveCurrentClaSet[i], saveCurrentClaSet[saveCurrentClaSet.size() - 1]);
                    saveCurrentClaSet.pop_back();
                    --i;
                }
            }
        }
        if (preGenCla.size() > 0) {
            for (int i = 0; i < preGenCla.size(); ++i) {
                fol.BackwardSubsumpSuper(*preGenCla[i],saveCurrentClaSet);
            }
        }
        preGenCla.clear();
        if (StrategyParam::eqnRes) {
            for (UINT32 i = 0; i < saveCurrentClaSet.size(); ++i) {
                if (saveCurrentClaSet[i]->hasPLit()) {
                    negEqnRes(fol, triAlg, saveCurrentClaSet[i], uNewClaID, &preGenCla, false);
                }
                else 
                    continue;
            }
        }
       if (preGenCla.size() > 0) {
            for (int i = 0; i < preGenCla.size(); ++i) {
                fol.BackwardSubsumpSuper(*preGenCla[i],saveCurrentClaSet); 
            }
        }
        preGenCla.clear();       
        if (firstSwitch){
            fol.resetSingleClaNumSaturate();
        }
        InsertSingleSetSaturate(fol,triAlg);
        if ((fol.setOriClas.size() < 2) && (fol.setCla.size() < 2) && (fol.setDeductCla.size() < 2)) {	
            FileOp::LogOut("得出结论:可满足！");
            cout<< "得出结论:可满足"<<endl;
            return SAT;
        }
        if(fol.setSingleCla.size() == 0){
            StrategyParam::noUnitRes = true;
        }
        if (fol.setSingleCla.size() > 0) {
            if(!StrategyParam::deductMode)
                InsertOutTri(triAlg, fol, loop, chooseTri); 
            else{
                InsertOutTriLR(triAlg, fol, loop, chooseTri);
            }
            givenClaP = triAlg.vOnlyOutTri[0]->claPtr;
            StartId = triAlg.vOnlyOutTri[0]->claPtr->uClaId;
            triAlg.startCluId = StartId;
            vector<Term*> chgVarT;
            chgVarT.reserve(6);
            UINT32 lens = triAlg.vOnlyOutTri.size();
            for (UINT32 i = 0; i < saveCurrentClaSet.size(); ++i) {
                if(saveCurrentClaSet[i]->uLitNum > 1)
                    continue;
                Literal* lit = saveCurrentClaSet[i]->LitPtr[0];
                for (UINT32 j = 0; j < lens; j++) {
                    if(lit == triAlg.vOnlyOutTri[j])
                        continue;
                    if (lit->isPairPred(triAlg.vOnlyOutTri[j]) && triAlg.unify.LitMgu(lit, triAlg.vOnlyOutTri[j], chgVarT)) {
                        string str;
                        FileOp::CreateRunInfoFile(FileOp::runFileId);
                        *FileOp::ofRealInfoOut << "从编号为" + to_string(saveCurrentClaSet[i]->uClaId) + "子句开始构建三角形" << endl;
                        triAlg.unify.WriteTriPath(lit);                     
                        triAlg.unify.WriteTriPath(triAlg.vOnlyOutTri[j]);
                        *FileOp::ofRealInfoOut << "R:空子句" << endl;
                        FileOp::LogOut("# ==子句" + to_string(saveCurrentClaSet[i]->uClaId) + "起步，结论正确(UNSAT)！");
                        if(!StrategyParam::orForCompition && (StrategyParam::problemDiv == 0))
                            cout << "得出结论:不可满足" << endl;
                        triAlg.unify.ClearVarBind(chgVarT);
                        return UNSAT;
                    }
                    triAlg.unify.ClearVarBind(chgVarT);
                }
            }
            triAlg.unify.ClearVarBind(chgVarT);
            chgVarT.clear();
            for (UINT32 ind = 0; ind< saveCurrentClaSet.size(); ++ind)
            {
                Clause* aClap = saveCurrentClaSet[ind];
                if ((aClap->uLitNum == 1)) {
                    continue;
                }
                vector<Term*> varBindT; 
                varBindT.reserve(6);
                UINT16 pairCount = 0;
                vector<Literal*> isOutPut;
                isOutPut.reserve(6);
                if (StrategyParam::noUnitRes) {
                    for (UINT16 ActLitInd = 0; ActLitInd < aClap->uLitNum; ++ActLitInd) {
                        Literal* QLit = aClap->LitPtr[ActLitInd];
                        for (UINT32 i = 0; i < lens; ++i) {
                            if (QLit->isPairPred(triAlg.vOnlyOutTri[i]) && triAlg.unify.LitMgu(QLit, triAlg.vOnlyOutTri[i], chgVarT)) {
                                aClap->clast = PiredSingle; 
                                varBindT.insert(varBindT.end(), chgVarT.begin(), chgVarT.end());
                                chgVarT.clear();
                                pairCount++;
                                isOutPut.push_back(triAlg.vOnlyOutTri[i]); 
                                break;
                            } else
                                triAlg.unify.ClearVarBind(chgVarT);
                        }
                    }
                    if (pairCount == aClap->uLitNum)
                    {
                        FileOp::CreateRunInfoFile(FileOp::runFileId);
                        *FileOp::ofRealInfoOut << "从编号为" + to_string(aClap->uClaId) + "子句开始构建三角形" << endl;
                        for (UINT16 ActLitInd = 0; ActLitInd < aClap->uLitNum; ++ActLitInd) {
                            isOutPut.push_back(aClap->LitPtr[ActLitInd]);
                        }
                        for (UINT32 k = 0; k < isOutPut.size(); k++) {
                            triAlg.unify.WriteTriPath(isOutPut[k]);
                        }
                        *FileOp::ofRealInfoOut << "R:空子句" << endl;
                        FileOp::LogOut("# ==子句" + to_string(aClap->uClaId) + "起步，结论正确(UNSAT)！");
                        if(!StrategyParam::orForCompition && (StrategyParam::problemDiv == 0))
                            cout<<"得出结论:不可满足" << endl;
                        return UNSAT;
                    } else if (pairCount == 0) {
                        aClap->clast = noPiredSingle;
                        if (fol.noPairSingleCla.find(aClap) == fol.noPairSingleCla.end()) {
                            fol.noPairSingleCla.insert(aClap);
                        }
                    }
                    triAlg.unify.ClearVarBind(varBindT);
                    triAlg.unify.ClearVarBind(chgVarT);
                }
                vector<Term*>().swap(varBindT); 
                vector<Literal*>().swap(isOutPut);
            }
            triAlg.unify.ClearVarBind(chgVarT);
            chgVarT.clear();
            int singleInClaSet = 0;
            vector<Clause*> simplfyCla; 
            simplfyCla.reserve(6);
            vector<Clause*> simplfyClaDel; 
            simplfyClaDel.reserve(6);
            triAlg.unify.ClearVarBind(chgVarT);
            if (StrategyParam::noUnitSimplify && (triTotalTimes %16 == 0)) {
                for (UINT32 ind = 0; ind < saveCurrentClaSet.size(); ++ind)
                {
                    Clause* aClap = saveCurrentClaSet[ind];
                    if ((aClap->uLitNum == 1))
                        continue;
                    vector<Term*> varBindT; 
                    varBindT.reserve(6);
                    UINT8 pairCount = 0; 
                    vector<Literal*> isOutPut; 
                    isOutPut.reserve(6);
                    bool noShareVar = false;
                    isOutPut.clear();
                    chgVarT.clear();
                    if (aClap->hasConLit()) {
                        bool simplyOut = false;
                        vector<Literal*> vSimplyNewR;
                        vSimplyNewR.reserve(6);
                        for (UINT16 ActLitInd = 0; ActLitInd < aClap->uLitNum; ++ActLitInd) {
                            Literal* QLit = aClap->LitPtr[ActLitInd];
                            QLit->isDel = 0;
                            if (QLit->subTerm->IsShareVarLit())
                                continue;
                            else {
                                for (UINT32 i = 0; i < lens; ++i) {
                                    if (QLit->isPairPred(triAlg.vOnlyOutTri[i]) && triAlg.unify.LitMgu(QLit, triAlg.vOnlyOutTri[i], chgVarT)) {
                                        QLit->isDel = 1;
                                        varBindT.insert(varBindT.end(), chgVarT.begin(), chgVarT.end());
                                        chgVarT.clear();
                                        isOutPut.push_back(triAlg.vOnlyOutTri[i]);
                                        simplyOut = true;
                                        break;
                                    } else
                                        triAlg.unify.ClearVarBind(chgVarT);
                                }
                            }
                        }
                        if (simplyOut) {
                            for (UINT16 ActLitInd = 0; ActLitInd < aClap->uLitNum; ++ActLitInd) {
                                Literal* QLit = aClap->LitPtr[ActLitInd];
                                isOutPut.push_back(QLit);
                                if (!QLit->isDel)
                                    vSimplyNewR.push_back(QLit);
                            }
                            FileOp::CreateRunInfoFile(FileOp::runFileId);
                            *FileOp::ofRealInfoOut << "从编号为" + to_string(aClap->uClaId) + "子句开始构建三角形" << endl;
                            for (UINT32 k = 0; k < isOutPut.size(); k++) {
                                triAlg.unify.WriteTriPath(isOutPut[k]);
                            }
                            triAlg.newCla = nullptr;
                            triAlg.newCla = new Clause(uNewClaID, 0);
                            triAlg.newCla->isAxtom = true;
                            size_t uRsize = vSimplyNewR.size();
                            if (uRsize > 1)
                                stable_sort(vSimplyNewR.begin(), vSimplyNewR.end(), SortRule::LitCmp);
                            triAlg.newCla->LitPtr = new Literal*[uRsize];
                            for (UINT16 i = 0; i < vSimplyNewR.size(); ++i) {
                                Term* litSubT = triAlg.unify.GetNewTerm(vSimplyNewR[i]->subTerm, triAlg.newCla); 
                                if (TNegEqn == GetCodeProp(litSubT->iTermCode)
                                        && litSubT->subTermPtr[0] == litSubT->subTermPtr[1]) {
                                    --uRsize;
                                    continue;
                                }
                                if (triAlg.newCla->uFuncDepth < litSubT->uLevel)
                                    triAlg.newCla->uFuncDepth = litSubT->uLevel;
                                triAlg.newCla->uVarCount += litSubT->uVarCount;		
                                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum] = new Literal(litSubT, EncodeLitCode(uNewClaID, triAlg.newCla->uLitNum + 1), triAlg.newCla, nullptr);
                                triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->parentLitPtr = vSimplyNewR[i]; 
                                if (triAlg.newCla ->deduceDeepth < (vSimplyNewR[i]->claPtr->deduceDeepth + 1)) {
                                    triAlg.newCla ->deduceDeepth = vSimplyNewR[i]->claPtr->deduceDeepth + 1;
                                }
                                if (vSimplyNewR[i]->claPtr->isAxtom && triAlg.newCla->isAxtom)
                                    triAlg.newCla->isAxtom = true;
                                else
                                    triAlg.newCla->isAxtom = false;
                                 if (triAlg.hasUnifGoalLit(fol, triAlg.newCla->LitPtr[triAlg.newCla->uLitNum])) {
                                    triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = 1;
                                    if (triAlg.newCla->goaldeepth > 1)
                                        triAlg.newCla->goaldeepth = 1;
                                } else {
                                    triAlg.newCla->LitPtr[triAlg.newCla->uLitNum]->goalDeepth = vSimplyNewR[i]->goalDeepth;
                                    if (triAlg.newCla->goaldeepth > vSimplyNewR[i]->goalDeepth)
                                        triAlg.newCla->goaldeepth = vSimplyNewR[i]->goalDeepth;
                                }
                                ++(triAlg.newCla->uLitNum);
                            }
                            fol.initGenerateClaAttr(triAlg.newCla);
                            uNewClaID++;
                            Clause* newCla = triAlg.newCla;
                            if (StrategyParam::termRewriting != 2) {
                                newCla->directEqn();
                            }
                            FileOp::WriteR(newCla);
                            if (StrategyParam::orForCompition) {
                                newCla->addRForCompetiton();
                                
                                string rStr = "";
                                newCla->toString(rStr);
                                string outClauses = "";
                                for (UINT8 k = 0; k < isOutPut.size(); ++k) {
                                    if(find(validPathOut[uNewClaID-1].begin(),validPathOut[uNewClaID-1].end(),isOutPut[k]-> claPtr->uClaId) == validPathOut[uNewClaID-1].end()){
                                        validPathOut[uNewClaID-1].push_back(isOutPut[k]-> claPtr->uClaId);
                                        outClauses += to_string(isOutPut[k]-> claPtr->uClaId) + ",";
                                    }
                                }
                                if(find(validPathOut[uNewClaID-1].begin(),validPathOut[uNewClaID-1].end(),aClap->uClaId) == validPathOut[uNewClaID-1].end()){
                                    validPathOut[uNewClaID-1].push_back(aClap->uClaId);
                                    outClauses += to_string(aClap->uClaId);
                                }
                                if(outClauses[outClauses.size() -1] == ','){
                                    outClauses.erase(outClauses.size() -1, 1);
                                }
                                int begPos = rStr.find_first_of("[") + 1;
                                int endPos = rStr.find_first_of("]");
                                string rId = rStr.substr(begPos, endPos - begPos);
                                string rContext = rStr.substr(rStr.find_first_of("]") + 1);
                                string out = "cnf(" + rId + ",plain,\n" + "   ("+ rContext+"),\n" + "   inference(scs_inference,[]," + "[" + outClauses +"]"+ ")).";
                                *FileOp::ofOutput << out << endl;
                            }                          
                            string vaildInf = "|[Y]";
                            FileOp::RInvaildInfoOut(vaildInf, StartId);
                            ++triTotalTimes;
                            simplfyCla.push_back(newCla); 
                            simplfyClaDel.push_back(aClap);
                            if (newCla->uLitNum == 1)
                                if (StrategyParam::R_Weight == -1)
                                    newCla->adeductNum = 0; 
                                else
                                    newCla->adeductNum = StrategyParam::R_Weight;
                            else
                                if (StrategyParam::R_Weight == -1)
                                newCla->pdeductNum = 0; 
                            else
                                newCla->pdeductNum = StrategyParam::R_Weight;
                        }
                        vSimplyNewR.clear();
                        vector<Literal*>().swap(vSimplyNewR);
                    }
                    triAlg.unify.ClearVarBind(varBindT);
                    vector<Term*>().swap(varBindT);
                    isOutPut.clear();
                    vector<Literal*>().swap(isOutPut);
                    triAlg.unify.ClearVarBind(chgVarT);
                    chgVarT.clear();
                }
                if (!simplfyCla.empty()) {
                    for (int m = 0; m < simplfyCla.size(); ++m) {
                        fol.BackwardSubsumpDirectSaturate(*(simplfyCla[m]), *(simplfyClaDel[m]),saveCurrentClaSet);
                        fol.InsertCla(simplfyCla[m]);
                        fol.setCla.insert(simplfyCla[m]);
                        if (StrategyParam::termRewriting != 2)
                            fol.judgeInsertSuper(simplfyCla[m]);
                    }
                    simplfyCla.clear();
                    vector<Clause*>().swap(simplfyCla);
                    simplfyClaDel.clear();
                    vector<Clause*>().swap(simplfyClaDel);
                }
            }
            triAlg.unify.ClearVarBind(chgVarT);
            chgVarT.clear();
            vector<Term*>().swap(chgVarT);
        }
        else{
            chooseTri = false;
            givenClaP = nullptr;
            if(fol.setSingleCla.size() == 0){
                for (auto& aClap : fol.setDeductCla){
                    if(aClap->uLitNum == 1){
                        continue;
                    }
                    if(!aClap->isUse){
                        insertNoPairCla(fol, aClap);
                    }
                }
            }
        }
        chgCla.clear();    
        triAlg.allNoPair = false;   
        firstSwitch = false;
        UINT8 resolutionRes = 0;
        if(chooseTri)
        {
            if(!StrategyParam::deductMode){
                FileOp::CreateRunInfoFile(FileOp::runFileId);
                resolutionRes = triAlg.GenerateTriByRecodePath(fol,&*givenClaP, uNewClaID, backType,chgCla,loop);
            }
            else{
                FileOp::CreateRunInfoFile(FileOp::runFileId);
                vector<Clause*> candidateClaSet;
                candidateClaSet.reserve(16);
                for (auto& aClap : fol.setCla) {
                    if(aClap->uLitNum != 1){
                        candidateClaSet.push_back(aClap);
                    }
                }
                for (auto& aClap : fol.setEnq){
                    if(aClap->uLitNum != 1){
                        candidateClaSet.push_back(aClap);
                    }
                }
               if((StrategyParam::unitSortRule == 0) || (StrategyParam::unitSortRule == 1)){
                   stable_sort(candidateClaSet.begin(), candidateClaSet.end(), SortRule::PoslitCmpProRL);
                }
                else
                    stable_sort(candidateClaSet.begin(), candidateClaSet.end(), SortRule::PoslitCmpPriorLR);
                if(StrategyParam::cseRunMode == 0){
                    resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                }
                else if(StrategyParam::cseRunMode == 1){
                    resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                }
                else if(StrategyParam::cseRunMode == 2){
                    if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::distriTime * 12) {
                        resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol, candidateClaSet, uNewClaID, backType, chgCla, loop);
                    }
                    else
                        resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                }
                else if((StrategyParam::cseRunMode == 3)){
                    if (((double) (clock() - StrategyParam::start) / (double) CLOCKS_PER_SEC) < StrategyParam::distriTime * 12) {
                        resolutionRes = triAlg.GenerateTriByRecodePathLR(fol,candidateClaSet, uNewClaID, backType,chgCla,loop);
                    }
                    else
                        resolutionRes = triAlg.GenerateTriByRecodePathLROptimalProofSearch(fol, candidateClaSet, uNewClaID, backType, chgCla, loop);
                }                              
                candidateClaSet.clear();
                vector<Clause*>().swap(candidateClaSet);
            }
            ++triTotalTimes;
            if(fol.noPairSingleCla.size()>0 &&StrategyParam::noUnitRes &&(triTotalTimes%triAlgTimes == 0))
                chooseTri = false;
            } else if (StrategyParam::noUnitRes && !chooseTri && (fol.noPairSingleCla.size() > 0)) {
                givenClaP = nullptr;
                vector<Clause*> claNoPairVec;
                claNoPairVec.reserve(8);
                if ((fol.noPairSingleCla.size() > 0) || insertNoUseCla) {
                    clock_t endPair = clock();
                    double time_sPair = (double) (endPair - StrategyParam::start) / (double) CLOCKS_PER_SEC;
                    if (insertNoUseCla) {
                        for (auto& claP : fol.setDeductCla) {
                            insertNoPairCla(fol, claP);
                        }
                        insertNoUseCla = false;
                    }
                    for (auto& claP : fol.noPairSingleCla) {
                        if (claP->uLitNum == 1) {
                            continue;
                        }
                        claNoPairVec.push_back(claP);
                    }
                    if ((fol.noPairSingleCla.size() == 0) || (claNoPairVec.size() == 0)) {
                        chooseTri = true;
                        continue;
                    }
                    if (claNoPairVec.size() != 0) {
                        stable_sort(claNoPairVec.begin(), claNoPairVec.end(), SortRule::noPairClaCmp);
                    }
                    if (claNoPairVec.size() >= noPairInd + 1) {
                        for (UINT32 m = 0; m <= noPairInd; ++m) {
                            if (claNoPairVec[m]->noPairResFiFg == false) {
                                swap(claNoPairVec[m], claNoPairVec[noPairInd]);
                                break;
                            }
                        }
                    }
                    if (noPairInd >= claNoPairVec.size())
                        noPairInd = 0;
                    givenClaP = claNoPairVec[noPairInd];
                    UINT32 countSize = 0;
                    while (givenClaP->uLitNum >= litSize) {
                        ++noPairInd;
                        if (noPairInd >= claNoPairVec.size())
                            noPairInd = 0;
                        givenClaP = claNoPairVec[noPairInd];
                        ++countSize;
                        if (countSize == claNoPairVec.size())
                            break;
                    }
                    noPairCla = givenClaP;
                }
                if (givenClaP) {
                    triAlg.vOutTri.clear();
                    FileOp::CreateRunInfoFile(FileOp::runFileId);
                    resolutionRes = triAlg.GenerateTriByRecodePathNoUnit(fol, &*givenClaP, uNewClaID, backType, chgCla); 
                    ++triTotalTimes;
                } else {
                    FileOp::delCreateRunInfoFile(FileOp::runFileId);
                    continue;
                }
                chooseTri = true;
            } else if (fol.noPairSingleCla.size() == 0) {
                chooseTri = true;
                continue;
            }
            if (triAlg.allNoPair) {
                if (resolutionRes == 33) {
                    FileOp::delCreateRunInfoFile(FileOp::runFileId);
                    for (auto& claP : fol.noPairSingleCla) {
                        claP->noUnitDeductTimes = 0;
                    }
                    givenClaP->noPairResFiFg = true;
                    ++noPairInd;
                } else if (resolutionRes == 22) {
                    FileOp::delCreateRunInfoFile(FileOp::runFileId);
                    triAlgTimes = 1;
                    if (fol.noPairSingleCla.size() == 0) {
                        fol.initNoPairCla();
                    }
                    StrategyParam::noUnitRes = true;
                    if (fol.setDeductCla.size() >= fol.setCla.size()) {
                        ++StrategyParam::R_MAX_FUNCLAYER;
                        if (StrategyParam::R_MAX_FUNCLAYER == (StrategyParam::R_INIT_MAX_FUNCLAYER * 2 + 1)) {
                            StrategyParam::R_MAX_FUNCLAYER = StrategyParam::R_MAX_FUNCLAYER_TMP;
                        }
                        ++StrategyParam::R_MAX_NUM;
                        if (StrategyParam::R_MAX_NUM == StrategyParam::R_INIT_MAX_NUM) {
                            StrategyParam::R_MAX_NUM = StrategyParam::R_MAX_NUM_TMP;
                        }
                        loop = 1;
                    }

                } else if (resolutionRes == 211) {
                    loop = 0;
                }
            long leftMem = FileOp::getMem();
            if (leftMem < 200000) {
                FileOp::LogOut("内存超出限制,无法判定！剩余内存:" + to_string(leftMem));
                string fileMemOut = FileOp::g_OutputPath + "/"  + "memleft.log";
                string cmdMem = "ps -aux | sort -k4nr | head -10 >> " + fileMemOut;
                int res = system(const_cast<char*> (cmdMem.c_str()));
                if (!StrategyParam::orForCompition)
                    cout << "无法判定,内存超出限制" << endl;
                else
                    cout << "TMO-Non" << endl;
                return UNKNOWN;
            }
            clock_t endNoPair = clock();
            double time_sNoPair = (double) (endNoPair - StrategyParam::start) / (double) CLOCKS_PER_SEC;
            double breakTimeA = 0.0;
            if((FileOp::runMode != 0 )&&(FileOp::runMode != 1)){
                breakTimeA = StrategyParam::timeForceLimit - 10;
            }
            else if((FileOp::runMode == 0 )||(FileOp::runMode == 1)){
                breakTimeA = StrategyParam::timeForceLimit - 10;
            }
            StrategyParam::timeLimit = StrategyParam::firstTimeLimit;
            if (time_sNoPair > StrategyParam::timeLimit *60)
            {
                triAlg.delSaveCopyCla(fol);
                FileOp::outReleaseOf();
                string str = "";
                if (!StrategyParam::orForCompition && (FileOp::runMode != 0)&&(FileOp::runMode != 1)){
                    str = "";
                    fol.uOrigClaSingleSize = 0;
                    str += fol.ToSingleString();
                    FileOp::PrintINFOEndRSinglePre(str); 
                }                
                if(!StrategyParam::orForCompition){
                    str = "";
                    str += fol.ToUnUseString();
                    FileOp::PrintINFONoUsePre(str); 
                    FileOp::LogOut("超出时间" + to_string(StrategyParam::timeLimit) +"分钟限制,无法判定！");
                }
                
                if ((!StrategyParam::orForCompition) && (FileOp::strategyMode != 3))
                    cout << "无法判定" << endl;
                else
                    cout << "TMO-Non" << endl;
                return UNKNOWN;
            }
            if (StrategyParam::genLitClause && (loop != 0) && (triAlg.total_Loop / loop < 2)) {
                --loop;
            }
            else if (((fol.noPairSingleCla.size()>0) &&StrategyParam::noUnitRes &&!chooseTri )||(chooseTri &&(!StrategyParam::noUnitRes || (fol.noPairSingleCla.size() == 0)) )) {
                loop = 0;
            }
            continue;
        }
        Clause* newCla = triAlg.newCla;
        bool rFilter = false;
        if((chgCla.size() == 0) || (triAlg.vNewR.empty() || triAlg.newCla->uLitNum == 0)){
            rFilter = false;
        }
        else{
            UINT32 midLens = chgCla.size();
            if(chgCla[midLens-1]->uLitNum != newCla->uLitNum){
                rFilter = false;
            }
            else{
                UINT32 countRSize = 0;
                for(UINT32 pos=0; pos < newCla->uLitNum; ++pos){
                    Literal* litR = newCla->LitPtr[pos];
                    Literal* midR = chgCla[midLens-1]->LitPtr[pos];
                    if(litR->isEqPred(midR) && triAlg.unify.CheckEqnR(litR->subTerm,midR->subTerm)){
                        ++countRSize;
                        continue;
                    }
                    else
                        rFilter = false;
                }
                if(countRSize == newCla->uLitNum)
                    rFilter = true;
            }
        }
        if(!rFilter) {
            *FileOp::ofRealInfoOut << "从编号为" + to_string(givenClaP->uClaId) + "子句开始构建三角形" << endl;	
            if (resolutionRes == 44)
                triAlg.PrintTriTwoPath();
            else{
                triAlgTimes = 16;
                UINT32 realStartId = triAlg.PrintTriPath();
                if(realStartId != givenClaP->uClaId){
                    loop = 1;
                }
            }
            if (triAlg.vNewR.empty() || triAlg.newCla->uLitNum == 0) {
                FileOp::WriteR(nullptr);
                FileOp::LogOut("# ==子句" + to_string(givenClaP->uClaId) + "起步，结论正确(UNSAT)！");
                preGenCla.clear();
                vector<Clause*>().swap(preGenCla);
                if (StrategyParam::orForCompition) {
                    string outClauses = triAlg.PrintCompetitionOut(0);
                    string out = "cnf($" + to_string(uNewClaID) + ",plain,\n" + "   ($false),\n" +"   inference(scs_inference,[]," + "[" + outClauses +"]" + "),\n" + "   ['proof']" + ").";
                   
                    *FileOp::ofOutput << out << endl;
                }
                return UNSAT;
            }
            if (StrategyParam::termRewriting != 2){
                    newCla->directEqn();
            }
            FileOp::WriteR(newCla);
            triAlg.BackPathClear();
            UINT16 rLen = newCla->uLitNum;
            vector<UINT32>invaildLitId; 
            string rundMsg; 
            Subsumption subsump; 
            bool isRundacyR = subsump.GenCla(fol, newCla, invaildLitId, rundMsg);
            bool RFactorFlag = false;
            FileOp::RInvaildInfoOut(rundMsg, StartId);
            if (isRundacyR) {
                newCla->DelLits(false);
                delete(newCla);
            } else {
                if (StrategyParam::orForCompition) {
                    newCla->addRForCompetiton();                   
                    string rStr = "";
                    newCla->toString(rStr);
                    string outClauses = triAlg.PrintCompetitionOut(uNewClaID - 1);
                    int begPos = rStr.find_first_of("[") + 1;
                    int endPos = rStr.find_first_of("]");
                    string rId = rStr.substr(begPos, endPos - begPos);
                    string rContext = rStr.substr(rStr.find_first_of("]") + 1);
                    string out = "cnf(" + rId + ",plain,\n" + "   ("+ rContext+"),\n";
                    out += "   inference(csc_inference,[],[" + outClauses +"]" + ")).";
                    *FileOp::ofOutput << out << endl;
                }
                
                bool resFlag = true;
                if (resFlag) {
                    fol.BackwardSubsumpClaSet(*newCla,chgCla,false); 
                    if (newCla->uLitNum == 1)
                        if (StrategyParam::R_Weight == -1)
                            newCla->adeductNum = 0; 
                        else
                            newCla->adeductNum = StrategyParam::R_Weight;
                    else
                        if (StrategyParam::R_Weight == -1)
                        newCla->pdeductNum = 0; 
                    else
                        newCla->pdeductNum = StrategyParam::R_Weight; 
                    fol.InsertCla(newCla);
                    fol.setCla.insert(newCla);
                    if (StrategyParam::termRewriting != 2)
                        fol.judgeInsertSuper(newCla);
                    if (1) {
                        insertNoPairCla(fol, newCla);
                    }
                }
            }
        }
        else {
            newCla->DelLits(false);
            delete(newCla);
            --uNewClaID;
        }
        for(int m=0;m<chgCla.size();m++)
        {
            if(0) {
                vector<UINT32> invaildLitIdChg; 
                string rundMsgChg; 
                Subsumption subsumpChg; 
                bool isRundacyRChg = subsumpChg.GenCla(fol,chgCla[m], invaildLitIdChg, rundMsgChg);
                if (isRundacyRChg) {
                    chgCla[m]->DelLits(false);
                    delete(chgCla[m]);
                    continue;
                }
            }
            if (1) {
                if (!StrategyParam::divideTri) {
                    *FileOp::ofRealInfoOut << "R[" + to_string(chgCla[m]->uClaId) + "]" << "[" + to_string(chgCla[m]->LitPtr[0]->ParentLitRow()) + "_" + to_string(chgCla[m]->LitPtr[0]->ParentLitCol()) + "]" << chgCla[m]->LitPtr[0]->subTerm->ToString();
                    *FileOp::ofRealInfoOut << "|[Y]" << endl;
                }
                bool resFlag = true;
                if(resFlag) 
                {
                    fol.BackwardSubsumpClaSet(*chgCla[m],chgCla,true); 
                    if (chgCla[m]->uLitNum == 1) {
                        if (StrategyParam::R_Weight == -1)
                            chgCla[m]->adeductNum = 0;
                        else
                            chgCla[m]->adeductNum = StrategyParam::R_Weight;
                    } else {
                        if (StrategyParam::R_Weight == -1)
                            chgCla[m]->pdeductNum = 0;
                        else
                            chgCla[m]->pdeductNum = StrategyParam::R_Weight;
                    }
                    if (StrategyParam::termRewriting != 2)
                        fol.judgeInsertSuper(chgCla[m]);
                    if (1){
                        insertNoPairCla(fol, chgCla[m]);
                    }
                }
                
                continue;
            }
        }
        long leftMem = FileOp::getMem();
        if(leftMem < 200000)
        {
            FileOp::LogOut("内存超出限制,无法判定！剩余内存:" + to_string(leftMem));
            string fileMemOut = FileOp::g_OutputPath + "/" + "memleft.log";
            string cmdMem = "ps -aux | sort -k4nr | head -10 >> " + fileMemOut;
            int res = system(const_cast<char*> (cmdMem.c_str()));
            if (!StrategyParam::orForCompition)
                cout << "无法判定,内存超出限制" << endl;
            else
                cout << "TMO-Non" << endl;
            return UNKNOWN;
        }
        clock_t end = clock();
        double time_s = (double)(end - StrategyParam::start)/(double)CLOCKS_PER_SEC;       
        double breakTimeB = 0.0;
        if ((FileOp::runMode != 0)&&(FileOp::runMode != 1)) {
            breakTimeB = StrategyParam::timeForceLimit - 10;
        } else if ((FileOp::runMode == 0) || (FileOp::runMode == 1)) {
            breakTimeB = StrategyParam::timeForceLimit - 10;
        }
        StrategyParam::timeLimit = StrategyParam::firstTimeLimit;
        if(time_s  > StrategyParam::timeLimit *60)
        {
            triAlg.delSaveCopyCla(fol);
            FileOp::outReleaseOf();
            string str = "";
            if (!StrategyParam::orForCompition && (FileOp::runMode != 0)&&(FileOp::runMode != 1)){
                str = "";
                fol.uOrigClaSingleSize = 0;
                str += fol.ToSingleString();
                FileOp::PrintINFOEndRSinglePre(str); 
            }
            
            if(!StrategyParam::orForCompition){
                str = "";
                str += fol.ToUnUseString();
                FileOp::PrintINFONoUsePre(str);
                FileOp::LogOut("超出时间"+ to_string(StrategyParam::timeLimit) +"分钟限制,无法判定！");
            }      
            if ((!StrategyParam::orForCompition) && (FileOp::strategyMode != 3))
                cout << "无法判定" << endl;
            else
                cout << "TMO-Non" << endl;
            preGenCla.clear();
            vector<Clause*>().swap(preGenCla);
            return UNKNOWN;
        }
    }
    return UNKNOWN;
}
